searchNodes=[{"doc":"LiveView provides rich, real-time user experiences with server-rendered HTML. The LiveView programming model is declarative: instead of saying &quot;once event X happens, change Y on the page&quot;, events in LiveView are regular messages which may cause changes to its state. Once the state changes, LiveView will re-render the relevant parts of its HTML template and push it to the browser, which updates itself in the most efficient manner. This means developers write LiveView templates as any other server-rendered HTML and LiveView does the hard work of tracking changes and sending the relevant diffs to the browser. At the end of the day, a LiveView is nothing more than a process that receives events as messages and updates its state. The state itself is nothing more than functional and immutable Elixir data structures. The events are either internal application messages (usually emitted by Phoenix.PubSub ) or sent by the client/browser. LiveView is first rendered statically as part of regular HTTP requests, which provides quick times for &quot;First Meaningful Paint&quot;, in addition to helping search and indexing engines. Then a persistent connection is established between client and server. This allows LiveView applications to react faster to user events as there is less work to be done and less data to be sent compared to stateless requests that have to authenticate, decode, load, and encode data on every request. The flipside is that LiveView uses more memory on the server compared to stateless requests. Use cases There are many use cases where LiveView is an excellent fit right now: Handling of user interaction and inputs, buttons, and forms - such as input validation, dynamic forms, autocomplete, etc; Events and updates pushed by server - such as notifications, dashboards, etc; Page and data navigation - such as navigating between pages, pagination, etc can be built with LiveView using the excellent live navigation feature set. This reduces the amount of data sent over the wire, gives developers full control over the LiveView life-cycle, while controlling how the browser tracks those changes in state; There are also use cases which are a bad fit for LiveView: Animations - animations, menus, and general UI events that do not need the server in the first place are a bad fit for LiveView. Those can be achieved without LiveView in multiple ways, such as with CSS and CSS transitions, using LiveView hooks, or even integrating with UI toolkits designed for this purpose, such as Bootstrap, Alpine.JS, and similar. Life-cycle A LiveView begins as a regular HTTP request and HTML response, and then upgrades to a stateful view on client connect, guaranteeing a regular HTML page even if JavaScript is disabled. Any time a stateful view changes or updates its socket assigns, it is automatically re-rendered and the updates are pushed to the client. You begin by rendering a LiveView typically from your router. When LiveView is first rendered, the mount/3 callback is invoked with the current params, the current session and the LiveView socket. As in a regular request, params contains public data that can be modified by the user. The session always contains private data set by the application itself. The mount/3 callback wires up socket assigns necessary for rendering the view. After mounting, render/1 is invoked and the HTML is sent as a regular HTML response to the client. After rendering the static page, LiveView connects from the client to the server where stateful views are spawned to push rendered updates to the browser, and receive client events via phx- bindings. Just like the first rendering, mount/3 is invoked with params, session, and socket state, where mount assigns values for rendering. However in the connected client case, a LiveView process is spawned on the server, pushes the result of render/1 to the client and continues on for the duration of the connection. If at any point during the stateful life-cycle a crash is encountered, or the client connection drops, the client gracefully reconnects to the server, calling mount/3 once again. Example Before writing your first example, make sure that Phoenix LiveView is properly installed. If you are just getting started, this can be easily done by running mix phx.new my_app --live . The phx.new command with the --live flag will create a new project with LiveView installed and configured. Otherwise, please follow the steps in the installation guide before continuing. A LiveView is a simple module that requires two callbacks: mount/3 and render/1 : defmodule MyAppWeb.ThermostatLive do # If you generated an app with mix phx.new --live, # the line below would be: use MyAppWeb, :live_view use Phoenix.LiveView def render ( assigns ) do ~H&quot;&quot;&quot; Current temperature: &lt;%= @temperature %&gt; &quot;&quot;&quot; end def mount ( _params , %{ &quot;current_user_id&quot; =&gt; user_id } , socket ) do temperature = Thermostat . get_user_reading ( user_id ) { :ok , assign ( socket , :temperature , temperature ) } end end The render/1 callback receives the socket.assigns and is responsible for returning rendered content. We use the ~H sigil to define a HEEx template, which stands for HTML+EEx. They are an extension of Elixir's builtin EEx templates, with support for HTML validation, syntax-based components, smart change tracking, and more. You can learn more about the template syntax in Phoenix.LiveView.Helpers.sigil_H/2 . Next, decide where you want to use your LiveView. You can serve the LiveView directly from your router (recommended): defmodule MyAppWeb.Router do use Phoenix.Router import Phoenix.LiveView.Router scope &quot;/&quot; , MyAppWeb do live &quot;/thermostat&quot; , ThermostatLive end end Note: the above assumes there is plug :put_root_layout call in your router that configures the LiveView layout. This call is automatically included by mix phx.new --live and described in the installation guide. If you don't want to configure a root layout, you must pass layout: {MyAppWeb.LayoutView, &quot;app.html&quot;} as an option to the Phoenix.LiveView.Router.live/3 macro above. Alternatively, you can live_render from any template. In your view: import Phoenix.LiveView.Helpers Then in your template: &lt; h1 &gt; Temperature Control &lt; / h1 &gt; &lt; % = live_render ( @conn , MyAppWeb.ThermostatLive ) % &gt; When a LiveView is rendered, all of the data currently stored in the connection session (see Plug.Conn.get_session/1 ) will be given to the LiveView. It is also possible to pass additional session information to the LiveView through a :session option: # In the router live &quot;/thermostat&quot; , ThermostatLive , session : %{ &quot;extra_token&quot; =&gt; &quot;foo&quot; } # In a view &lt; % = live_render ( @conn , MyAppWeb.ThermostatLive , session : %{ &quot;extra_token&quot; =&gt; &quot;foo&quot; } ) % &gt; Notice the :session uses string keys as a reminder that session data is serialized and sent to the client. So you should always keep the data in the session to a minimum. For example, instead of storing a User struct, you should store the &quot;user_id&quot; and load the User when the LiveView mounts. Once the LiveView is rendered, a regular HTML response is sent. In your app.js file, you should find the following: import { Socket } from &quot;phoenix&quot; import { LiveSocket } from &quot;phoenix_live_view&quot; let csrfToken = document . querySelector ( &quot;meta[name=&#39;csrf-token&#39;]&quot; ) . getAttribute ( &quot;content&quot; ) let liveSocket = new LiveSocket ( &quot;/live&quot; , Socket , { params : { _csrf_token : csrfToken } } ) liveSocket . connect ( ) After the client connects, mount/3 will be invoked inside a spawned LiveView process. At this point, you can use connected?/1 to conditionally perform stateful work, such as subscribing to pubsub topics, sending messages, etc. For example, you can periodically update a LiveView with a timer: defmodule DemoWeb.ThermostatLive do use Phoenix.LiveView ... def mount ( _params , %{ &quot;current_user_id&quot; =&gt; user_id } , socket ) do if connected? ( socket ) , do : Process . send_after ( self ( ) , :update , 30000 ) case Thermostat . get_user_reading ( user_id ) do { :ok , temperature } -&gt; { :ok , assign ( socket , temperature : temperature , user_id : user_id ) } { :error , _reason } -&gt; { :ok , redirect ( socket , to : &quot;/error&quot; ) } end end def handle_info ( :update , socket ) do Process . send_after ( self ( ) , :update , 30000 ) { :ok , temperature } = Thermostat . get_reading ( socket . assigns . user_id ) { :noreply , assign ( socket , :temperature , temperature ) } end end We used connected?(socket) on mount to send our view a message every 30s if the socket is in a connected state. We receive the :update message in the handle_info/2 callback, just like in an Elixir GenServer , and update our socket assigns. Whenever a socket's assigns change, render/1 is automatically invoked, and the updates are sent to the client. Colocating templates In the examples above, we have placed the template directly inside the LiveView: defmodule MyAppWeb.ThermostatLive do use Phoenix.LiveView def render ( assigns ) do ~H&quot;&quot;&quot; Current temperature: &lt;%= @temperature %&gt; &quot;&quot;&quot; end For larger templates, you can place them in a file in the same directory and same name as the LiveView. For example, if the file above is placed at lib/my_app_web/live/thermostat_live.ex , you can also remove the render/1 definition above and instead put the template code at lib/my_app_web/live/thermostat_live.html.heex . Alternatively, you can keep the render/1 callback but delegate to an existing Phoenix.View module in your application. For example: defmodule MyAppWeb.ThermostatLive do use Phoenix.LiveView def render ( assigns ) do Phoenix.View . render ( MyAppWeb.PageView , &quot;page.html&quot; , assigns ) end end In all cases, each assign in the template will be accessible as @assign . You can learn more about assigns and HEEx templates in their own guide . Bindings Phoenix supports DOM element bindings for client-server interaction. For example, to react to a click on a button, you would render the element: &lt; button phx - click = &quot;inc_temperature&quot; &gt; + &lt; / button &gt; Then on the server, all LiveView bindings are handled with the handle_event/3 callback, for example: def handle_event ( &quot;inc_temperature&quot; , _value , socket ) do { :ok , new_temp } = Thermostat . inc_temperature ( socket . assigns . id ) { :noreply , assign ( socket , :temperature , new_temp ) } end Binding Attributes Params phx-value-* Click Events phx-click , phx-capture-click Focus/Blur Events phx-blur , phx-focus , phx-window-blur , phx-window-focus Key Events phx-keydown , phx-keyup , phx-window-keydown , phx-window-keyup , phx-key Form Events phx-change , phx-submit , phx-feedback-for , phx-disable-with , phx-trigger-action , phx-auto-recover Rate Limiting phx-debounce , phx-throttle DOM Patching phx-update JS Interop phx-hook Compartmentalize state, markup, and events in LiveView LiveView supports two extension mechanisms: function components, provided by HEEx templates, and stateful components. Function components are any function that receives an assigns map, similar to render(assigns) in our LiveView, and returns a ~H template. For example: def weather_greeting ( assigns ) do ~H&quot;&quot;&quot; &lt;div title=&quot;My div&quot; class={@class}&gt; &lt;p&gt;Hello &lt;%= @name %&gt;&lt;/p&gt; &lt;MyApp.Weather.city name=&quot;Kraków&quot;/&gt; &lt;/div&gt; &quot;&quot;&quot; end You can learn more about function components in the Phoenix.Component module. At the end of the day, they are useful mechanism to reuse markup in your LiveViews. However, sometimes you need to compartmentlize or reuse more than markup. Perhaps you want to move part of the state or part of the events in your LiveView to a separate module. For these cases, LiveView provides Phoenix.LiveComponent , which are rendered using live_component/2 : &lt; % = live_component ( UserComponent , id : user . id , user : user ) % &gt; Components have their own mount/3 and handle_event/3 callbacks, as well as their own state with change tracking support. Components are also lightweight as they &quot;run&quot; in the same process as the parent LiveView . However, this means an error in a component would cause the whole view to fail to render. See Phoenix.LiveComponent for a complete rundown on components. Finally, if you want complete isolation between parts of a LiveView, you can always render a LiveView inside another LiveView by calling live_render/3 . This child LiveView runs in a separate process than the parent, with its own callbacks. If a child LiveView crashes, it won't affect the parent. If the parent crashes, all children are terminated. When rendering a child LiveView, the :id option is required to uniquely identify the child. A child LiveView will only ever be rendered and mounted a single time, provided its ID remains unchanged. Given that a LiveView runs on its own process, it is an excellent tool for creating completely isolated UI elements, but it is a slightly expensive abstraction if all you want is to compartmentalize markup or events (or both). To sum it up: use Phoenix.Component to compartmentalize/reuse markup use Phoenix.LiveComponent to compartmentalize state, markup, and events use nested Phoenix.LiveView to compartmentalize state, markup, events, and error isolation Endpoint configuration LiveView accepts the following configuration in your endpoint under the :live_view key: :signing_salt (required) - the salt used to sign data sent to the client :hibernate_after (optional) - the idle time in milliseconds allowed in the LiveView before compressing its own memory and state. Defaults to 15000ms (15 seconds) Guides LiveView has many guides to help you on your journey. Server-side These guides focus on server-side functionality: Assigns and HEEx templates Error and exception handling Live Layouts Live Navigation Security considerations of the LiveView model Telemetry Uploads Using Gettext for internationalization Client-side These guides focus on LiveView bindings and client-side integration: Bindings Form bindings DOM patching and temporary assigns JavaScript interoperability Uploads (External)","ref":"Phoenix.LiveView.html","title":"Phoenix.LiveView","type":"behaviour"},{"doc":"Uses LiveView in the current module to mark it a LiveView. use Phoenix.LiveView , namespace : MyAppWeb , container : { :tr , class : &quot;colorized&quot; } , layout : { MyAppWeb.LayoutView , &quot;live.html&quot; } Options :namespace - configures the namespace the LiveView is in :container - configures the container the LiveView will be wrapped in :layout - configures the layout the LiveView will be rendered in","ref":"Phoenix.LiveView.html#__using__/1","title":"Phoenix.LiveView.__using__/1","type":"macro"},{"doc":"Allows an upload for the provided name. Options :accept - Required. A list of unique file type specifiers or the atom :any to allow any kind of file. For example, [&quot;.jpeg&quot;] , :any , etc. :max_entries - The maximum number of selected files to allow per file input. Defaults to 1. :max_file_size - The maximum file size in bytes to allow to be uploaded. Defaults 8MB. For example, 12_000_000 . :chunk_size - The chunk size in bytes to send when uploading. Defaults 64_000 . :chunk_timeout - The time in milliseconds to wait before closing the upload channel when a new chunk has not been received. Defaults 10_000 . :external - The 2-arity function for generating metadata for external client uploaders. See the Uploads section for example usage. :progress - The optional 3-arity function for receiving progress events :auto_upload - Instructs the client to upload the file automatically on file selection instead of waiting for form submits. Default false. Raises when a previously allowed upload under the same name is still active. Examples allow_upload ( socket , :avatar , accept : ~w(.jpg .jpeg) , max_entries : 2 ) allow_upload ( socket , :avatar , accept : :any ) For consuming files automatically as they are uploaded, you can pair auto_upload: true with a custom progress function to consume the entries as they are completed. For example: allow_upload ( socket , :avatar , accept : :any , progress : &amp; handle_progress / 3 , auto_upload : true ) defp handle_progress ( :avatar , entry , socket ) do if entry . done? do uploaded_file = consume_uploaded_entry ( socket , entry , fn %{ } = meta -&gt; ... end ) { :noreply , put_flash ( socket , :info , &quot;file \#{ uploaded_file . name } uploaded&quot; ) } else { :noreply , socket } end end","ref":"Phoenix.LiveView.html#allow_upload/3","title":"Phoenix.LiveView.allow_upload/3","type":"function"},{"doc":"Adds key-value pairs to assigns. The first argument is either a LiveView socket or an assigns map from function components. A keyword list or a map of assigns must be given as argument to be merged into existing assigns. Examples iex&gt; assign ( socket , name : &quot;Elixir&quot; , logo : &quot;💧&quot; ) iex&gt; assign ( socket , %{ name : &quot;Elixir&quot; } )","ref":"Phoenix.LiveView.html#assign/2","title":"Phoenix.LiveView.assign/2","type":"function"},{"doc":"Adds a key - value pair to socket_or_assigns . The first argument is either a LiveView socket or an assigns map from function components. Examples iex&gt; assign ( socket , :name , &quot;Elixir&quot; )","ref":"Phoenix.LiveView.html#assign/3","title":"Phoenix.LiveView.assign/3","type":"function"},{"doc":"Assigns the given key with value from fun into socket_or_assigns if one does not yet exist. The first argument is either a LiveView socket or an assigns map from function components. Useful for lazily assigning values and referencing parent assigns. Referencing parent assigns When a LiveView is mounted in a disconnected state, the Plug.Conn assigns will be available for reference via assign_new/3 , allowing assigns to be shared for the initial HTTP request. The Plug.Conn assigns will not be available during the connected mount. Likewise, nested LiveView children have access to their parent's assigns on mount using assign_new/3 , which allows assigns to be shared down the nested LiveView tree. Examples # controller conn |&gt; assign ( :current_user , user ) |&gt; LiveView.Controller . live_render ( MyLive , session : %{ &quot;user_id&quot; =&gt; user . id } ) # LiveView mount def mount ( _params , %{ &quot;user_id&quot; =&gt; user_id } , socket ) do { :ok , assign_new ( socket , :current_user , fn -&gt; Accounts . get_user! ( user_id ) end ) } end","ref":"Phoenix.LiveView.html#assign_new/3","title":"Phoenix.LiveView.assign_new/3","type":"function"},{"doc":"Attaches the given fun by name for the lifecycle stage into socket . Note: This function is for server-side lifecycle callbacks. For client-side hooks, see the JS Interop guide . Hooks provide a mechanism to tap into key stages of the LiveView lifecycle in order to bind/update assigns, intercept events, patches, and regular messages when necessary, and to inject common functionality. Hooks may be attached to any of the following lifecycle stages: :mount (via on_mount/1 ), :handle_params , :handle_event , and :handle_info . Return Values Lifecycle hooks take place immediately before a given lifecycle callback is invoked on the LiveView. A hook may return {:halt, socket} to halt the reduction, otherwise it must return {:cont, socket} so the operation may continue until all hooks have been invoked for the current stage. Halting the lifecycle Note that halting from a hook will halt the entire lifecycle stage . This means that when a hook returns {:halt, socket} then the LiveView callback will not be invoked. This has some implications. Implications for plugin authors When defining a plugin that matches on specific callbacks, you must define a catch-all clause, as your hook will be invoked even for events you may not be interested on. Implications for end-users Allowing a hook to halt the invocation of the callback means that you can attach hooks to intercept specific events before detaching themselves, while allowing other events to continue normally. Examples def mount ( _params , _session , socket ) do socket = attach_hook ( socket , :my_hook , :handle_event , fn &quot;very-special-event&quot; , _params , socket -&gt; # Handle the very special event and then detach the hook { :halt , detach_hook ( socket , :my_hook , :handle_event ) } _event , _params , socket -&gt; { :cont , socket } end ) { :ok , socket } end","ref":"Phoenix.LiveView.html#attach_hook/4","title":"Phoenix.LiveView.attach_hook/4","type":"function"},{"doc":"Cancels an upload for the given entry. Examples &lt; % = for entry &lt;- @uploads . avatar . entries do % &gt; ... &lt; button phx - click = &quot;cancel-upload&quot; phx - value - ref = &quot;&lt;%= entry.ref %&gt;&quot; &gt; cancel &lt; / button &gt; &lt; % end % &gt; def handle_event ( &quot;cancel-upload&quot; , %{ &quot;ref&quot; =&gt; ref } , socket ) do { :noreply , cancel_upload ( socket , :avatar , ref ) } end","ref":"Phoenix.LiveView.html#cancel_upload/3","title":"Phoenix.LiveView.cancel_upload/3","type":"function"},{"doc":"Checks if the given key changed in socket_or_assigns . The first argument is either a LiveView socket or an assigns map from function components. Examples iex&gt; changed? ( socket , :count )","ref":"Phoenix.LiveView.html#changed?/2","title":"Phoenix.LiveView.changed?/2","type":"function"},{"doc":"Clears the flash. Examples iex&gt; clear_flash ( socket )","ref":"Phoenix.LiveView.html#clear_flash/1","title":"Phoenix.LiveView.clear_flash/1","type":"function"},{"doc":"Clears a key from the flash. Examples iex&gt; clear_flash ( socket , :info )","ref":"Phoenix.LiveView.html#clear_flash/2","title":"Phoenix.LiveView.clear_flash/2","type":"function"},{"doc":"Returns true if the socket is connected. Useful for checking the connectivity status when mounting the view. For example, on initial page render, the view is mounted statically, rendered, and the HTML is sent to the client. Once the client connects to the server, a LiveView is then spawned and mounted statefully within a process. Use connected?/1 to conditionally perform stateful work, such as subscribing to pubsub topics, sending messages, etc. Examples defmodule DemoWeb.ClockLive do use Phoenix.LiveView ... def mount ( _params , _session , socket ) do if connected? ( socket ) , do : :timer . send_interval ( 1000 , self ( ) , :tick ) { :ok , assign ( socket , date : :calendar . local_time ( ) ) } end def handle_info ( :tick , socket ) do { :noreply , assign ( socket , date : :calendar . local_time ( ) ) } end end","ref":"Phoenix.LiveView.html#connected?/1","title":"Phoenix.LiveView.connected?/1","type":"function"},{"doc":"Consumes the uploaded entries. Raises when there are still entries in progress. Typically called when submitting a form to handle the uploaded entries alongside the form data. For form submissions, it is guaranteed that all entries have completed before the submit event is invoked. Once entries are consumed, they are removed from the upload. Examples def handle_event ( &quot;save&quot; , _params , socket ) do uploaded_files = consume_uploaded_entries ( socket , :avatar , fn %{ path : path } , _entry -&gt; dest = Path . join ( &quot;priv/static/uploads&quot; , Path . basename ( path ) ) File . cp! ( path , dest ) Routes . static_path ( socket , &quot;/uploads/ \#{ Path . basename ( dest ) } &quot; ) end ) { :noreply , update ( socket , :uploaded_files , &amp; ( &amp;1 ++ uploaded_files ) ) } end","ref":"Phoenix.LiveView.html#consume_uploaded_entries/3","title":"Phoenix.LiveView.consume_uploaded_entries/3","type":"function"},{"doc":"Consumes an individual uploaded entry. Raises when the entry is still in progress. Typically called when submitting a form to handle the uploaded entries alongside the form data. Once entries are consumed, they are removed from the upload. This is a lower-level feature than consume_uploaded_entries/3 and useful for scenarios where you want to consume entries as they are individually completed. Examples def handle_event ( &quot;save&quot; , _params , socket ) do case uploaded_entries ( socket , :avatar ) do { [ _ | _ ] = entries , [ ] } -&gt; uploaded_files = for entry &lt;- entries do consume_uploaded_entry ( socket , entry , fn %{ path : path } -&gt; dest = Path . join ( &quot;priv/static/uploads&quot; , Path . basename ( path ) ) File . cp! ( path , dest ) Routes . static_path ( socket , &quot;/uploads/ \#{ Path . basename ( dest ) } &quot; ) end ) end { :noreply , update ( socket , :uploaded_files , &amp; ( &amp;1 ++ uploaded_files ) ) } _ -&gt; { :noreply , socket } end end","ref":"Phoenix.LiveView.html#consume_uploaded_entry/3","title":"Phoenix.LiveView.consume_uploaded_entry/3","type":"function"},{"doc":"Detaches a hook with the given name from the lifecycle stage . Note: This function is for server-side lifecycle callbacks. For client-side hooks, see the JS Interop guide . If no hook is found, this function is a no-op. Examples def handle_event ( _ , socket ) do { :noreply , detach_hook ( socket , :hook_that_was_attached , :handle_event ) } end","ref":"Phoenix.LiveView.html#detach_hook/3","title":"Phoenix.LiveView.detach_hook/3","type":"function"},{"doc":"Revokes a previously allowed upload from allow_upload/3 . Examples disallow_upload ( socket , :avatar )","ref":"Phoenix.LiveView.html#disallow_upload/2","title":"Phoenix.LiveView.disallow_upload/2","type":"function"},{"doc":"Accesses the connect info from the socket to use on connected mount. Connect info are only sent when the client connects to the server and only remain available during mount. nil is returned when called in a disconnected state and a RuntimeError is raised if called after mount. Examples First, when invoking the LiveView socket, you need to declare the connect_info you want to receive. Typically, it includes at least the session but it may include other keys, such as :peer_data . See Phoenix.Endpoint.socket/3 : socket &quot;/live&quot; , Phoenix.LiveView.Socket , websocket : [ connect_info : [ :peer_data , session : @session_options ] ] Those values can now be accessed on the connected mount as get_connect_info/1 : def mount ( _params , _session , socket ) do if info = get_connect_info ( socket ) do { :ok , assign ( socket , ip : info . peer_data . address ) } else { :ok , assign ( socket , ip : nil ) } end end","ref":"Phoenix.LiveView.html#get_connect_info/1","title":"Phoenix.LiveView.get_connect_info/1","type":"function"},{"doc":"Accesses the connect params sent by the client for use on connected mount. Connect params are only sent when the client connects to the server and only remain available during mount. nil is returned when called in a disconnected state and a RuntimeError is raised if called after mount. Reserved params The following params have special meaning in LiveView: &quot;_csrf_token&quot; - the CSRF Token which must be explicitly set by the user when connecting &quot;_mounts&quot; - the number of times the current LiveView is mounted. It is 0 on first mount, then increases on each reconnect. It resets when navigating away from the current LiveView or on errors &quot;_track_static&quot; - set automatically with a list of all href/src from tags with the phx-track-static annotation in them. If there are no such tags, nothing is sent Examples def mount ( _params , _session , socket ) do { :ok , assign ( socket , width : get_connect_params ( socket ) [ &quot;width&quot; ] || @width ) } end","ref":"Phoenix.LiveView.html#get_connect_params/1","title":"Phoenix.LiveView.get_connect_params/1","type":"function"},{"doc":"","ref":"Phoenix.LiveView.html#c:handle_call/3","title":"Phoenix.LiveView.handle_call/3","type":"callback"},{"doc":"","ref":"Phoenix.LiveView.html#c:handle_cast/2","title":"Phoenix.LiveView.handle_cast/2","type":"callback"},{"doc":"","ref":"Phoenix.LiveView.html#c:handle_event/3","title":"Phoenix.LiveView.handle_event/3","type":"callback"},{"doc":"","ref":"Phoenix.LiveView.html#c:handle_info/2","title":"Phoenix.LiveView.handle_info/2","type":"callback"},{"doc":"","ref":"Phoenix.LiveView.html#c:handle_params/3","title":"Phoenix.LiveView.handle_params/3","type":"callback"},{"doc":"The LiveView entry-point. For each LiveView in the root of a template, mount/3 is invoked twice: once to do the initial page load and again to establish the live socket. It expects three arguments: params - a map of string keys which contain public information that can be set by the user. The map contains the query params as well as any router path parameter. If the LiveView was not mounted at the router, this argument is the atom :not_mounted_at_router session - the connection session socket - the LiveView socket It must return either {:ok, socket} or {:ok, socket, options} , where options is one of: :temporary_assigns - a keyword list of assigns that are temporary and must be reset to their value after every render. Note that once the value is reset, it won't be re-rendered again until it is explicitly assigned :layout - the optional layout to be used by the LiveView","ref":"Phoenix.LiveView.html#c:mount/3","title":"Phoenix.LiveView.mount/3","type":"callback"},{"doc":"Declares a module-function to be invoked on the LiveView's mount. The given module-function will be invoked before both disconnected and connected mounts. The hook has the option to either halt or continue the mounting process as usual. If you wish to redirect the LiveView, you must halt, otherwise an error will be raised. Registering on_mount hooks can be useful to perform authentication as well as add custom behaviour to other callbacks via attach_hook/4 . Examples defmodule DemoWeb.InitAssigns do import Phoenix.LiveView # Ensures common `assigns` are applied to all LiveViews # that attach this module as an `on_mount` hook def mount ( _params , _session , socket ) do { :cont , assign ( socket , :page_title , &quot;DemoWeb&quot; ) } end end defmodule DemoWeb.PageLive do use Phoenix.LiveView on_mount { DemoWeb.LiveAuth , :ensure_mounted_current_user } on_mount DemoWeb.InitAssigns end","ref":"Phoenix.LiveView.html#on_mount/1","title":"Phoenix.LiveView.on_mount/1","type":"macro"},{"doc":"Pushes an event to the client to be consumed by hooks. Note : events will be dispatched to all active hooks on the client who are handling the given event . Scoped events can be achieved by namespacing your event names. Examples { :noreply , push_event ( socket , &quot;scores&quot; , %{ points : 100 , user : &quot;josé&quot; } ) }","ref":"Phoenix.LiveView.html#push_event/3","title":"Phoenix.LiveView.push_event/3","type":"function"},{"doc":"Annotates the socket for navigation within the current LiveView. When navigating to the current LiveView, handle_params/3 is immediately invoked to handle the change of params and URL state. Then the new state is pushed to the client, without reloading the whole page while also maintaining the current scroll position. For live redirects to another LiveView, use push_redirect/2 . Options :to - the required path to link to. It must always be a local path :replace - the flag to replace the current history or push a new state. Defaults false . Examples { :noreply , push_patch ( socket , to : &quot;/&quot; ) } { :noreply , push_patch ( socket , to : &quot;/&quot; , replace : true ) }","ref":"Phoenix.LiveView.html#push_patch/2","title":"Phoenix.LiveView.push_patch/2","type":"function"},{"doc":"Annotates the socket for navigation to another LiveView. The current LiveView will be shutdown and a new one will be mounted in its place, without reloading the whole page. This can also be used to remount the same LiveView, in case you want to start fresh. If you want to navigate to the same LiveView without remounting it, use push_patch/2 instead. Options :to - the required path to link to. It must always be a local path :replace - the flag to replace the current history or push a new state. Defaults false . Examples { :noreply , push_redirect ( socket , to : &quot;/&quot; ) } { :noreply , push_redirect ( socket , to : &quot;/&quot; , replace : true ) }","ref":"Phoenix.LiveView.html#push_redirect/2","title":"Phoenix.LiveView.push_redirect/2","type":"function"},{"doc":"Adds a flash message to the socket to be displayed. Note : While you can use put_flash/3 inside a Phoenix.LiveComponent , components have their own @flash assigns. The @flash assign in a component is only copied to its parent LiveView if the component calls push_redirect/2 or push_patch/2 . Note : You must also place the Phoenix.LiveView.Router.fetch_live_flash/2 plug in your browser's pipeline in place of fetch_flash for LiveView flash messages be supported, for example: import Phoenix.LiveView.Router pipeline :browser do ... plug :fetch_live_flash end Examples iex&gt; put_flash ( socket , :info , &quot;It worked!&quot; ) iex&gt; put_flash ( socket , :error , &quot;You can&#39;t access that page&quot; )","ref":"Phoenix.LiveView.html#put_flash/3","title":"Phoenix.LiveView.put_flash/3","type":"function"},{"doc":"Annotates the socket for redirect to a destination path. Note : LiveView redirects rely on instructing client to perform a window.location update on the provided redirect location. The whole page will be reloaded and all state will be discarded. Options :to - the path to redirect to. It must always be a local path :external - an external path to redirect to","ref":"Phoenix.LiveView.html#redirect/2","title":"Phoenix.LiveView.redirect/2","type":"function"},{"doc":"","ref":"Phoenix.LiveView.html#c:render/1","title":"Phoenix.LiveView.render/1","type":"callback"},{"doc":"Asynchronously updates a Phoenix.LiveComponent with new assigns. The :id that identifies the component must be passed as part of the assigns and it will be used to identify the live components to be updated. The pid argument is optional and it defaults to the current process, which means the update instruction will be sent to a component running on the same LiveView. If the current process is not a LiveView or you want to send updates to a live component running on another LiveView, you should explicitly pass the LiveView's pid instead. When the component receives the update, first the optional preload/1 then update/2 is invoked with the new assigns. If update/2 is not defined all assigns are simply merged into the socket. While a component may always be updated from the parent by updating some parent assigns which will re-render the child, thus invoking update/2 on the child component, send_update/3 is useful for updating a component that entirely manages its own state, as well as messaging between components mounted in the same LiveView. Examples def handle_event ( &quot;cancel-order&quot; , _ , socket ) do ... send_update ( Cart , id : &quot;cart&quot; , status : &quot;cancelled&quot; ) { :noreply , socket } end def handle_event ( &quot;cancel-order-asynchronously&quot; , _ , socket ) do ... pid = self ( ) Task . async ( fn -&gt; # Do something asynchronously send_update ( pid , Cart , id : &quot;cart&quot; , status : &quot;cancelled&quot; ) end ) { :noreply , socket } end","ref":"Phoenix.LiveView.html#send_update/3","title":"Phoenix.LiveView.send_update/3","type":"function"},{"doc":"Similar to send_update/3 but the update will be delayed according to the given time_in_milliseconds . Examples def handle_event ( &quot;cancel-order&quot; , _ , socket ) do ... send_update_after ( Cart , [ id : &quot;cart&quot; , status : &quot;cancelled&quot; ] , 3000 ) { :noreply , socket } end def handle_event ( &quot;cancel-order-asynchronously&quot; , _ , socket ) do ... pid = self ( ) Task . async ( fn -&gt; # Do something asynchronously send_update_after ( pid , Cart , [ id : &quot;cart&quot; , status : &quot;cancelled&quot; ] , 3000 ) end ) { :noreply , socket } end","ref":"Phoenix.LiveView.html#send_update_after/4","title":"Phoenix.LiveView.send_update_after/4","type":"function"},{"doc":"Returns true if the socket is connected and the tracked static assets have changed. This function is useful to detect if the client is running on an outdated version of the marked static files. It works by comparing the static paths sent by the client with the one on the server. Note: this functionality requires Phoenix v1.5.2 or later. To use this functionality, the first step is to annotate which static files you want to be tracked by LiveView, with the phx-track-static . For example: &lt; link phx - track - static rel = &quot;stylesheet&quot; href = &quot;&lt;%= Routes.static_path(@conn, &quot; / css / app . css &quot;) %&gt;&quot; / &gt; &lt; script defer phx - track - static type = &quot;text/javascript&quot; src = &quot;&lt;%= Routes.static_path(@conn, &quot; / js / app . js &quot;) %&gt;&quot; &gt; &lt; / script &gt; Now, whenever LiveView connects to the server, it will send a copy src or href attributes of all tracked statics and compare those values with the latest entries computed by mix phx.digest in the server. The tracked statics on the client will match the ones on the server the huge majority of times. However, if there is a new deployment, those values may differ. You can use this function to detect those cases and show a banner to the user, asking them to reload the page. To do so, first set the assign on mount: def mount ( params , session , socket ) do { :ok , assign ( socket , static_changed? : static_changed? ( socket ) ) } end And then in your views: &lt; % = if @static_changed? do % &gt; &lt; div id = &quot;reload-static&quot; &gt; The app has been updated . Click here to &lt; a href = &quot;#&quot; onclick = &quot;window.location.reload()&quot; &gt; reload &lt; / a &gt; . &lt; / div &gt; &lt; % end % &gt; If you prefer, you can also send a JavaScript script that immediately reloads the page. Note: only set phx-track-static on your own assets. For example, do not set it in external JavaScript files: &lt; script defer phx - track - static type = &quot;text/javascript&quot; src = &quot;https://ajax.googleapis.com/ajax/libs/jquery/3.4.1/jquery.min.js&quot; &gt; &lt; / script &gt; Because you don't actually serve the file above, LiveView will interpret the static above as missing, and this function will return true.","ref":"Phoenix.LiveView.html#static_changed?/1","title":"Phoenix.LiveView.static_changed?/1","type":"function"},{"doc":"","ref":"Phoenix.LiveView.html#c:terminate/2","title":"Phoenix.LiveView.terminate/2","type":"callback"},{"doc":"Returns the transport pid of the socket. Raises ArgumentError if the socket is not connected. Examples iex&gt; transport_pid ( socket ) # PID &lt; 0 . 107 . 0 &gt;","ref":"Phoenix.LiveView.html#transport_pid/1","title":"Phoenix.LiveView.transport_pid/1","type":"function"},{"doc":"Updates an existing key with fun in the given socket_or_assigns . The first argument is either a LiveView socket or an assigns map from function components. The update function receives the current key's value and returns the updated value. Raises if the key does not exist. Examples iex&gt; update ( socket , :count , fn count -&gt; count + 1 end ) iex&gt; update ( socket , :count , &amp; ( &amp;1 + 1 ) )","ref":"Phoenix.LiveView.html#update/3","title":"Phoenix.LiveView.update/3","type":"function"},{"doc":"Returns the completed and in progress entries for the upload. Examples case uploaded_entries ( socket , :photos ) do { [ _ | _ ] = completed , [ ] } -&gt; # all entries are completed { [ ] , [ _ | _ ] = in_progress } -&gt; # all entries are still in progress end","ref":"Phoenix.LiveView.html#uploaded_entries/2","title":"Phoenix.LiveView.uploaded_entries/2","type":"function"},{"doc":"","ref":"Phoenix.LiveView.html#t:unsigned_params/0","title":"Phoenix.LiveView.unsigned_params/0","type":"type"},{"doc":"Helpers for rendering LiveViews from a controller.","ref":"Phoenix.LiveView.Controller.html","title":"Phoenix.LiveView.Controller","type":"module"},{"doc":"Renders a live view from a Plug request and sends an HTML response. Before rendering, the @live_module assign will be added to the connection assigns for reference. Options See Phoenix.LiveView.Helpers.live_render/3 for all supported options. Examples defmodule ThermostatController do ... import Phoenix.LiveView.Controller def show ( conn , %{ &quot;id&quot; =&gt; thermostat_id } ) do live_render ( conn , ThermostatLive , session : %{ &quot;thermostat_id&quot; =&gt; id , &quot;current_user_id&quot; =&gt; get_session ( conn , :user_id ) } ) end end","ref":"Phoenix.LiveView.Controller.html#live_render/3","title":"Phoenix.LiveView.Controller.live_render/3","type":"function"},{"doc":"A collection of helpers to be imported into your views.","ref":"Phoenix.LiveView.Helpers.html","title":"Phoenix.LiveView.Helpers","type":"module"},{"doc":"Filters the assigns as a list of keywords for use in dynamic tag attributes. Useful for transforming caller assigns into dynamic attributes while stripping reserved keys from the result. Examples Imagine the following my_link component which allows a caller to pass a new_window assign, along with any other attributes they would like to add to the element, such as class, data attributes, etc: &lt; . my_link href = &quot;/&quot; id = { @id } new_window = { true } class = &quot;my-class&quot; &gt; Home &lt; / . my_link &gt; We could support the dynamic attributes with the following component: def my_link ( assigns ) do target = if assigns [ :new_window ] , do : &quot;_blank&quot; , else : false assigns = assigns |&gt; Phoenix.LiveView . assign ( :target , target ) |&gt; Phoenix.LiveView . assign ( :extra , assigns_to_attributes ( assigns , [ :new_window ] ) ) ~H&quot;&quot;&quot; &lt;a href={@href} target={@target} {@extra}&gt; &lt;%= render_block(@inner_block) %&gt; &lt;/a&gt; &quot;&quot;&quot; end The optional second argument to assigns_to_attributes takes a list of keys to exclude which will typically be the keys reserved by the component itself which either do not belong in the markup, or are already handled explicitly by the component.","ref":"Phoenix.LiveView.Helpers.html#assigns_to_attributes/2","title":"Phoenix.LiveView.Helpers.assigns_to_attributes/2","type":"function"},{"doc":"Renders a component defined by the given function. It takes two optional arguments, the assigns to pass to the given function and a do-block - which will be converted into a @inner_block assign (see render_block/2 for more information). The given function must expect one argument, which are the assigns as a map. All of the assigns given are forwarded directly to the function as a single argument. Examples The function can be either local: &lt; % = component ( &amp; weather_component / 1 , city : &quot;Kraków&quot; ) % &gt; Or remote: &lt; % = component ( &amp; MyApp.Weather . component / 1 , city : &quot;Kraków&quot; ) % &gt;","ref":"Phoenix.LiveView.Helpers.html#component/3","title":"Phoenix.LiveView.Helpers.component/3","type":"macro"},{"doc":"Renders a form function component. This function is built on top of Phoenix.HTML.Form.form_for/4 . For more information about options and how to build inputs, see Phoenix.HTML.Form . Options The :for assign is the form's source data and the optional :action assign can be provided for the form's action. Additionally accepts the same options as Phoenix.HTML.Form.form_for/4 as optional assigns: :as - the server side parameter in which all params for this form will be collected (i.e. as: :user_params would mean all fields for this form will be accessed as conn.params.user_params server side). Automatically inflected when a changeset is given. :method - the HTTP method. If the method is not &quot;get&quot; nor &quot;post&quot;, an input tag with name _method is generated along-side the form tag. Defaults to &quot;post&quot;. :multipart - when true, sets enctype to &quot;multipart/form-data&quot;. Required when uploading files :csrf_token - for &quot;post&quot; requests, the form tag will automatically include an input tag with name _csrf_token . When set to false, this is disabled :errors - use this to manually pass a keyword list of errors to the form (for example from conn.assigns[:errors] ). This option is only used when a connection is used as the form source and it will make the errors available under f.errors :id - the ID of the form attribute. If an ID is given, all form inputs will also be prefixed by the given ID All further assigns will be passed to the form tag. Examples &lt; . form let = { f } for = { @changeset } &gt; &lt; % = text_input f , :name % &gt; &lt; / . form &gt; &lt; . form let = { user_form } for = { @changeset } as = &quot;user&quot; multipart { @extra } &gt; &lt; % = text_input user_form , :name % &gt; &lt; / . form &gt;","ref":"Phoenix.LiveView.Helpers.html#form/1","title":"Phoenix.LiveView.Helpers.form/1","type":"function"},{"doc":"Renders a Phoenix.LiveComponent within a parent LiveView. While LiveView s can be nested, each LiveView starts its own process. A LiveComponent provides similar functionality to LiveView, except they run in the same process as the LiveView , with its own encapsulated state. LiveComponent comes in two shapes, stateful and stateless. See Phoenix.LiveComponent for more information. Examples All of the assigns given are forwarded directly to the live_component : &lt; % = live_component ( MyApp.WeatherComponent , id : &quot;thermostat&quot; , city : &quot;Kraków&quot; ) % &gt; Note the :id won't necessarily be used as the DOM ID. That's up to the component. However, note that the :id has a special meaning: whenever an :id is given, the component becomes stateful. Otherwise, :id is always set to nil .","ref":"Phoenix.LiveView.Helpers.html#live_component/3","title":"Phoenix.LiveView.Helpers.live_component/3","type":"macro"},{"doc":"","ref":"Phoenix.LiveView.Helpers.html#live_component/4","title":"Phoenix.LiveView.Helpers.live_component/4","type":"macro"},{"doc":"Builds a file input tag for a LiveView upload. Options may be passed through to the tag builder for custom attributes. Drag and Drop Drag and drop is supported by annotating the droppable container with a phx-drop-target attribute pointing to the DOM ID of the file input. By default, the file input ID is the upload ref , so the following markup is all that is required for drag and drop support: &lt; div class = &quot;container&quot; phx - drop - target = &quot;&lt;%= @uploads.avatar.ref %&gt;&quot; &gt; ... &lt; % = live_file_input @uploads . avatar % &gt; &lt; / div &gt; Examples &lt; % = live_file_input @uploads . avatar % &gt;","ref":"Phoenix.LiveView.Helpers.html#live_file_input/2","title":"Phoenix.LiveView.Helpers.live_file_input/2","type":"function"},{"doc":"Returns the flash message from the LiveView flash assign. Examples &lt; p class = &quot;alert alert-info&quot; &gt; &lt; % = live_flash ( @flash , :info ) % &gt; &lt; / p &gt; &lt; p class = &quot;alert alert-danger&quot; &gt; &lt; % = live_flash ( @flash , :error ) % &gt; &lt; / p &gt;","ref":"Phoenix.LiveView.Helpers.html#live_flash/2","title":"Phoenix.LiveView.Helpers.live_flash/2","type":"function"},{"doc":"Generates an image preview on the client for a selected file. Examples &lt; % = for entry &lt;- @uploads . avatar . entries do % &gt; &lt; % = live_img_preview entry , width : 75 % &gt; &lt; % end % &gt;","ref":"Phoenix.LiveView.Helpers.html#live_img_preview/2","title":"Phoenix.LiveView.Helpers.live_img_preview/2","type":"function"},{"doc":"Generates a link that will patch the current LiveView. When navigating to the current LiveView, Phoenix.LiveView.handle_params/3 is immediately invoked to handle the change of params and URL state. Then the new state is pushed to the client, without reloading the whole page while also maintaining the current scroll position. For live redirects to another LiveView, use live_redirect/2 . Options :to - the required path to link to. :replace - the flag to replace the current history or push a new state. Defaults false . All other options are forwarded to the anchor tag. Examples &lt; % = live_patch &quot;home&quot; , to : Routes . page_path ( @socket , :index ) % &gt; &lt; % = live_patch &quot;next&quot; , to : Routes . live_path ( @socket , MyLive , @page + 1 ) % &gt; &lt; % = live_patch to : Routes . live_path ( @socket , MyLive , dir : :asc ) , replace : false do % &gt; Sort By Price &lt; % end % &gt;","ref":"Phoenix.LiveView.Helpers.html#live_patch/2","title":"Phoenix.LiveView.Helpers.live_patch/2","type":"function"},{"doc":"Generates a link that will redirect to a new LiveView of the same live session. The current LiveView will be shut down and a new one will be mounted in its place, without reloading the whole page. This can also be used to remount the same LiveView, in case you want to start fresh. If you want to navigate to the same LiveView without remounting it, use live_patch/2 instead. Note : The live redirects are only supported between two LiveViews defined under the same live session. See Phoenix.LiveView.Router.live_session/3 for more details. Options :to - the required path to link to. :replace - the flag to replace the current history or push a new state. Defaults false . All other options are forwarded to the anchor tag. Examples &lt; % = live_redirect &quot;home&quot; , to : Routes . page_path ( @socket , :index ) % &gt; &lt; % = live_redirect &quot;next&quot; , to : Routes . live_path ( @socket , MyLive , @page + 1 ) % &gt; &lt; % = live_redirect to : Routes . live_path ( @socket , MyLive , dir : :asc ) , replace : false do % &gt; Sort By Price &lt; % end % &gt;","ref":"Phoenix.LiveView.Helpers.html#live_redirect/2","title":"Phoenix.LiveView.Helpers.live_redirect/2","type":"function"},{"doc":"Renders a LiveView within an originating plug request or within a parent LiveView. Options :session - the map of extra session data to be serialized and sent to the client. Note that all session data currently in the connection is automatically available in LiveViews. You can use this option to provide extra data. Also note that the keys in the session are strings keys, as a reminder that data has to be serialized first. :container - an optional tuple for the HTML tag and DOM attributes to be used for the LiveView container. For example: {:li, style: &quot;color: blue;&quot;} . By default it uses the module definition container. See the &quot;Containers&quot; section below for more information. :id - both the DOM ID and the ID to uniquely identify a LiveView. An :id is automatically generated when rendering root LiveViews but it is a required option when rendering a child LiveView. :router - an optional router that enables this LiveView to perform live navigation. Only a single LiveView in a page may have the :router set. LiveViews defined at the router with the live macro automatically have the :router option set. Examples # within eex template &lt; % = live_render ( @conn , MyApp.ThermostatLive ) % &gt; # within leex template &lt; % = live_render ( @socket , MyApp.ThermostatLive , id : &quot;thermostat&quot; ) % &gt; Containers When a LiveView is rendered, its contents are wrapped in a container. By default, said container is a div tag with a handful of LiveView specific attributes. The container can be customized in different ways: You can change the default container on use Phoenix.LiveView : use Phoenix.LiveView , container : { :tr , id : &quot;foo-bar&quot; } You can override the container tag and pass extra attributes when calling live_render (as well as on your live call in your router): live_render socket , MyLiveView , container : { :tr , class : &quot;highlight&quot; }","ref":"Phoenix.LiveView.Helpers.html#live_render/3","title":"Phoenix.LiveView.Helpers.live_render/3","type":"function"},{"doc":"Renders a title tag with automatic prefix/suffix on @page_title updates. Examples &lt; % = live_title_tag assigns [ :page_title ] || &quot;Welcome&quot; , prefix : &quot;MyApp – &quot; % &gt; &lt; % = live_title_tag assigns [ :page_title ] || &quot;Welcome&quot; , suffix : &quot; – MyApp&quot; % &gt;","ref":"Phoenix.LiveView.Helpers.html#live_title_tag/2","title":"Phoenix.LiveView.Helpers.live_title_tag/2","type":"function"},{"doc":"Renders the @inner_block assign of a component with the given argument . &lt; % = render_block ( @inner_block , value : @value )","ref":"Phoenix.LiveView.Helpers.html#render_block/2","title":"Phoenix.LiveView.Helpers.render_block/2","type":"macro"},{"doc":"Provides ~H sigil with HTML-safe and HTML-aware syntax inside source files. Note: HEEx requires Elixir &gt;= 1.12.0 in order to provide accurate file:line:column information in error messages. Earlier Elixir versions will work but will show inaccurate error messages. HEEx is a HTML-aware and component-friendly extension of EEx that provides: Built-in handling of HTML attributes An HTML-like notation for injecting function components Compile-time validation of the structure of the template The ability to minimize the amount of data sent over the wire Example ~H&quot;&quot;&quot; &lt;div title=&quot;My div&quot; class={@class}&gt; &lt;p&gt;Hello &lt;%= @name %&gt;&lt;/p&gt; &lt;MyApp.Weather.city name=&quot;Kraków&quot;/&gt; &lt;/div&gt; &quot;&quot;&quot; Syntax HEEx is built on top of Embedded Elixir ( EEx ), a templating syntax that uses &lt;%= ... %&gt; for interpolating results. In this section, we are going to cover the basic constructs in HEEx templates as well as its syntax extensions. Interpolation Both HEEx and EEx templates use &lt;%= ... %&gt; for interpolating code inside the body of HTML tags: &lt; p &gt; Hello , &lt; % = @name % &gt; &lt; / p &gt; Similarly, conditionals and other block Elixir constructs are supported: &lt; % = if @show_greeting? do % &gt; &lt; p &gt; Hello , &lt; % = @name % &gt; &lt; / p &gt; &lt; % end % &gt; Note we don't include the equal sign = in the closing tag (because the closing tag does not output anything). There is one important difference between HEEx and Elixir's builtin EEx . HEEx uses a specific annotation for interpolating HTML tags and attributes. Let's check it out. HEEx extension: Defining attributes Since HEEx must parse and validate the HTML structure, code interpolation using &lt;%= ... %&gt; and &lt;% ... %&gt; are restricted to the body (inner content) of the HTML/component nodes and it cannot be applied within tags. For instance, the following syntax is invalid: &lt; div class = &quot;&lt;%= @class %&gt;&quot; &gt; ... &lt; / div &gt; Instead do: &lt; div class = { @class } &gt; ... &lt; / div &gt; For multiple dynamic attributes, you can use the same notation but without assigning the expression to any specific attribute. &lt; div { @dynamic_attrs } &gt; ... &lt; / div &gt; The expression inside { ... } must be either a keyword list or a map containing the key-value pairs representing the dynamic attributes. HEEx extension: Defining function components Function components are stateless components implemented as pure functions with the help of the Phoenix.Component module. They can be either local (same module) or remote (external module). HEEx allows invoking these function components directly in the template using an HTML-like notation. For example, a remote function: &lt; MyApp.Weather . city name = &quot;Kraków&quot; / &gt; A local function can be invoked with a leading dot: &lt; . city name = &quot;Kraków&quot; / &gt; where the component could be defined as follows: defmodule MyApp.Weather do use Phoenix.Component def city ( assigns ) do ~H&quot;&quot;&quot; The chosen city is: &lt;%= @name %&gt;. &quot;&quot;&quot; end def country ( assigns ) do ~H&quot;&quot;&quot; The chosen country is: &lt;%= @name %&gt;. &quot;&quot;&quot; end end It is typically best to group related functions into a single module, as opposed to having many modules with a single render/1 function. You can learn more about components in Phoenix.Component .","ref":"Phoenix.LiveView.Helpers.html#sigil_H/2","title":"Phoenix.LiveView.Helpers.sigil_H/2","type":"macro"},{"doc":"Provides ~L sigil with HTML safe Live EEx syntax inside source files. iex&gt; ~L&quot;&quot;&quot; ...&gt; Hello &lt;%= &quot;world&quot; %&gt; ...&gt; &quot;&quot;&quot; { :safe , [ &quot;Hello &quot; , &quot;world&quot; , &quot; \\n &quot; ] }","ref":"Phoenix.LiveView.Helpers.html#sigil_L/2","title":"Phoenix.LiveView.Helpers.sigil_L/2","type":"macro"},{"doc":"Returns the entry errors for an upload. The following errors may be returned: :too_many_files - The number of selected files exceeds the :max_entries constraint Examples def error_to_string ( :too_many_files ) , do : &quot;You have selected too many files&quot; &lt; % = for err &lt;- upload_errors ( @uploads . avatar ) do % &gt; &lt; div class = &quot;alert alert-danger&quot; &gt; &lt; % = error_to_string ( err ) % &gt; &lt; / div &gt; &lt; % end % &gt;","ref":"Phoenix.LiveView.Helpers.html#upload_errors/1","title":"Phoenix.LiveView.Helpers.upload_errors/1","type":"function"},{"doc":"Returns the entry errors for an upload. The following errors may be returned: :too_large - The entry exceeds the :max_file_size constraint :not_accepted - The entry does not match the :accept MIME types Examples def error_to_string ( :too_large ) , do : &quot;Too large&quot; def error_to_string ( :not_accepted ) , do : &quot;You have selected an unacceptable file type&quot; &lt; % = for entry &lt;- @uploads . avatar . entries do % &gt; &lt; % = for err &lt;- upload_errors ( @uploads . avatar , entry ) do % &gt; &lt; div class = &quot;alert alert-danger&quot; &gt; &lt; % = error_to_string ( err ) % &gt; &lt; / div &gt; &lt; % end % &gt; &lt; % end % &gt;","ref":"Phoenix.LiveView.Helpers.html#upload_errors/2","title":"Phoenix.LiveView.Helpers.upload_errors/2","type":"function"},{"doc":"Provides LiveView routing for Phoenix routers.","ref":"Phoenix.LiveView.Router.html","title":"Phoenix.LiveView.Router","type":"module"},{"doc":"Fetches the LiveView and merges with the controller flash. Replaces the default :fetch_flash plug used by Phoenix.Router . Examples defmodule AppWeb.Router do use LiveGenWeb , :router import Phoenix.LiveView.Router pipeline :browser do ... plug :fetch_live_flash end ... end","ref":"Phoenix.LiveView.Router.html#fetch_live_flash/2","title":"Phoenix.LiveView.Router.fetch_live_flash/2","type":"function"},{"doc":"Defines a LiveView route. A LiveView can be routed to by using the live macro with a path and the name of the LiveView: live &quot;/thermostat&quot; , ThermostatLive By default, you can generate a route to this LiveView by using the live_path helper: live_path ( @socket , ThermostatLive ) Actions and live navigation It is common for a LiveView to have multiple states and multiple URLs. For example, you can have a single LiveView that lists all articles on your web app. For each article there is an &quot;Edit&quot; button which, when pressed, opens up a modal on the same page to edit the article. It is a best practice to use live navigation in those cases, so when you click edit, the URL changes to &quot;/articles/1/edit&quot;, even though you are still within the same LiveView. Similarly, you may also want to show a &quot;New&quot; button, which opens up the modal to create new entries, and you want this to be reflected in the URL as &quot;/articles/new&quot;. In order to make it easier to recognize the current &quot;action&quot; your LiveView is on, you can pass the action option when defining LiveViews too: live &quot;/articles&quot; , ArticleLive.Index , :index live &quot;/articles/new&quot; , ArticleLive.Index , :new live &quot;/articles/:id/edit&quot; , ArticleLive.Index , :edit When an action is given, the generated route helpers are named after the LiveView itself (in the same way as for a controller). For the example above, we will have: article_index_path ( @socket , :index ) article_index_path ( @socket , :new ) article_index_path ( @socket , :edit , 123 ) The current action will always be available inside the LiveView as the @live_action assign, that can be used to render a LiveComponent: &lt; % = if @live_action == :new do % &gt; &lt; % = live_component MyAppWeb.ArticleLive.FormComponent % &gt; &lt; % end % &gt; Or can be used to show or hide parts of the template: &lt; % = if @live_action == :edit do % &gt; &lt; % = render ( &quot;form.html&quot; , user : @user ) % &gt; &lt; % end % &gt; Note that @live_action will be nil if no action is given on the route definition. Options :container - an optional tuple for the HTML tag and DOM attributes to be used for the LiveView container. For example: {:li, style: &quot;color: blue;&quot;} . See Phoenix.LiveView.Helpers.live_render/3 for more information and examples. :as - optionally configures the named helper. Defaults to :live when using a LiveView without actions or defaults to the LiveView name when using actions. :metadata - a map to optional feed metadata used on telemetry events and route info, for example: %{route_name: :foo, access: :user} . :private - an optional map of private data to put in the plug connection. for example: %{route_name: :foo, access: :user} . Examples defmodule MyApp.Router use Phoenix.Router import Phoenix.LiveView.Router scope &quot;/&quot; , MyApp do pipe_through [ :browser ] live &quot;/thermostat&quot; , ThermostatLive live &quot;/clock&quot; , ClockLive live &quot;/dashboard&quot; , DashboardLive , container : { :main , class : &quot;row&quot; } end end iex&gt; MyApp.Router.Helpers . live_path ( MyApp.Endpoint , MyApp.ThermostatLive ) &quot;/thermostat&quot;","ref":"Phoenix.LiveView.Router.html#live/4","title":"Phoenix.LiveView.Router.live/4","type":"macro"},{"doc":"Defines a live session for live redirects within a group of live routes. live_session/3 allow routes defined with live/4 to support live_redirect from the client with navigation purely over the existing websocket connection. This allows live routes defined in the router to mount a new root LiveView without additional HTTP requests to the server. Security Considerations You must always perform authentication and authorization in your LiveViews. If your application handle both regular HTTP requests and LiveViews, then you must perform authentication and authorization on both. This is important because live_redirect s do not go through the plug pipeline . live_session can be used to draw boundaries between groups of LiveViews. Redirecting between live_session s will always force a full page reload and establish a brand new LiveView connection. This is useful when LiveViews require different authentication strategies or simply when they use different root layouts (as the root layout is not updated between live redirects). Please read our guide on the security model for a detailed description and general tips on authentication, authorization, and more. Options :session - The optional extra session map or MFA tuple to be merged with the LiveView session. For example, %{&quot;admin&quot; =&gt; true} , {MyMod, :session, []} . For MFA, the function is invoked, passing the Plug.Conn struct is prepended to the arguments list. :root_layout - The optional root layout tuple for the initial HTTP render to override any existing root layout set in the router. :on_mount - The optional list of hooks to attach to the mount lifecycle of each LiveView in the session . See Phoenix.LiveView.on_mount/1 . Passing a single value is also accepted. Examples scope &quot;/&quot; , MyAppWeb do pipe_through :browser live_session :default do live &quot;/feed&quot; , FeedLive , :index live &quot;/status&quot; , StatusLive , :index live &quot;/status/:id&quot; , StatusLive , :show end live_session :admin , on_mount : MyAppWeb.AdminLiveAuth do live &quot;/admin&quot; , AdminDashboardLive , :index live &quot;/admin/posts&quot; , AdminPostLive , :index end end In the example above, we have two live sessions. Live navigation between live views in the different sessions is not possible and will always require a full page reload. This is important in the example above because the :admin live session has authentication requirements, defined by on_mount: MyAppWeb.AdminLiveAuth , that the other LiveViews do not have. If you have both regular HTTP routes (via get, post, etc) and live routes, then you need to perform the same authentication and authorization rules in both. For example, if you were to add a get &quot;/admin/health&quot; entry point inside the :admin live session above, then you must create your own plug that performs the same authentication and authorization rules as MyAppWeb.AdminLiveAuth , and then pipe through it: live_session :admin , on_mount : MyAppWeb.AdminLiveAuth do # Regular routes pipe_through [ MyAppWeb.AdminPlugAuth ] get &quot;/admin/health&quot; # Live routes live &quot;/admin&quot; , AdminDashboardLive , :index live &quot;/admin/posts&quot; , AdminPostLive , :index end The opposite is also true, if you have regular http routes and you want to add your own live routes, the same authentication and authorization checks executed by the plugs listed in pipe_through must be ported to LiveViews and be executed via on_mount hooks.","ref":"Phoenix.LiveView.Router.html#live_session/3","title":"Phoenix.LiveView.Router.live_session/3","type":"macro"},{"doc":"The LiveView socket for Phoenix Endpoints. This is typically mounted directly in your endpoint. socket &quot;/live&quot; , Phoenix.LiveView.Socket","ref":"Phoenix.LiveView.Socket.html","title":"Phoenix.LiveView.Socket","type":"module"},{"doc":"The data in a LiveView as stored in the socket.","ref":"Phoenix.LiveView.Socket.html#t:assigns/0","title":"Phoenix.LiveView.Socket.assigns/0","type":"type"},{"doc":"Struct returned when assigns is not in the socket.","ref":"Phoenix.LiveView.Socket.html#t:assigns_not_in_socket/0","title":"Phoenix.LiveView.Socket.assigns_not_in_socket/0","type":"opaque"},{"doc":"","ref":"Phoenix.LiveView.Socket.html#t:fingerprints/0","title":"Phoenix.LiveView.Socket.fingerprints/0","type":"type"},{"doc":"","ref":"Phoenix.LiveView.Socket.html#t:t/0","title":"Phoenix.LiveView.Socket.t/0","type":"type"},{"doc":"API for function components. A function component is any function that receives an assigns map as argument and returns a rendered struct built with the ~H sigil . Here is an example: defmodule MyComponent do use Phoenix.Component # Optionally also bring the HTML helpers # use Phoenix.HTML def greet ( assigns ) do ~H&quot;&quot;&quot; &lt;p&gt;Hello, &lt;%= assigns.name %&gt;&lt;/p&gt; &quot;&quot;&quot; end end The component can be invoked as a regular function: MyComponent . greet ( %{ name : &quot;Jane&quot; } ) But it is typically invoked using the function component syntax from the ~H sigil: ~H&quot;&quot;&quot; &lt;MyComponent.greet name=&quot;Jane&quot; /&gt; &quot;&quot;&quot; If the MyComponent module is imported or if the function is defined locally, you can skip the module name: ~H&quot;&quot;&quot; &lt;.greet name=&quot;Jane&quot; /&gt; &quot;&quot;&quot; Learn more about the ~H sigil in its documentation . use Phoenix.Component Modules that have to define function components should call use Phoenix.Component at the top. Doing so will import the functions from both Phoenix.LiveView and Phoenix.LiveView.Helpers modules. Note it is not necessary to use Phoenix.Component inside Phoenix.LiveView and Phoenix.LiveComponent . Assigns While inside a function component, you must use the assign/3 and assign_new/3 functions in Phoenix.LiveView to manipulate assigns, so that LiveView can track changes to the assigns values. For example, let's imagine a component that receives the first name and last name and must compute the name assign. One option would be: def show_name ( assigns ) do assigns = assign ( assigns , :name , assigns . first_name &lt;&gt; assigns . last_name ) ~H&quot;&quot;&quot; &lt;p&gt;Your name is: &lt;%= @name %&gt;&lt;/p&gt; &quot;&quot;&quot; end However, when possible, it may be cleaner to break the logic over function calls instead of precomputed assigns: def show_name ( assigns ) do ~H&quot;&quot;&quot; &lt;p&gt;Your name is: &lt;%= full_name(@first_name, @last_name) %&gt;&lt;/p&gt; &quot;&quot;&quot; end defp full_name ( first_name , last_name ) , do : first_name &lt;&gt; last_name Another example is making an assign optional by providing a default value: def field_label ( assigns ) do assigns = assign_new ( assigns , :help , fn -&gt; nil end ) ~H&quot;&quot;&quot; &lt;label&gt; &lt;%= @text %&gt; &lt;%= if @help do %&gt; &lt;span class=&quot;help&quot;&gt;&lt;%= @help %&gt;&lt;/span&gt; &lt;% end %&gt; &lt;/label&gt; &quot;&quot;&quot; end Blocks It is also possible to give HTML blocks to function components as in regular HTML tags. For example, you could create a button component that looks like this: def button ( assigns ) do ~H&quot;&quot;&quot; &lt;button class=&quot;btn&quot;&gt; &lt;%= render_block(@inner_block) %&gt; &lt;/button&gt; &quot;&quot;&quot; end and now you can invoke it as: &lt; . button &gt; This renders &lt; strong &gt; inside &lt; / strong &gt; the button! &lt; / . button &gt; In a nutshell, the block given to the component is assigned to @inner_block and then we use render_block to render it. You can even have the component give a value back to the caller, by using let . Imagine this component: def unordered_list ( assigns ) do ~H&quot;&quot;&quot; &lt;ul&gt; &lt;%= for entry &lt;- @entries do %&gt; &lt;li&gt;&lt;%= render_block(@inner_block, entry) %&gt;&lt;/li&gt; &lt;% end %&gt; &lt;/ul&gt; &quot;&quot;&quot; end And now you can invoke it as: &lt; . unordered_list let = { entry } entries = { ~w(apple banana cherry) } &gt; I like &lt; % = entry % &gt; &lt; / . unordered_list &gt;","ref":"Phoenix.Component.html","title":"Phoenix.Component","type":"module"},{"doc":"Components are a mechanism to compartmentalize state, markup, and events in LiveView. Components are defined by using Phoenix.LiveComponent and are used by calling Phoenix.LiveView.Helpers.live_component/3 in a parent LiveView. Components run inside the LiveView process but have their own life-cycle. Components which are rendered with an :id are stateful, and those rendered without an :id are stateless. Stateful components can also receive events. Note stateless LiveComponents are deprecated. Please use Phoenix.Component instead. The simplest component only needs to define a render/1 function: defmodule HeroComponent do # If you generated an app with mix phx.new --live, # the line below would be: use MyAppWeb, :live_component use Phoenix.LiveComponent def render ( assigns ) do ~H&quot;&quot;&quot; &lt;div class=&quot;hero&quot;&gt;&lt;%= @content %&gt;&lt;/div&gt; &quot;&quot;&quot; end end When use Phoenix.LiveComponent is used, all functions in Phoenix.LiveView and Phoenix.LiveView.Helpers are imported. A component can be invoked as: &lt; % = live_component HeroComponent , id : :hero , content : @content % &gt; A component must receive the :id assign as argument, which is used to uniquely identify the component. A component will be treated as the same component as long as its :id does not change. Note: previous LiveView versions allowed the :id to be skipped on live_component but those are now discouraged since the addition of function components, outlined in Phoenix.Component . Life-cycle Stateful components are identified by the component module and their ID. Therefore, two different component modules with the same ID are different components. This means we can often tie the component ID to some application based ID: &lt; % = live_component UserComponent , id : @user . id , user : @user % &gt; When live_component/3 is called, mount/1 is called once, when the component is first added to the page. mount/1 receives the socket as argument. Then update/2 is invoked with all of the assigns given to live_component/3 . If update/2 is not defined all assigns are simply merged into the socket. After the component is updated, render/1 is called with all assigns. On first render, we get: mount ( socket ) -&gt; update ( assigns , socket ) -&gt; render ( assigns ) On further rendering: update ( assigns , socket ) -&gt; render ( assigns ) Note all stateful components require a single root element in the HTML template and you will receive a warning otherwise. Furthermore, the given :id is not necessarily used as the DOM ID. If you want to set a DOM ID, it is your responsibility to do so when rendering: defmodule UserComponent do use Phoenix.LiveComponent def render ( assigns ) do ~H&quot;&quot;&quot; &lt;div id={&quot;user-\#{@id}&quot;} class=&quot;user&quot;&gt; &lt;%= @user.name %&gt; &lt;/div&gt; &quot;&quot;&quot; end end Targeting Component Events Stateful components can also implement the handle_event/3 callback that works exactly the same as in LiveView. For a client event to reach a component, the tag must be annotated with a phx-target . If you want to send the event to yourself, you can simply use the @myself assign, which is an internal unique reference to the component instance: &lt; a href = &quot;#&quot; phx - click = &quot;say_hello&quot; phx - target = { @myself } &gt; Say hello! &lt; / a &gt; Note that @myself is not set for stateless components, as they cannot receive events. If you want to target another component, you can also pass an ID or a class selector to any element inside the targeted component. For example, if there is a UserComponent with the DOM ID of &quot;user-13&quot; , using a query selector, we can send an event to it with: &lt; a href = &quot;#&quot; phx - click = &quot;say_hello&quot; phx - target = &quot;#user-13&quot; &gt; Say hello! &lt; / a &gt; In both cases, handle_event/3 will be called with the &quot;say_hello&quot; event. When handle_event/3 is called for a component, only the diff of the component is sent to the client, making them extremely efficient. Any valid query selector for phx-target is supported, provided that the matched nodes are children of a LiveView or LiveComponent, for example to send the close event to multiple components: &lt; a href = &quot;#&quot; phx - click = &quot;close&quot; phx - target = &quot;#modal, #sidebar&quot; &gt; Dismiss &lt; / a &gt; Preloading and update Stateful components also support an optional preload/1 callback. The preload/1 callback is useful when multiple components of the same type are rendered on the page and you want to preload or augment their data in batches. For each rendering, the optional preload/1 and update/2 callbacks are called before render/1 . So on first render, the following callbacks will be invoked: preload ( list_of_assigns ) -&gt; mount ( socket ) -&gt; update ( assigns , socket ) -&gt; render ( assigns ) On subsequent renders, these callbacks will be invoked: preload ( list_of_assigns ) -&gt; update ( assigns , socket ) -&gt; render ( assigns ) To provide a more complete understanding of why both callbacks are necessary, let's see an example. Imagine you are implementing a component and the component needs to load some state from the database. For example: &lt; % = live_component UserComponent , id : user_id % &gt; A possible implementation would be to load the user on the update/2 callback: def update ( assigns , socket ) do user = Repo . get! ( User , assigns . id ) { :ok , assign ( socket , :user , user ) } end However, the issue with said approach is that, if you are rendering multiple user components in the same page, you have a N+1 query problem. The preload/1 callback helps address this problem as it is invoked with a list of assigns for all components of the same type. For example, instead of implementing update/2 as above, one could implement: def preload ( list_of_assigns ) do list_of_ids = Enum . map ( list_of_assigns , &amp; &amp;1 . id ) users = from ( u in User , where : u . id in ^ list_of_ids , select : { u . id , u } ) |&gt; Repo . all ( ) |&gt; Map . new ( ) Enum . map ( list_of_assigns , fn assigns -&gt; Map . put ( assigns , :user , users [ assigns . id ] ) end ) end Now only a single query to the database will be made. In fact, the preloading algorithm is a breadth-first tree traversal, which means that even for nested components, the amount of queries are kept to a minimum. Finally, note that preload/1 must return an updated list_of_assigns , keeping the assigns in the same order as they were given. Managing state Now that we have learned how to define and use components, as well as how to use preload/1 as a data loading optimization, it is important to talk about how to manage state in components. Generally speaking, you want to avoid both the parent LiveView and the LiveComponent working on two different copies of the state. Instead, you should assume only one of them to be the source of truth. Let's discuss the two different approaches in detail. Imagine a scenario where a LiveView represents a board with each card in it as a separate stateful LiveComponent. Each card has a form to allow update of the card title directly in the component, as follows: defmodule CardComponent do use Phoenix.LiveComponent def render ( assigns ) do ~H&quot;&quot;&quot; &lt;form phx-submit=&quot;...&quot; phx-target={@myself}&gt; &lt;input name=&quot;title&quot;&gt;&lt;%= @card.title %&gt;&lt;/input&gt; ... &lt;/form&gt; &quot;&quot;&quot; end ... end We will see how to organize the data flow to keep either the board LiveView or the card LiveComponents as the source of truth. LiveView as the source of truth If the board LiveView is the source of truth, it will be responsible for fetching all of the cards in a board. Then it will call live_component/3 for each card, passing the card struct as argument to CardComponent : &lt; % = for card &lt;- @cards do % &gt; &lt; % = live_component CardComponent , card : card , id : card . id , board_id : @id % &gt; &lt; % end % &gt; Now, when the user submits the form, CardComponent.handle_event/3 will be triggered. However, if the update succeeds, you must not change the card struct inside the component. If you do so, the card struct in the component will get out of sync with the LiveView. Since the LiveView is the source of truth, you should instead tell the LiveView that the card was updated. Luckily, because the component and the view run in the same process, sending a message from the LiveComponent to the parent LiveView is as simple as sending a message to self() : defmodule CardComponent do ... def handle_event ( &quot;update_title&quot; , %{ &quot;title&quot; =&gt; title } , socket ) do send self ( ) , { :updated_card , %{ socket . assigns . card | title : title } } { :noreply , socket } end end The LiveView then receives this event using Phoenix.LiveView.handle_info/2 : defmodule BoardView do ... def handle_info ( { :updated_card , card } , socket ) do # update the list of cards in the socket { :noreply , updated_socket } end end Because the list of cards in the parent socket was updated, the parent LiveView will be re-rendered, sending the updated card to the component. So in the end, the component does get the updated card, but always driven from the parent. Alternatively, instead of having the component send a message directly to the parent view, the component could broadcast the update using Phoenix.PubSub . Such as: defmodule CardComponent do ... def handle_event ( &quot;update_title&quot; , %{ &quot;title&quot; =&gt; title } , socket ) do message = { :updated_card , %{ socket . assigns . card | title : title } } Phoenix.PubSub . broadcast ( MyApp.PubSub , board_topic ( socket ) , message ) { :noreply , socket } end defp board_topic ( socket ) do &quot;board:&quot; &lt;&gt; socket . assigns . board_id end end As long as the parent LiveView subscribes to the board:&lt;ID&gt; topic, it will receive updates. The advantage of using PubSub is that we get distributed updates out of the box. Now, if any user connected to the board changes a card, all other users will see the change. LiveComponent as the source of truth If each card LiveComponent is the source of truth, then the board LiveView must no longer fetch the card structs from the database. Instead, the board LiveView must only fetch the card ids, then render each component only by passing an ID: &lt; % = for card_id &lt;- @card_ids do % &gt; &lt; % = live_component CardComponent , id : card_id , board_id : @id % &gt; &lt; % end % &gt; Now, each CardComponent will load its own card. Of course, doing so per card could be expensive and lead to N queries, where N is the number of cards, so we can use the preload/1 callback to make it efficient. Once the card components are started, they can each manage their own card, without concerning themselves with the parent LiveView. However, note that components do not have a Phoenix.LiveView.handle_info/2 callback. Therefore, if you want to track distributed changes on a card, you must have the parent LiveView receive those events and redirect them to the appropriate card. For example, assuming card updates are sent to the &quot;board:ID&quot; topic, and that the board LiveView is subscribed to said topic, one could do: def handle_info ( { :updated_card , card } , socket ) do send_update CardComponent , id : card . id , board_id : socket . assigns . id { :noreply , socket } end With Phoenix.LiveView.send_update/3 , the CardComponent given by id will be invoked, triggering both preload and update callbacks, which will load the most up to date data from the database. LiveComponent blocks When live_component/3 is invoked, it is also possible to pass a do/end block: &lt; % = live_component GridComponent , entries : @entries do % &gt; &lt; % entry -&gt; % &gt; New entry : &lt; % = entry % &gt; &lt; % end % &gt; The do/end will be available in an assign named @inner_block . You can render its contents by calling render_block with the assign itself and a keyword list of assigns to inject into the rendered content. For example, the grid component above could be implemented as: defmodule GridComponent do use Phoenix.LiveComponent def render ( assigns ) do ~H&quot;&quot;&quot; &lt;div class=&quot;grid&quot;&gt; &lt;%= for entry &lt;- @entries do %&gt; &lt;div class=&quot;column&quot;&gt; &lt;%= render_block(@inner_block, entry) %&gt; &lt;/div&gt; &lt;% end %&gt; &lt;/div&gt; &quot;&quot;&quot; end end Where the entry variable was injected into the do/end block. Note the @inner_block assign is also passed to update/2 along all other assigns. So if you have a custom update/2 implementation, make sure to assign it to the socket like so: def update ( %{ inner_block : inner_block } , socket ) do { :ok , assign ( socket , inner_block : inner_block ) } end Live patches and live redirects A template rendered inside a component can use Phoenix.LiveView.Helpers.live_patch/2 and Phoenix.LiveView.Helpers.live_redirect/2 calls. The live_patch/2 is always handled by the parent LiveView , as components do not provide handle_params . Cost of stateful components The internal infrastructure LiveView uses to keep track of stateful components is very lightweight. However, be aware that in order to provide change tracking and to send diffs over the wire, all of the components' assigns are kept in memory - exactly as it is done in LiveViews themselves. Therefore it is your responsibility to keep only the assigns necessary in each component. For example, avoid passing all of LiveView's assigns when rendering a component: &lt; % = live_component MyComponent , assigns % &gt; Instead pass only the keys that you need: &lt; % = live_component MyComponent , user : @user , org : @org % &gt; Luckily, because LiveViews and LiveComponents are in the same process, they share the data structure representations in memory. For example, in the code above, the view and the component will share the same copies of the @user and @org assigns. You should also avoid using stateful components to provide abstract DOM components. As a guideline, a good LiveComponent encapsulates application concerns and not DOM functionality. For example, if you have a page that shows products for sale, you can encapsulate the rendering of each of those products in a component. This component may have many buttons and events within it. On the opposite side, do not write a component that is simply encapsulating generic DOM components. For instance, do not do this: defmodule MyButton do use Phoenix.LiveComponent def render ( assigns ) do ~H&quot;&quot;&quot; &lt;button class=&quot;css-framework-class&quot; phx-click=&quot;click&quot;&gt; &lt;%= @text %&gt; &lt;/button&gt; &quot;&quot;&quot; end def handle_event ( &quot;click&quot; , _ , socket ) do _ = socket . assigns . on_click . ( ) { :noreply , socket } end end Instead, it is much simpler to create a function: def my_button ( text , click ) do assigns = %{ text : text , click : click } ~H&quot;&quot;&quot; &lt;button class=&quot;css-framework-class&quot; phx-click={@click}&gt; &lt;%= @text %&gt; &lt;/button&gt; &quot;&quot;&quot; end If you keep components mostly as an application concern with only the necessary assigns, it is unlikely you will run into issues related to stateful components. Limitations Components require at least one HTML tag Components must only contain HTML tags at their root. At least one HTML tag must be present. It is not possible to have components that render only text or text mixed with tags at the root. Change tracking requirement Another limitation of components is that they must always be change tracked. For example, if you render a component inside content_tag , like this: &lt; % = content_tag :div , @div_attrs do % &gt; &lt; % = live_component SomeComponent , id : :example % &gt; &lt; % end % &gt; The component ends up enclosed by the content_tag , where LiveView cannot track it. In such cases, you may receive an error such as: ** (ArgumentError) cannot convert component SomeComponent to HTML. A component must always be returned directly as part of a LiveView template Luckily, there is little reason to use content_tag inside HEEx templates. So instead you can do: &lt; div { @div_attrs } &gt; &lt; % = live_component SomeComponent , id : :example % &gt; &lt; / div &gt; Similarly, they also work inside any function component, such as form : &lt; . form let = { f } for = { @changeset } url = &quot;#&quot; &gt; &lt; % = live_component FormComponent , id : :form % &gt; &lt; / . form &gt; SVG support Given components compartmentalize markup on the server, they are also rendered in isolation on the client, which provides great performance benefits on the client too. However, when rendering components on the client, the client needs to choose the mime type of the component contents, which defaults to HTML. This is the best default but in some cases it may lead to unexpected results. For example, if you are rendering SVG, the SVG will be interpreted as HTML. This may work just fine for most components but you may run into corner cases. For example, the &lt;image&gt; SVG tag may be rewritten to the &lt;img&gt; tag, since &lt;image&gt; is an obsolete HTML tag. Luckily, there is a simple solution to this problem. Since SVG allows &lt;svg&gt; tags to be nested, you can wrap the component content into an &lt;svg&gt; tag. This will ensure that it is correctly interpreted by the browser.","ref":"Phoenix.LiveComponent.html","title":"Phoenix.LiveComponent","type":"behaviour"},{"doc":"","ref":"Phoenix.LiveComponent.html#c:handle_event/3","title":"Phoenix.LiveComponent.handle_event/3","type":"callback"},{"doc":"","ref":"Phoenix.LiveComponent.html#c:mount/1","title":"Phoenix.LiveComponent.mount/1","type":"callback"},{"doc":"","ref":"Phoenix.LiveComponent.html#c:preload/1","title":"Phoenix.LiveComponent.preload/1","type":"callback"},{"doc":"","ref":"Phoenix.LiveComponent.html#c:render/1","title":"Phoenix.LiveComponent.render/1","type":"callback"},{"doc":"","ref":"Phoenix.LiveComponent.html#c:update/2","title":"Phoenix.LiveComponent.update/2","type":"callback"},{"doc":"Conveniences for testing Phoenix LiveViews. In LiveView tests, we interact with views via process communication in substitution of a browser. Like a browser, our test process receives messages about the rendered updates from the view which can be asserted against to test the life-cycle and behavior of LiveViews and their children. LiveView Testing The life-cycle of a LiveView as outlined in the Phoenix.LiveView docs details how a view starts as a stateless HTML render in a disconnected socket state. Once the browser receives the HTML, it connects to the server and a new LiveView process is started, remounted in a connected socket state, and the view continues statefully. The LiveView test functions support testing both disconnected and connected mounts separately, for example: import Plug.Conn import Phoenix.ConnTest import Phoenix.LiveViewTest @endpoint MyEndpoint test &quot;disconnected and connected mount&quot; , %{ conn : conn } do conn = get ( conn , &quot;/my-path&quot; ) assert html_response ( conn , 200 ) =~ &quot;&lt;h1&gt;My Disconnected View&lt;/h1&gt;&quot; { :ok , view , html } = live ( conn ) end test &quot;redirected mount&quot; , %{ conn : conn } do assert { :error , { :redirect , %{ to : &quot;/somewhere&quot; } } } = live ( conn , &quot;my-path&quot; ) end Here, we start by using the familiar Phoenix.ConnTest function, get/2 to test the regular HTTP GET request which invokes mount with a disconnected socket. Next, live/1 is called with our sent connection to mount the view in a connected state, which starts our stateful LiveView process. In general, it's often more convenient to test the mounting of a view in a single step, provided you don't need the result of the stateless HTTP render. This is done with a single call to live/2 , which performs the get step for us: test &quot;connected mount&quot; , %{ conn : conn } do { :ok , _view , html } = live ( conn , &quot;/my-path&quot; ) assert html =~ &quot;&lt;h1&gt;My Connected View&lt;/h1&gt;&quot; end Testing Events The browser can send a variety of events to a LiveView via phx- bindings, which are sent to the handle_event/3 callback. To test events sent by the browser and assert on the rendered side effect of the event, use the render_* functions: render_click/1 - sends a phx-click event and value, returning the rendered result of the handle_event/3 callback. render_focus/2 - sends a phx-focus event and value, returning the rendered result of the handle_event/3 callback. render_blur/1 - sends a phx-blur event and value, returning the rendered result of the handle_event/3 callback. render_submit/1 - sends a form phx-submit event and value, returning the rendered result of the handle_event/3 callback. render_change/1 - sends a form phx-change event and value, returning the rendered result of the handle_event/3 callback. render_keydown/1 - sends a form phx-keydown event and value, returning the rendered result of the handle_event/3 callback. render_keyup/1 - sends a form phx-keyup event and value, returning the rendered result of the handle_event/3 callback. render_hook/3 - sends a hook event and value, returning the rendered result of the handle_event/3 callback. For example: { :ok , view , _html } = live ( conn , &quot;/thermo&quot; ) assert view |&gt; element ( &quot;button#inc&quot; ) |&gt; render_click ( ) =~ &quot;The temperature is: 31℉&quot; In the example above, we are looking for a particular element on the page and triggering its phx-click event. LiveView takes care of making sure the element has a phx-click and automatically sends its values to the server. You can also bypass the element lookup and directly trigger the LiveView event in most functions: assert render_click ( view , :inc , %{ } ) =~ &quot;The temperature is: 31℉&quot; The element style is preferred as much as possible, as it helps LiveView perform validations and ensure the events in the HTML actually matches the event names on the server. Testing regular messages LiveViews are GenServer 's under the hood, and can send and receive messages just like any other server. To test the side effects of sending or receiving messages, simply message the view and use the render function to test the result: send ( view . pid , { :set_temp , 50 } ) assert render ( view ) =~ &quot;The temperature is: 50℉&quot; Testing components There are two main mechanisms for testing components. To test stateless components or just a regular rendering of a component, one can use render_component/2 : assert render_component ( MyComponent , id : 123 , user : % User { } ) =~ &quot;some markup in component&quot; If you want to test how components are mounted by a LiveView and interact with DOM events, you can use the regular live/2 macro to build the LiveView with the component and then scope events by passing the view and a DOM selector in a list: { :ok , view , html } = live ( conn , &quot;/users&quot; ) html = view |&gt; element ( &quot;#user-13 a&quot; , &quot;Delete&quot; ) |&gt; render_click ( ) refute html =~ &quot;user-13&quot; refute view |&gt; element ( &quot;#user-13&quot; ) |&gt; has_element? ( ) In the example above, LiveView will lookup for an element with ID=user-13 and retrieve its phx-target . If phx-target points to a component, that will be the component used, otherwise it will fallback to the view.","ref":"Phoenix.LiveViewTest.html","title":"Phoenix.LiveViewTest","type":"module"},{"doc":"Asserts a live patch will happen within timeout milliseconds. The default timeout is 100. It returns the new path. To assert on the flash message, you can assert on the result of the rendered LiveView. Examples render_click ( view , :event_that_triggers_patch ) assert_patch view render_click ( view , :event_that_triggers_patch ) assert_patch view , 30 render_click ( view , :event_that_triggers_patch ) path = assert_patch view assert path =~ ~r/path/ \d + /","ref":"Phoenix.LiveViewTest.html#assert_patch/2","title":"Phoenix.LiveViewTest.assert_patch/2","type":"function"},{"doc":"Asserts a live patch will to a given path within timeout milliseconds. The default timeout is 100. It always returns :ok . To assert on the flash message, you can assert on the result of the rendered LiveView. Examples render_click ( view , :event_that_triggers_patch ) assert_patch view , &quot;/path&quot; render_click ( view , :event_that_triggers_patch ) assert_patch view , &quot;/path&quot; , 30","ref":"Phoenix.LiveViewTest.html#assert_patch/3","title":"Phoenix.LiveViewTest.assert_patch/3","type":"function"},{"doc":"Asserts a live patch was performed, and returns the new path. To assert on the flash message, you can assert on the result of the rendered LiveView. Examples render_click ( view , :event_that_triggers_redirect ) assert_patched view , &quot;/path&quot;","ref":"Phoenix.LiveViewTest.html#assert_patched/2","title":"Phoenix.LiveViewTest.assert_patched/2","type":"function"},{"doc":"Asserts an event will be pushed within timeout . Examples assert_push_event view , &quot;scores&quot; , %{ points : 100 , user : &quot;josé&quot; }","ref":"Phoenix.LiveViewTest.html#assert_push_event/4","title":"Phoenix.LiveViewTest.assert_push_event/4","type":"macro"},{"doc":"Asserts a redirect will happen within timeout milliseconds. The default timeout is 100. It returns a tuple containing the new path and the flash messages from said redirect, if any. Note the flash will contain string keys. Examples render_click ( view , :event_that_triggers_redirect ) { _path , flash } = assert_redirect view assert flash [ &quot;info&quot; ] == &quot;Welcome&quot; assert path =~ ~r/path/ \d + / render_click ( view , :event_that_triggers_redirect ) assert_redirect view , 30","ref":"Phoenix.LiveViewTest.html#assert_redirect/2","title":"Phoenix.LiveViewTest.assert_redirect/2","type":"function"},{"doc":"Asserts a redirect will happen to a given path within timeout milliseconds. The default timeout is 100. It returns the flash messages from said redirect, if any. Note the flash will contain string keys. Examples render_click ( view , :event_that_triggers_redirect ) flash = assert_redirect view , &quot;/path&quot; assert flash [ &quot;info&quot; ] == &quot;Welcome&quot; render_click ( view , :event_that_triggers_redirect ) assert_redirect view , &quot;/path&quot; , 30","ref":"Phoenix.LiveViewTest.html#assert_redirect/3","title":"Phoenix.LiveViewTest.assert_redirect/3","type":"function"},{"doc":"Asserts a redirect was performed. It returns a tuple containing the new path and the flash messages from said redirect, if any. Note the flash will contain string keys. Examples render_click ( view , :event_that_triggers_redirect ) { _path , flash } = assert_redirected view , &quot;/path&quot; assert flash [ &quot;info&quot; ] == &quot;Welcome&quot;","ref":"Phoenix.LiveViewTest.html#assert_redirected/2","title":"Phoenix.LiveViewTest.assert_redirected/2","type":"function"},{"doc":"Asserts a hook reply was returned from a handle_event callback. Examples assert_reply view , %{ result : &quot;ok&quot; , transaction_id : _ }","ref":"Phoenix.LiveViewTest.html#assert_reply/3","title":"Phoenix.LiveViewTest.assert_reply/3","type":"macro"},{"doc":"Returns an element to scope a function to. It expects the current LiveView, a query selector, and a text filter. An optional text filter may be given to filter the results by the query selector. If the text filter is a string or a regex, it will match any element that contains the string or matches the regex. After the text filter is applied, only one element must remain, otherwise an error is raised. If no text filter is given, then the query selector itself must return a single element. assert view |&gt; element ( &quot;#term a:first-child()&quot; , &quot;Increment&quot; ) |&gt; render ( ) =~ &quot;Increment&lt;/a&gt;&quot; Attribute selectors are also supported, and may be used on special cases like ids which contain periods: assert view |&gt; element ( ~s{[href=&quot;/foo&quot;][id=&quot;foo.bar.baz&quot;]} ) |&gt; render ( ) =~ &quot;Increment&lt;/a&gt;&quot;","ref":"Phoenix.LiveViewTest.html#element/3","title":"Phoenix.LiveViewTest.element/3","type":"function"},{"doc":"Builds a file input for testing uploads within a form. Given the form DOM selector, the upload name, and a list of maps of client metadata for the upload, the returned file input can be passed to render_upload/2 . Client metadata takes the following form: :last_modified - the last modified timestamp :name - the name of the file :content - the binary content of the file :size - the byte size of the content :type - the MIME type of the file Examples avatar = file_input ( lv , &quot;#my-form-id&quot; , :avatar , [ %{ last_modified : 1_594_171_879_000 , name : &quot;myfile.jpeg&quot; , content : File . read! ( &quot;myfile.jpg&quot; ) , size : 1_396_009 , type : &quot;image/jpeg&quot; } ] ) assert render_upload ( avatar , &quot;myfile.jpeg&quot; ) =~ &quot;100%&quot;","ref":"Phoenix.LiveViewTest.html#file_input/4","title":"Phoenix.LiveViewTest.file_input/4","type":"macro"},{"doc":"Gets the nested LiveView child by child_id from the parent LiveView. Examples { :ok , view , _html } = live ( conn , &quot;/thermo&quot; ) assert clock_view = find_live_child ( view , &quot;clock&quot; ) assert render_click ( clock_view , :snooze ) =~ &quot;snoozing&quot;","ref":"Phoenix.LiveViewTest.html#find_live_child/2","title":"Phoenix.LiveViewTest.find_live_child/2","type":"function"},{"doc":"Follows the redirect from a render_* action or an {:error, redirect} tuple. Imagine you have a LiveView that redirects on a render_click event. You can make it sure it immediately redirects after the render_click action by calling follow_redirect/3 : live_view |&gt; render_click ( &quot;redirect&quot; ) |&gt; follow_redirect ( conn ) Or in the case of an error tuple: assert { :error , { :redirect , %{ to : &quot;/somewhere&quot; } } } = result = live ( conn , &quot;my-path&quot; ) { :ok , view , html } = follow_redirect ( result , conn ) follow_redirect/3 expects a connection as second argument. This is the connection that will be used to perform the underlying request. If the LiveView redirects with a live redirect, this macro returns {:ok, live_view, disconnected_html} with the content of the new LiveView, the same as the live/3 macro. If the LiveView redirects with a regular redirect, this macro returns {:ok, conn} with the rendered redirected page. In any other case, this macro raises. Finally, note that you can optionally assert on the path you are being redirected to by passing a third argument: live_view |&gt; render_click ( &quot;redirect&quot; ) |&gt; follow_redirect ( conn , &quot;/redirected/page&quot; )","ref":"Phoenix.LiveViewTest.html#follow_redirect/3","title":"Phoenix.LiveViewTest.follow_redirect/3","type":"macro"},{"doc":"Receives a form_element and asserts that phx-trigger-action has been set to true, following up on that request. Imagine you have a LiveView that sends an HTTP form submission. Say that it sets the phx-trigger-action to true, as a response to a submit event. You can follow the trigger action like this: form = form ( live_view , selector , %{ &quot;form&quot; =&gt; &quot;data&quot; } ) # First we submit the form. Optionally verify that phx-trigger-action # is now part of the form. assert render_submit ( form ) =~ ~r/phx-trigger-action/ # Now follow the request made by the form conn = follow_trigger_action ( form , conn ) assert conn . method == &quot;POST&quot; assert conn . params == %{ &quot;form&quot; =&gt; &quot;data&quot; }","ref":"Phoenix.LiveViewTest.html#follow_trigger_action/2","title":"Phoenix.LiveViewTest.follow_trigger_action/2","type":"macro"},{"doc":"Returns a form element to scope a function to. It expects the current LiveView, a query selector, and the form data. The query selector must return a single element. The form data will be validated directly against the form markup and make sure the data you are changing/submitting actually exists, failing otherwise. Examples assert view |&gt; form ( &quot;#term&quot; , user : %{ name : &quot;hello&quot; } ) |&gt; render_submit ( ) =~ &quot;Name updated&quot; This function is meant to mimic what the user can actually do, so you cannot set hidden input values. However, hidden values can be given when calling render_submit/2 or render_change/2 , see their docs for examples.","ref":"Phoenix.LiveViewTest.html#form/3","title":"Phoenix.LiveViewTest.form/3","type":"function"},{"doc":"Checks if the given element exists on the page. Examples assert view |&gt; element ( &quot;#some-element&quot; ) |&gt; has_element? ( )","ref":"Phoenix.LiveViewTest.html#has_element?/1","title":"Phoenix.LiveViewTest.has_element?/1","type":"function"},{"doc":"Checks if the given selector with text_filter is on view . See element/3 for more information. Examples assert has_element? ( view , &quot;#some-element&quot; )","ref":"Phoenix.LiveViewTest.html#has_element?/3","title":"Phoenix.LiveViewTest.has_element?/3","type":"function"},{"doc":"Spawns a connected LiveView process. If a path is given, then a regular get(conn, path) is done and the page is upgraded to a LiveView . If no path is given, it assumes a previously rendered %Plug.Conn{} is given, which will be converted to a LiveView immediately. Examples { :ok , view , html } = live ( conn , &quot;/path&quot; ) assert view . module = MyLive assert html =~ &quot;the count is 3&quot; assert { :error , { :redirect , %{ to : &quot;/somewhere&quot; } } } = live ( conn , &quot;/path&quot; )","ref":"Phoenix.LiveViewTest.html#live/2","title":"Phoenix.LiveViewTest.live/2","type":"macro"},{"doc":"Returns the current list of LiveView children for the parent LiveView. Children are returned in the order they appear in the rendered HTML. Examples { :ok , view , _html } = live ( conn , &quot;/thermo&quot; ) assert [ clock_view ] = live_children ( view ) assert render_click ( clock_view , :snooze ) =~ &quot;snoozing&quot;","ref":"Phoenix.LiveViewTest.html#live_children/1","title":"Phoenix.LiveViewTest.live_children/1","type":"function"},{"doc":"Spawns a connected LiveView process mounted in isolation as the sole rendered element. Useful for testing LiveViews that are not directly routable, such as those built as small components to be re-used in multiple parents. Testing routable LiveViews is still recommended whenever possible since features such as live navigation require routable LiveViews. Options :session - the session to be given to the LiveView All other options are forwarded to the LiveView for rendering. Refer to Phoenix.LiveView.Helpers.live_render/3 for a list of supported render options. Examples { :ok , view , html } = live_isolated ( conn , AppWeb.ClockLive , session : %{ &quot;tz&quot; =&gt; &quot;EST&quot; } ) Use put_connect_params/2 to put connect params for a call to Phoenix.LiveView.get_connect_params/1 in Phoenix.LiveView.mount/3 : { :ok , view , html } = conn |&gt; put_connect_params ( %{ &quot;param&quot; =&gt; &quot;value&quot; } ) |&gt; live_isolated ( AppWeb.ClockLive , session : %{ &quot;tz&quot; =&gt; &quot;EST&quot; } )","ref":"Phoenix.LiveViewTest.html#live_isolated/3","title":"Phoenix.LiveViewTest.live_isolated/3","type":"macro"},{"doc":"Performs a live redirect from one LiveView to another. When redirecting between two LiveViews of the same live_session , mounts the new LiveView and shutsdown the previous one, which mimics general browser live navigation behaviour. When attempting to navigate from a LiveView of a different live_session , an error redirect condition is returned indicating a failed live_redirect from the client. Examples assert { :ok , page_live , _html } = live ( conn , &quot;/page/1&quot; ) assert { :ok , page2_live , _html } = live ( conn , &quot;/page/2&quot; ) assert { :error , { :redirect , _ } } = live_redirect ( page2_live , to : &quot;/admin&quot; )","ref":"Phoenix.LiveViewTest.html#live_redirect/2","title":"Phoenix.LiveViewTest.live_redirect/2","type":"function"},{"doc":"Open the default browser to display current HTML of view_or_element . Examples view |&gt; element ( &quot;#term a:first-child()&quot; , &quot;Increment&quot; ) |&gt; open_browser ( ) assert view |&gt; form ( &quot;#term&quot; , user : %{ name : &quot;hello&quot; } ) |&gt; open_browser ( ) |&gt; render_submit ( ) =~ &quot;Name updated&quot;","ref":"Phoenix.LiveViewTest.html#open_browser/2","title":"Phoenix.LiveViewTest.open_browser/2","type":"function"},{"doc":"Returns the most recent title that was updated via a page_title assign. Examples render_click ( view , :event_that_triggers_page_title_update ) assert page_title ( view ) =~ &quot;my title&quot;","ref":"Phoenix.LiveViewTest.html#page_title/1","title":"Phoenix.LiveViewTest.page_title/1","type":"function"},{"doc":"Performs a preflight upload request. Useful for testing external uploaders to retrieve the :external entry metadata. Examples avatar = file_input ( lv , &quot;#my-form-id&quot; , :avatar , [ %{ name : ... , ... } , ... ] ) assert { :ok , %{ ref : _ref , config : %{ chunk_size : _ } } } = preflight_upload ( avatar )","ref":"Phoenix.LiveViewTest.html#preflight_upload/1","title":"Phoenix.LiveViewTest.preflight_upload/1","type":"function"},{"doc":"Puts connect info to be used on LiveView connections. See Phoenix.LiveView.get_connect_info/1 .","ref":"Phoenix.LiveViewTest.html#put_connect_info/2","title":"Phoenix.LiveViewTest.put_connect_info/2","type":"function"},{"doc":"Puts connect params to be used on LiveView connections. See Phoenix.LiveView.get_connect_params/1 .","ref":"Phoenix.LiveViewTest.html#put_connect_params/2","title":"Phoenix.LiveViewTest.put_connect_params/2","type":"function"},{"doc":"Refutes a redirect to a given path was performed. It returns :ok if the specified redirect isn't already in the mailbox. Examples render_click ( view , :event_that_triggers_redirect_to_path ) :ok = refute_redirect view , &quot;/wrong_path&quot;","ref":"Phoenix.LiveViewTest.html#refute_redirected/2","title":"Phoenix.LiveViewTest.refute_redirected/2","type":"function"},{"doc":"Returns the HTML string of the rendered view or element. If a view is provided, the entire LiveView is rendered. If a view after calling with_target/2 or an element are given, only that particular context is returned. Examples { :ok , view , _html } = live ( conn , &quot;/thermo&quot; ) assert render ( view ) =~ ~s|&lt;button id=&quot;alarm&quot;&gt;Snooze&lt;/div&gt;| assert view |&gt; element ( &quot;#alarm&quot; ) |&gt; render ( ) == &quot;Snooze&quot;","ref":"Phoenix.LiveViewTest.html#render/1","title":"Phoenix.LiveViewTest.render/1","type":"function"},{"doc":"Sends a blur event given by element and returns the rendered result. The element is created with element/3 and must point to a single element on the page with a phx-blur attribute in it. The event name given set on phx-blur is then sent to the appropriate LiveView (or component if phx-target is set accordingly). All phx-value-* entries in the element are sent as values. Extra values can be given with the value argument. It returns the contents of the whole LiveView or an {:error, redirect} tuple. Examples { :ok , view , html } = live ( conn , &quot;/thermo&quot; ) assert view |&gt; element ( &quot;#inactive&quot; ) |&gt; render_blur ( ) =~ &quot;Tap to wake&quot;","ref":"Phoenix.LiveViewTest.html#render_blur/2","title":"Phoenix.LiveViewTest.render_blur/2","type":"function"},{"doc":"Sends a blur event to the view and returns the rendered result. It returns the contents of the whole LiveView or an {:error, redirect} tuple. Examples { :ok , view , html } = live ( conn , &quot;/thermo&quot; ) assert html =~ &quot;The temp is: 30℉&quot; assert render_blur ( view , :inactive ) =~ &quot;Tap to wake&quot;","ref":"Phoenix.LiveViewTest.html#render_blur/3","title":"Phoenix.LiveViewTest.render_blur/3","type":"function"},{"doc":"Sends a form change event given by element and returns the rendered result. The element is created with element/3 and must point to a single element on the page with a phx-change attribute in it. The event name given set on phx-change is then sent to the appropriate LiveView (or component if phx-target is set accordingly). All phx-value-* entries in the element are sent as values. If you need to pass any extra values or metadata, such as the &quot;_target&quot; parameter, you can do so by giving a map under the value argument. It returns the contents of the whole LiveView or an {:error, redirect} tuple. Examples { :ok , view , html } = live ( conn , &quot;/thermo&quot; ) assert view |&gt; element ( &quot;form&quot; ) |&gt; render_change ( %{ deg : 123 } ) =~ &quot;123 exceeds limits&quot; # Passing metadata { :ok , view , html } = live ( conn , &quot;/thermo&quot; ) assert view |&gt; element ( &quot;form&quot; ) |&gt; render_change ( %{ _target : [ &quot;deg&quot; ] , deg : 123 } ) =~ &quot;123 exceeds limits&quot; As with render_submit/2 , hidden input field values can be provided like so: refute view |&gt; form ( &quot;#term&quot; , user : %{ name : &quot;hello&quot; } ) |&gt; render_change ( %{ user : %{ &quot;hidden_field&quot; =&gt; &quot;example&quot; } } ) =~ &quot;can&#39;t be blank&quot;","ref":"Phoenix.LiveViewTest.html#render_change/2","title":"Phoenix.LiveViewTest.render_change/2","type":"function"},{"doc":"Sends a form change event to the view and returns the rendered result. It returns the contents of the whole LiveView or an {:error, redirect} tuple. Examples { :ok , view , html } = live ( conn , &quot;/thermo&quot; ) assert html =~ &quot;The temp is: 30℉&quot; assert render_change ( view , :validate , %{ deg : 123 } ) =~ &quot;123 exceeds limits&quot;","ref":"Phoenix.LiveViewTest.html#render_change/3","title":"Phoenix.LiveViewTest.render_change/3","type":"function"},{"doc":"Sends a click event given by element and returns the rendered result. The element is created with element/3 and must point to a single element on the page with a phx-click attribute in it. The event name given set on phx-click is then sent to the appropriate LiveView (or component if phx-target is set accordingly). All phx-value-* entries in the element are sent as values. Extra values can be given with the value argument. If the element is does not have a phx-click attribute but it is a link (the &lt;a&gt; tag), the link will be followed accordingly: if the link is a live_patch , the current view will be patched if the link is a live_redirect , this function will return {:error, {:live_redirect, %{to: url}}} , which can be followed with follow_redirect/2 if the link is a regular link, this function will return {:error, {:redirect, %{to: url}}} , which can be followed with follow_redirect/2 It returns the contents of the whole LiveView or an {:error, redirect} tuple. Examples { :ok , view , html } = live ( conn , &quot;/thermo&quot; ) assert view |&gt; element ( &quot;buttons&quot; , &quot;Increment&quot; ) |&gt; render_click ( ) =~ &quot;The temperature is: 30℉&quot;","ref":"Phoenix.LiveViewTest.html#render_click/2","title":"Phoenix.LiveViewTest.render_click/2","type":"function"},{"doc":"Sends a click event to the view with value and returns the rendered result. It returns the contents of the whole LiveView or an {:error, redirect} tuple. Examples { :ok , view , html } = live ( conn , &quot;/thermo&quot; ) assert html =~ &quot;The temperature is: 30℉&quot; assert render_click ( view , :inc ) =~ &quot;The temperature is: 31℉&quot;","ref":"Phoenix.LiveViewTest.html#render_click/3","title":"Phoenix.LiveViewTest.render_click/3","type":"function"},{"doc":"Renders a component. The first argument may either be a function component, as an anonymous function: assert render_component ( &amp; Weather . city / 1 , name : &quot;Kraków&quot; ) =~ &quot;some markup in component&quot; Or a stateful component as a module. In this case, this function will mount, update, and render the component. The :id option is a required argument: assert render_component ( MyComponent , id : 123 , user : % User { } ) =~ &quot;some markup in component&quot; If your component is using the router, you can pass it as argument: assert render_component ( MyComponent , %{ id : 123 , user : % User { } } , router : SomeRouter ) =~ &quot;some markup in component&quot;","ref":"Phoenix.LiveViewTest.html#render_component/3","title":"Phoenix.LiveViewTest.render_component/3","type":"macro"},{"doc":"Sends a focus event given by element and returns the rendered result. The element is created with element/3 and must point to a single element on the page with a phx-focus attribute in it. The event name given set on phx-focus is then sent to the appropriate LiveView (or component if phx-target is set accordingly). All phx-value-* entries in the element are sent as values. Extra values can be given with the value argument. It returns the contents of the whole LiveView or an {:error, redirect} tuple. Examples { :ok , view , html } = live ( conn , &quot;/thermo&quot; ) assert view |&gt; element ( &quot;#inactive&quot; ) |&gt; render_focus ( ) =~ &quot;Tap to wake&quot;","ref":"Phoenix.LiveViewTest.html#render_focus/2","title":"Phoenix.LiveViewTest.render_focus/2","type":"function"},{"doc":"Sends a focus event to the view and returns the rendered result. It returns the contents of the whole LiveView or an {:error, redirect} tuple. Examples { :ok , view , html } = live ( conn , &quot;/thermo&quot; ) assert html =~ &quot;The temp is: 30℉&quot; assert render_focus ( view , :inactive ) =~ &quot;Tap to wake&quot;","ref":"Phoenix.LiveViewTest.html#render_focus/3","title":"Phoenix.LiveViewTest.render_focus/3","type":"function"},{"doc":"Sends a hook event to the view or an element and returns the rendered result. It returns the contents of the whole LiveView or an {:error, redirect} tuple. Examples { :ok , view , html } = live ( conn , &quot;/thermo&quot; ) assert html =~ &quot;The temp is: 30℉&quot; assert render_hook ( view , :refresh , %{ deg : 32 } ) =~ &quot;The temp is: 32℉&quot; If you are pushing events from a hook to a component, then you must pass an element , created with element/3 , as first argument and it must point to a single element on the page with a phx-target attribute in it: { :ok , view , _html } = live ( conn , &quot;/thermo&quot; ) assert view |&gt; element ( &quot;#thermo-component&quot; ) |&gt; render_hook ( :refresh , %{ deg : 32 } ) =~ &quot;The temp is: 32℉&quot;","ref":"Phoenix.LiveViewTest.html#render_hook/3","title":"Phoenix.LiveViewTest.render_hook/3","type":"function"},{"doc":"Sends a keydown event given by element and returns the rendered result. The element is created with element/3 and must point to a single element on the page with a phx-keydown or phx-window-keydown attribute in it. The event name given set on phx-keydown is then sent to the appropriate LiveView (or component if phx-target is set accordingly). All phx-value-* entries in the element are sent as values. Extra values can be given with the value argument. It returns the contents of the whole LiveView or an {:error, redirect} tuple. Examples { :ok , view , html } = live ( conn , &quot;/thermo&quot; ) assert html =~ &quot;The temp is: 30℉&quot; assert view |&gt; element ( &quot;#inc&quot; ) |&gt; render_keydown ( ) =~ &quot;The temp is: 31℉&quot;","ref":"Phoenix.LiveViewTest.html#render_keydown/2","title":"Phoenix.LiveViewTest.render_keydown/2","type":"function"},{"doc":"Sends a keydown event to the view and returns the rendered result. It returns the contents of the whole LiveView or an {:error, redirect} tuple. Examples { :ok , view , html } = live ( conn , &quot;/thermo&quot; ) assert html =~ &quot;The temp is: 30℉&quot; assert render_keydown ( view , :inc ) =~ &quot;The temp is: 31℉&quot;","ref":"Phoenix.LiveViewTest.html#render_keydown/3","title":"Phoenix.LiveViewTest.render_keydown/3","type":"function"},{"doc":"Sends a keyup event given by element and returns the rendered result. The element is created with element/3 and must point to a single element on the page with a phx-keyup or phx-window-keyup attribute in it. The event name given set on phx-keyup is then sent to the appropriate LiveView (or component if phx-target is set accordingly). All phx-value-* entries in the element are sent as values. Extra values can be given with the value argument. It returns the contents of the whole LiveView or an {:error, redirect} tuple. Examples { :ok , view , html } = live ( conn , &quot;/thermo&quot; ) assert html =~ &quot;The temp is: 30℉&quot; assert view |&gt; element ( &quot;#inc&quot; ) |&gt; render_keyup ( ) =~ &quot;The temp is: 31℉&quot;","ref":"Phoenix.LiveViewTest.html#render_keyup/2","title":"Phoenix.LiveViewTest.render_keyup/2","type":"function"},{"doc":"Sends a keyup event to the view and returns the rendered result. It returns the contents of the whole LiveView or an {:error, redirect} tuple. Examples { :ok , view , html } = live ( conn , &quot;/thermo&quot; ) assert html =~ &quot;The temp is: 30℉&quot; assert render_keyup ( view , :inc ) =~ &quot;The temp is: 31℉&quot;","ref":"Phoenix.LiveViewTest.html#render_keyup/3","title":"Phoenix.LiveViewTest.render_keyup/3","type":"function"},{"doc":"Simulates a live_patch to the given path and returns the rendered result.","ref":"Phoenix.LiveViewTest.html#render_patch/2","title":"Phoenix.LiveViewTest.render_patch/2","type":"function"},{"doc":"Sends a form submit event given by element and returns the rendered result. The element is created with element/3 and must point to a single element on the page with a phx-submit attribute in it. The event name given set on phx-submit is then sent to the appropriate LiveView (or component if phx-target is set accordingly). All phx-value-* entries in the element are sent as values. Extra values, including hidden input fields, can be given with the value argument. It returns the contents of the whole LiveView or an {:error, redirect} tuple. Examples { :ok , view , html } = live ( conn , &quot;/thermo&quot; ) assert view |&gt; element ( &quot;form&quot; ) |&gt; render_submit ( %{ deg : 123 , avatar : upload } ) =~ &quot;123 exceeds limits&quot; To submit a form along with some with hidden input values: assert view |&gt; form ( &quot;#term&quot; , user : %{ name : &quot;hello&quot; } ) |&gt; render_submit ( %{ user : %{ &quot;hidden_field&quot; =&gt; &quot;example&quot; } } ) =~ &quot;Name updated&quot;","ref":"Phoenix.LiveViewTest.html#render_submit/2","title":"Phoenix.LiveViewTest.render_submit/2","type":"function"},{"doc":"Sends a form submit event to the view and returns the rendered result. It returns the contents of the whole LiveView or an {:error, redirect} tuple. Examples { :ok , view , html } = live ( conn , &quot;/thermo&quot; ) assert html =~ &quot;The temp is: 30℉&quot; assert render_submit ( view , :refresh , %{ deg : 32 } ) =~ &quot;The temp is: 32℉&quot;","ref":"Phoenix.LiveViewTest.html#render_submit/3","title":"Phoenix.LiveViewTest.render_submit/3","type":"function"},{"doc":"Performs an upload of a file input and renders the result. See file_input/4 for details on building a file input. Examples Given the following LiveView template: &lt; % = for entry &lt;- @uploads . avatar . entries do % &gt; &lt; % = entry . name % &gt; : &lt; % = entry . progress % &gt; % &lt; % end % &gt; Your test case can assert the uploaded content: avatar = file_input ( lv , &quot;#my-form-id&quot; , :avatar , [ %{ last_modified : 1_594_171_879_000 , name : &quot;myfile.jpeg&quot; , content : File . read! ( &quot;myfile.jpg&quot; ) , size : 1_396_009 , type : &quot;image/jpeg&quot; } ] ) assert render_upload ( avatar , &quot;myfile.jpeg&quot; ) =~ &quot;100%&quot; By default, the entire file is chunked to the server, but an optional percentage to chunk can be passed to test chunk-by-chunk uploads: assert render_upload ( avatar , &quot;myfile.jpeg&quot; , 49 ) =~ &quot;49%&quot; assert render_upload ( avatar , &quot;myfile.jpeg&quot; , 51 ) =~ &quot;100%&quot;","ref":"Phoenix.LiveViewTest.html#render_upload/3","title":"Phoenix.LiveViewTest.render_upload/3","type":"function"},{"doc":"Sets the target of the view for events. This emulates phx-target directly in tests, without having to dispatch the event to a specific element. This can be useful for invoking events to one or multiple components at the same time: view |&gt; with_target ( &quot;#user-1,#user-2&quot; ) |&gt; render_click ( &quot;Hide&quot; , %{ } )","ref":"Phoenix.LiveViewTest.html#with_target/2","title":"Phoenix.LiveViewTest.with_target/2","type":"function"},{"doc":"The struct returned by Phoenix.LiveViewTest.element/3 . The following public fields represent the element: selector - The query selector text_filter - The text to further filter the element See the Phoenix.LiveViewTest documentation for usage.","ref":"Phoenix.LiveViewTest.Element.html","title":"Phoenix.LiveViewTest.Element","type":"module"},{"doc":"The struct returned by Phoenix.LiveViewTest.file_input/4 . The following public fields represent the element: selector - The query selector entries - The list of selected file entries See the Phoenix.LiveViewTest documentation for usage.","ref":"Phoenix.LiveViewTest.Upload.html","title":"Phoenix.LiveViewTest.Upload","type":"module"},{"doc":"The struct for testing LiveViews. The following public fields represent the LiveView: id - The DOM id of the LiveView module - The module of the running LiveView pid - The Pid of the running LiveView endpoint - The endpoint for the LiveView target - The target to scope events to See the Phoenix.LiveViewTest documentation for usage.","ref":"Phoenix.LiveViewTest.View.html","title":"Phoenix.LiveViewTest.View","type":"module"},{"doc":"The struct representing an upload.","ref":"Phoenix.LiveView.UploadConfig.html","title":"Phoenix.LiveView.UploadConfig","type":"module"},{"doc":"","ref":"Phoenix.LiveView.UploadConfig.html#t:t/0","title":"Phoenix.LiveView.UploadConfig.t/0","type":"type"},{"doc":"The struct representing an upload entry.","ref":"Phoenix.LiveView.UploadEntry.html","title":"Phoenix.LiveView.UploadEntry","type":"module"},{"doc":"","ref":"Phoenix.LiveView.UploadEntry.html#t:t/0","title":"Phoenix.LiveView.UploadEntry.t/0","type":"type"},{"doc":"The struct representing an internal unique reference to the component instance, available as the @myself assign in stateful components. Read more about the uses of @myself in the Phoenix.LiveComponent docs.","ref":"Phoenix.LiveComponent.CID.html","title":"Phoenix.LiveComponent.CID","type":"module"},{"doc":"The struct returned by components in .heex templates. This component is never meant to be output directly into the template. It should always be handled by the diffing algorithm.","ref":"Phoenix.LiveView.Component.html","title":"Phoenix.LiveView.Component","type":"module"},{"doc":"","ref":"Phoenix.LiveView.Component.html#t:t/0","title":"Phoenix.LiveView.Component.t/0","type":"type"},{"doc":"The struct returned by for-comprehensions in .heex templates. See a description about its fields and use cases in Phoenix.LiveView.Engine docs.","ref":"Phoenix.LiveView.Comprehension.html","title":"Phoenix.LiveView.Comprehension","type":"module"},{"doc":"","ref":"Phoenix.LiveView.Comprehension.html#t:t/0","title":"Phoenix.LiveView.Comprehension.t/0","type":"type"},{"doc":"An EEx template engine that tracks changes. This is often used by Phoenix.LiveView.HTMLEngine which also adds HTML validation. In the documentation below, we will explain how it works internally. For user-facing documentation, see Phoenix.LiveView . Phoenix.LiveView.Rendered Whenever you render a live template, it returns a Phoenix.LiveView.Rendered structure. This structure has three fields: :static , :dynamic and :fingerprint . The :static field is a list of literal strings. This allows the Elixir compiler to optimize this list and avoid allocating its strings on every render. The :dynamic field contains a function that takes a boolean argument (see &quot;Tracking changes&quot; below), and returns a list of dynamic content. Each element in the list is either one of: iodata - which is the dynamic content nil - the dynamic content did not change another Phoenix.LiveView.Rendered struct, see &quot;Nesting and fingerprinting&quot; below a Phoenix.LiveView.Comprehension struct, see &quot;Comprehensions&quot; below a Phoenix.LiveView.Component struct, see &quot;Component&quot; below When you render a live template, you can convert the rendered structure to iodata by alternating the static and dynamic fields, always starting with a static entry followed by a dynamic entry. The last entry will always be static too. So the following structure: % Phoenix.LiveView.Rendered { static : [ &quot;foo&quot; , &quot;bar&quot; , &quot;baz&quot; ] , dynamic : fn track_changes? -&gt; [ &quot;left&quot; , &quot;right&quot; ] end } Results in the following content to be sent over the wire as iodata: [ &quot;foo&quot; , &quot;left&quot; , &quot;bar&quot; , &quot;right&quot; , &quot;baz&quot; ] This is also what calling Phoenix.HTML.Safe.to_iodata/1 with a Phoenix.LiveView.Rendered structure returns. Of course, the benefit of live templates is exactly that you do not need to send both static and dynamic segments every time. So let's talk about tracking changes. Tracking changes By default, a live template does not track changes. Change tracking can be enabled by including a changed map in the assigns with the key __changed__ and passing true to the dynamic parts. The map should contain the name of any changed field as key and the boolean true as value. If a field is not listed in __changed__ , then it is always considered unchanged. If a field is unchanged and live believes a dynamic expression no longer needs to be computed, its value in the dynamic list will be nil . This information can be leveraged to avoid sending data to the client. Nesting and fingerprinting Phoenix.LiveView also tracks changes across live templates. Therefore, if your view has this: &lt; % = render &quot;form.html&quot; , assigns % &gt; Phoenix will be able to track what is static and dynamic across templates, as well as what changed. A rendered nested live template will appear in the dynamic list as another Phoenix.LiveView.Rendered structure, which must be handled recursively. However, because the rendering of live templates can be dynamic in itself, it is important to distinguish which live template was rendered. For example, imagine this code: &lt; % = if something? , do : render ( &quot;one.html&quot; , assigns ) , else : render ( &quot;other.html&quot; , assigns ) % &gt; To solve this, all Phoenix.LiveView.Rendered structs also contain a fingerprint field that uniquely identifies it. If the fingerprints are equal, you have the same template, and therefore it is possible to only transmit its changes. Comprehensions Another optimization done by live templates is to track comprehensions. If your code has this: &lt; % = for point &lt;- @points do % &gt; x : &lt; % = point . x % &gt; y : &lt; % = point . y % &gt; &lt; % end % &gt; Instead of rendering all points with both static and dynamic parts, it returns a Phoenix.LiveView.Comprehension struct with the static parts, that are shared across all points, and a list of dynamics to be interpolated inside the static parts. If @points is a list with %{x: 1, y: 2} and %{x: 3, y: 4} , the above expression would return: % Phoenix.LiveView.Comprehension { static : [ &quot; \\n x: &quot; , &quot; \\n y: &quot; , &quot; \\n &quot; ] , dynamics : [ [ &quot;1&quot; , &quot;2&quot; ] , [ &quot;3&quot; , &quot;4&quot; ] ] } This allows live templates to drastically optimize the data sent by comprehensions, as the static parts are emitted only once, regardless of the number of items. The list of dynamics is always a list of iodatas or components, as we don't perform change tracking inside the comprehensions themselves. Similarly, comprehensions do not have fingerprints because they are only optimized at the root, so conditional evaluation, as the one seen in rendering, is not possible. The only possible outcome for a dynamic field that returns a comprehension is nil . Components Live also supports stateful components defined with Phoenix.LiveComponent . Since they are stateful, they are always handled lazily by the diff algorithm.","ref":"Phoenix.LiveView.Engine.html","title":"Phoenix.LiveView.Engine","type":"module"},{"doc":"The HTMLEngine that powers .heex templates and the ~H sigil.","ref":"Phoenix.LiveView.HTMLEngine.html","title":"Phoenix.LiveView.HTMLEngine","type":"module"},{"doc":"Callback implementation for EEx.Engine.handle_text/3 .","ref":"Phoenix.LiveView.HTMLEngine.html#handle_text/3","title":"Phoenix.LiveView.HTMLEngine.handle_text/3","type":"function"},{"doc":"The struct returned by .heex templates. See a description about its fields and use cases in Phoenix.LiveView.Engine docs.","ref":"Phoenix.LiveView.Rendered.html","title":"Phoenix.LiveView.Rendered","type":"module"},{"doc":"","ref":"Phoenix.LiveView.Rendered.html#t:t/0","title":"Phoenix.LiveView.Rendered.t/0","type":"type"},{"doc":"","ref":"Phoenix.LiveView.HTMLTokenizer.ParseError.html","title":"Phoenix.LiveView.HTMLTokenizer.ParseError","type":"exception"},{"doc":"","ref":"Phoenix.LiveView.HTMLTokenizer.ParseError.html#format_file_line_column/4","title":"Phoenix.LiveView.HTMLTokenizer.ParseError.format_file_line_column/4","type":"function"},{"doc":"Installation Note: Phoenix v1.5 comes with built-in support for LiveView apps. Just create your application with mix phx.new my_app --live . If you are using earlier Phoenix versions or your app already exists, keep on reading. The instructions below will serve if you are installing the latest stable version from Hex. To start using LiveView, add one of the following dependencies to your mix.exs and run mix deps.get . If installing from Hex, use the latest version from there: def deps do [ { :phoenix_live_view , &quot;~&gt; 0.16.4&quot; } , { :floki , &quot;&gt;= 0.30.0&quot; , only : :test } ] end If you want the latest features, install from GitHub: def deps do [ { :phoenix_live_view , github : &quot;phoenixframework/phoenix_live_view&quot; } , { :floki , &quot;&gt;= 0.30.0&quot; , only : :test } ] Once installed, update your endpoint's configuration to include a signing salt. You can generate a signing salt by running mix phx.gen.secret 32 : # config/config.exs config :my_app , MyAppWeb.Endpoint , live_view : [ signing_salt : &quot;SECRET_SALT&quot; ] Next, add the following imports to your web file in lib/my_app_web.ex : # lib/my_app_web.ex def view do quote do ... import Phoenix.LiveView.Helpers end end def router do quote do ... import Phoenix.LiveView.Router end end Then add the Phoenix.LiveView.Router.fetch_live_flash/2 plug to your browser pipeline, in place of :fetch_flash : # lib/my_app_web/router.ex pipeline :browser do ... plug :fetch_session - plug :fetch_flash + plug :fetch_live_flash end Next, expose a new socket for LiveView updates in your app's endpoint module. # lib/my_app_web/endpoint.ex defmodule MyAppWeb.Endpoint do use Phoenix.Endpoint # ... socket &quot;/live&quot; , Phoenix.LiveView.Socket , websocket : [ connect_info : [ session : @session_options ] ] # ... end Where @session_options are the options given to plug Plug.Session by using a module attribute. If you don't have a @session_options in your endpoint yet, here is how to create one: Find plug Plug.Session in your endpoint.ex plug Plug.Session store : :cookie , key : &quot;_my_app_key&quot; , signing_salt : &quot;somesigningsalt&quot; Move the options to a module attribute at the top of your file: @session_options [ store : :cookie , key : &quot;_my_app_key&quot; , signing_salt : &quot;somesigningsalt&quot; ] Change the plug Plug.Session to use that attribute: plug Plug.Session , @session_options Finally, ensure you have placed a CSRF meta tag inside the &lt;head&gt; tag in your layout ( lib/my_app_web/templates/layout/app.html.* ) before app.js is included, like so: &lt;%= csrf_meta_tag() %&gt; &lt;script defer type=&quot;text/javascript&quot; src=&quot;&lt;%= Routes.static_path(@conn, &quot;/js/app.js&quot;) %&gt;&quot;&gt;&lt;/script&gt; and enable connecting to a LiveView socket in your app.js file. // assets/js/app.js import {Socket} from &quot;phoenix&quot; import {LiveSocket} from &quot;phoenix_live_view&quot; let csrfToken = document.querySelector(&quot;meta[name='csrf-token']&quot;).getAttribute(&quot;content&quot;) let liveSocket = new LiveSocket(&quot;/live&quot;, Socket, {params: {_csrf_token: csrfToken}}) // Connect if there are any LiveViews on the page liveSocket.connect() // Expose liveSocket on window for web console debug logs and latency simulation: // &gt;&gt; liveSocket.enableDebug() // &gt;&gt; liveSocket.enableLatencySim(1000) // The latency simulator is enabled for the duration of the browser session. // Call disableLatencySim() to disable: // &gt;&gt; liveSocket.disableLatencySim() window.liveSocket = liveSocket The JavaScript above expects phoenix_live_view to be available as a JavaScript dependency. Let's do that.","ref":"installation.html","title":"Installation","type":"extras"},{"doc":"If using npm , you need to add LiveView to your assets/package.json . For a regular project, do: { &quot;dependencies&quot;: { &quot;phoenix&quot;: &quot;file:../deps/phoenix&quot;, &quot;phoenix_html&quot;: &quot;file:../deps/phoenix_html&quot;, &quot;phoenix_live_view&quot;: &quot;file:../deps/phoenix_live_view&quot; } } However, if you're adding phoenix_live_view to an umbrella project, the dependency paths should be modified appropriately: { &quot;dependencies&quot;: { &quot;phoenix&quot;: &quot;file:../../../deps/phoenix&quot;, &quot;phoenix_html&quot;: &quot;file:../../../deps/phoenix_html&quot;, &quot;phoenix_live_view&quot;: &quot;file:../../../deps/phoenix_live_view&quot; } } Now run the next commands from your web app root: npm install --prefix assets If you had previously installed phoenix_live_view and want to get the latest javascript, then force an install with: npm install --force phoenix_live_view --prefix assets","ref":"installation.html#npm-dependencies","title":"Installation - npm dependencies","type":"extras"},{"doc":"LiveView does not use the default app layout. Instead, you typically call put_root_layout in your router to specify a layout that is used by both &quot;regular&quot; views and live views. In your router, do: pipeline :browser do ... plug :put_root_layout , { MyAppWeb.LayoutView , :root } ... end The layout given to put_root_layout must use &lt;%= @inner_content %&gt; instead of &lt;%= render(@view_module, @view_template, assigns) %&gt; . It is typically very barebones, with mostly &lt;head&gt; and &lt;body&gt; tags. For example: &lt; ! DOCTYPE html &gt; &lt; html lang = &quot;en&quot; &gt; &lt; head &gt; &lt; % = csrf_meta_tag ( ) % &gt; &lt; % = live_title_tag assigns [ :page_title ] || &quot;MyApp&quot; % &gt; &lt; link rel = &quot;stylesheet&quot; href = &quot;&lt;%= Routes.static_path(@conn, &quot; / css / app . css &quot;) %&gt;&quot; / &gt; &lt; script defer type = &quot;text/javascript&quot; src = &quot;&lt;%= Routes.static_path(@conn, &quot; / js / app . js &quot;) %&gt;&quot; &gt; &lt; / script &gt; &lt; / head &gt; &lt; body &gt; &lt; % = @inner_content % &gt; &lt; / body &gt; &lt; / html &gt; Once you have specified a root layout, &quot;app.html.heex&quot; will be rendered within your root layout for all non-LiveViews. You may also optionally define a &quot;live.html.heex&quot; layout to be used across all LiveViews, as we will describe in the next section. Optionally, you can add a phx-track-static to all script and link elements in your layout that use src and href . This way you can detect when new assets have been deployed by calling static_changed? . &lt; link phx - track - static rel = &quot;stylesheet&quot; href = { Routes . static_path ( @conn , &quot;/css/app.css&quot; ) } / &gt; &lt; script phx - track - static defer type = &quot;text/javascript&quot; src = { Routes . static_path ( @conn , &quot;/js/app.js&quot; ) } &gt; &lt; / script &gt;","ref":"installation.html#layouts","title":"Installation - Layouts","type":"extras"},{"doc":"While the above instructions are enough to install LiveView in a Phoenix app, if you want to use the phx.gen.live generators that come as part of Phoenix v1.5, you need to do one more change, as those generators assume your application was created with mix phx.new --live . The change is to define the live_view and live_component functions in your my_app_web.ex file, while refactoring the view function. At the end, they will look like this: def view do quote do use Phoenix.View , root : &quot;lib/&lt;%= lib_web_name %&gt;/templates&quot; , namespace : &lt; % = web_namespace % &gt; # Import convenience functions from controllers import Phoenix.Controller , only : [ get_flash : 1 , get_flash : 2 , view_module : 1 , view_template : 1 ] # Include shared imports and aliases for views unquote ( view_helpers ( ) ) end end def live_view do quote do use Phoenix.LiveView , layout : { &lt; % = web_namespace % &gt; . LayoutView , &quot;live.html&quot; } unquote ( view_helpers ( ) ) end end def live_component do quote do use Phoenix.LiveComponent unquote ( view_helpers ( ) ) end end defp view_helpers do quote do # Use all HTML functionality (forms, tags, etc) use Phoenix.HTML # Import LiveView helpers (live_render, live_component, live_patch, etc) import Phoenix.LiveView.Helpers # Import basic rendering functionality (render, render_layout, etc) import Phoenix.View import MyAppWeb.ErrorHelpers import MyAppWeb.Gettext alias MyAppWeb.Router.Helpers , as : Routes end end Note that LiveViews are automatically configured to use a &quot;live.html.heex&quot; layout in this line: use Phoenix.LiveView , layout : { &lt; % = web_namespace % &gt; . LayoutView , &quot;live.html&quot; } &quot;layouts/root.html.heex&quot; is shared by regular and live views, &quot;app.html.heex&quot; is rendered inside the root layout for regular views, and &quot;live.html.heex&quot; is rendered inside the root layout for LiveViews. &quot;live.html.heex&quot; typically starts out as a copy of &quot;app.html.heex&quot;, but using the @socket assign instead of @conn . Check the Live Layouts guide for more information.","ref":"installation.html#phx-gen-live-support","title":"Installation - phx.gen.live support","type":"extras"},{"doc":"If you want to show a progress bar as users perform live actions, we recommend using topbar . You can either add a copy of topbar to assets/vendor/topbar.js or add it as a npm dependency by calling: $ npm install --prefix assets --save topbar Then customize LiveView to use it in your assets/js/app.js , right before the liveSocket.connect() call: import topbar from &quot;topbar&quot; // Show progress bar on live navigation and form submits topbar.config({barColors: {0: &quot;#29d&quot;}, shadowColor: &quot;rgba(0, 0, 0, .3)&quot;}) window.addEventListener(&quot;phx:page-loading-start&quot;, info =&gt; topbar.show()) window.addEventListener(&quot;phx:page-loading-stop&quot;, info =&gt; topbar.hide())","ref":"installation.html#progress-animation","title":"Installation - Progress animation","type":"extras"},{"doc":"By convention your LiveView modules and heex templates should be placed in lib/my_app_web/live/ directory.","ref":"installation.html#location-for-liveview-modules","title":"Installation - Location for LiveView modules","type":"extras"},{"doc":"Assigns and HEEx templates All of the data in a LiveView is stored in the socket as assigns. The Phoenix.LiveView.assign/2 and Phoenix.LiveView.assign/3 functions help store those values. Those values can be accessed in the LiveView as socket.assigns.name but they are accessed inside LiveView templates as @name . Phoenix.LiveView 's built-in templates are identified by the .heex extension (HTML EEx) or ~H sigil. You can learn more about them by checking the docs for Phoenix.LiveView.Helpers.sigil_H/2 . They are an extension of regular .eex templates with additional features such as: validation of HTML friendly-syntax for defining HTML components minimize the amount of data sent over the wire by splitting static and dynamic parts provide change tracking to avoid recomputing parts of the template that did not change In this section, we are going to cover how LiveView minimizes the payload over the wire by understanding the interplay between assigns and templates.","ref":"assigns-eex.html","title":"Assigns and HEEx templates","type":"extras"},{"doc":"When you first render a .heex template, it will send all of the static and dynamic parts of the template to the client. Imagine the following template: &lt; h1 &gt; &lt; % = expand_title ( @title ) % &gt; &lt; / h1 &gt; It has two static parts, &lt;h1&gt; and &lt;/h1&gt; and one dynamic part made of expand_title(@title) . Further rendering of this template won't resend the static parts and it will only resend the dynamic part if it changes. The tracking of changes is done via assigns. If the @title assign changes, then LiveView will execute expand_title(@title) and send the new content. If @title is the same, nothing is executed and nothing is sent. Change tracking also works when accessing map/struct fields. Take this template: &lt; div id = { &quot;user_ \#{ @user . id } &quot; } &gt; &lt; % = @user . name % &gt; &lt; / div &gt; If the @user.name changes but @user.id doesn't, then LiveView will re-render only @user.name and it will not execute or resend @user.id at all. The change tracking also works when rendering other templates as long as they are also .heex templates: &lt; % = render &quot;child_template.html&quot; , assigns % &gt; Or when using function components: &lt; . show_name name = { @user . name } / &gt; The assign tracking feature also implies that you MUST avoid performing direct operations in the template. For example, if you perform a database query in your template: &lt; % = for user &lt;- Repo . all ( User ) do % &gt; &lt; % = user . name % &gt; &lt; % end % &gt; Then Phoenix will never re-render the section above, even if the number of users in the database changes. Instead, you need to store the users as assigns in your LiveView before it renders the template: assign ( socket , :users , Repo . all ( User ) ) Generally speaking, data loading should never happen inside the template , regardless if you are using LiveView or not. The difference is that LiveView enforces this best practice.","ref":"assigns-eex.html#change-tracking","title":"Assigns and HEEx templates - Change tracking","type":"extras"},{"doc":"There are two common pitfalls to keep in mind when using the ~H sigil or .heex templates inside LiveViews. When it comes to do/end blocks, change tracking is supported only on blocks given to Elixir's basic constructs, such as if , case , for , and similar. If the do/end block is given to a library function or user function, such as content_tag , change tracking won't work. For example, imagine the following template that renders a div : &lt; % = content_tag :div , id : &quot;user_ \#{ @id } &quot; do % &gt; &lt; % = @name % &gt; &lt; % = @description % &gt; &lt; % end % &gt; LiveView knows nothing about content_tag , which means the whole div will be sent whenever any of the assigns change. Luckily, HEEx templates provide a nice syntax for building tags, so there is rarely a need to use content_tag inside .heex templates: &lt; div id = { &quot;user_ \#{ @id } &quot; } &gt; &lt; % = @name % &gt; &lt; % = @description % &gt; &lt; / div &gt; The next pitfall is related to variables. Due to the scope of variables, LiveView has to disable change tracking whenever variables are used in the template, with the exception of variables introduced by Elixir basic case , for , and other block constructs. Therefore, you must avoid code like this in your LiveView templates: &lt; % some_var = @x + @y % &gt; &lt; % = some_var % &gt; Instead, use a function: &lt; % = sum ( @x , @y ) % &gt; Similarly, do not define variables at the top of your render function: def render ( assigns ) do sum = assigns . x + assigns . y ~H&quot;&quot;&quot; &lt;%= sum %&gt; &quot;&quot;&quot; end Instead explicitly precompute the assign in your LiveView, outside of render: assign ( socket , sum : socket . assigns . x + socket . assigns . y ) Generally speaking, avoid accessing variables inside LiveViews, as code that access variables is always executed on every render. This also applies to the assigns variable. The exception are variables introduced by Elixir's block constructs. For example, accessing the post variable defined by the comprehension below works as expected: &lt; % = for post &lt;- @posts do % &gt; ... &lt; % end % &gt; To sum up: Avoid passing block expressions to library and custom functions, instead prefer to use the conveniences in HEEx templates Avoid defining local variables, except within Elixir's constructs","ref":"assigns-eex.html#pitfalls","title":"Assigns and HEEx templates - Pitfalls","type":"extras"},{"doc":"Error and exception handling As with any other Elixir code, exceptions may happen during the LiveView life-cycle. In this section we will describe how LiveView reacts to errors at different stages.","ref":"error-handling.html","title":"Error and exception handling","type":"extras"},{"doc":"In this section, we will talk about error cases that you expect to happen within your application. For example, a user filling in a form with invalid data is expected. In a LiveView, we typically handle those cases by storing a change in the LiveView state, which causes the LiveView to be re-rendered with the error message. We may also use flash messages for this. For example, imagine you have a page to manage all &quot;Team members&quot; in an organization. However, if there is only one member left in the organization, they should not be allowed to leave. You may want to handle this by using flash messages: if MyApp.Org . leave ( socket . assigns . current_org , member ) do { :noreply , socket } else { :noreply , put_flash ( socket , :error , &quot;last member cannot leave organization&quot; ) } end However, one may argue that, if the last member of an organization cannot leave it, it may be better to not even show the &quot;Leave&quot; button in the UI when the organization has only one member. Given the button does not appear in the UI, triggering the &quot;leave&quot; when the organization has now only one member is an unexpected scenario. This means we can probably rewrite the code above to: true = MyApp.Org . leave ( socket . assigns . current_org , member ) { :noreply , socket } If leave returns false by any chance, it will just raise. Or you can even provide a leave! function that raises a specific exception: MyApp.Org . leave! ( socket . assigns . current_org , member ) { :noreply , socket } However, what will happen with a LiveView in case of exceptions? Let's talk about unexpected scenarios.","ref":"error-handling.html#expected-scenarios","title":"Error and exception handling - Expected scenarios","type":"extras"},{"doc":"Elixir developers tend to write assertive code. This means that, if we expect leave to always return true, we can explicitly match on its result, as we did above: true = MyApp.Org . leave ( socket . assigns . current_org , member ) { :noreply , socket } If leave fails and returns false , an exception is raised. It is common for Elixir developers to use exceptions for unexpected scenarios in their Phoenix applications. For example, if you are building an application where a user may belong to one or more organizations, when accessing the organization page, you may want to check that the user has access to it like this: organizations_query = Ecto . assoc ( socket . assigns . current_user , :organizations ) Repo . get! ( organizations_query , params [ &quot;org_id&quot; ] ) The code above builds a query that returns all organizations that belongs to the current user and then validates that the given &quot;org_id&quot; belongs to the user. If there is no such &quot;org_id&quot; or if the user has no access to it, an Ecto.NotFoundError exception is raised. During a regular controller request, this exception will be converted to a 404 exception and rendered as a custom error page, as detailed here . To understand how a LiveView reacts to exceptions, we need to consider two scenarios: exceptions on mount and during any event.","ref":"error-handling.html#unexpected-scenarios","title":"Error and exception handling - Unexpected scenarios","type":"extras"},{"doc":"Given the code on mount runs both on the initial disconnected render and the connected render, an exception on mount will trigger the following events: Exceptions during disconnected render: An exception on mount is caught and converted to an exception page by Phoenix error views - pretty much like the way it works with controllers Exceptions during connected render: An exception on mount will crash the LiveView process - which will be logged Once the client has noticed the crash during mount , it will fully reload the page Reloading the page will start a disconnected render, that will cause mount to be invoked again and most likely raise the same exception. Except this time it will be caught and converted to an exception page by Phoenix error views In other words, LiveView will reload the page in case of errors, making it fail as if LiveView was not involved in the rendering in the first place.","ref":"error-handling.html#exceptions-on-mount","title":"Error and exception handling - Exceptions on mount","type":"extras"},{"doc":"If the error happens during an event, the LiveView process will crash. The client will notice the error and remount the LiveView - without reloading the page. This is enough to update the page and show the user the latest information. For example, let's say two users try to leave the organization at the same time. In this case, both of them see the &quot;Leave&quot; button, but our leave function call will succeed only for one of them: true = MyApp.Org . leave ( socket . assigns . current_org , member ) { :noreply , socket } When the exception raises, the client will remount the LiveView. Once you remount, your code will now notice that there is only one user in the organization and therefore no longer show the &quot;Leave&quot; button. In other words, by remounting, we often update the state of the page, allowing exceptions to be automatically handled. Note that the choice between conditionally checking on the result of the leave function with an if , or simply asserting it returns true , is completely up to you. If the likelihood of everyone leaving the organization at the same time is low, then you may as well treat it as an unexpected scenario. Although other developers will be more comfortable by explicitly handling those cases. In both scenarios, LiveView has you covered.","ref":"error-handling.html#exceptions-on-events-handle_info-handle_event-etc","title":"Error and exception handling - Exceptions on events (handle_info, handle_event, etc)","type":"extras"},{"doc":"Live layouts NOTE: Make sure you've read the Assigns and LiveEEx templates guide before moving forward. When working with LiveViews, there are usually three layouts to be considered: the root layout - this is a layout used by both LiveView and regular views. This layout typically contains the &lt;html&gt; definition alongside the head and body tags. Any content defined in the root layout will remain the same, even as you live navigate across LiveViews. All LiveViews defined at the router must have a root layout. The root layout is typically declared on the router with put_root_layout and defined as &quot;root.html.heex&quot; in your MyAppWeb.LayoutView . It may also be given via the :layout option to the router's live macro. the app layout - this is the default application layout which is not included or used by LiveViews. It defaults to &quot;app.html.heex&quot; in your MyAppWeb.LayoutView . the live layout - this is the layout which wraps a LiveView and is rendered as part of the LiveView life-cycle. It must be opt-in by passing the :layout option on use Phoenix.LiveView . It is typically set to &quot;live.html.heex&quot;in your MyAppWeb.LayoutView . Overall, those layouts are found in templates/layout with the following names: * root . html . heex * app . html . heex * live . html . heex Note: if you are using earlier Phoenix versions, those layouts may use .eex and .leex extensions instead of .heex , but we have since then normalized on the latter. All layouts must call &lt;%= @inner_content %&gt; to inject the content rendered by the layout. The &quot;root&quot; layout is shared by both &quot;app&quot; and &quot;live&quot; layouts. It is rendered only on the initial request and therefore it has access to the @conn assign. The root layout must be defined in your router: plug :put_root_layout , { MyAppWeb.LayoutView , :root } The &quot;app&quot; and &quot;live&quot; layouts are often small and similar to each other, but the &quot;app&quot; layout uses the @conn and is used as part of the regular request life-cycle. The &quot;live&quot; layout is part of the LiveView and therefore has direct access to the @socket . For example, you can define a new live.html.heex layout with dynamic content. You must use @inner_content where the output of the actual template will be placed at: &lt; p &gt; &lt; % = live_flash ( @flash , :notice ) % &gt; &lt; / p &gt; &lt; p &gt; &lt; % = live_flash ( @flash , :error ) % &gt; &lt; / p &gt; &lt; % = @inner_content % &gt; To use the live layout, update your LiveView to pass the :layout option to use Phoenix.LiveView : use Phoenix.LiveView , layout : { MyAppWeb.LayoutView , &quot;live.html&quot; } If you are using Phoenix v1.5, the layout is automatically set when generating apps with the mix phx.new --live flag. The :layout option on use does not apply to LiveViews rendered within other LiveViews. If you want to render child live views or opt-in to a layout, use :layout as an option in mount: def mount ( _params , _session , socket ) do socket = assign ( socket , new_message_count : 0 ) { :ok , socket , layout : { MyAppWeb.LayoutView , &quot;live.html&quot; } } end Note : The live layout is always wrapped by the LiveView's :container tag.","ref":"live-layouts.html","title":"Live layouts","type":"extras"},{"doc":"Because the root layout from the Plug pipeline is rendered outside of LiveView, the contents cannot be dynamically changed. The one exception is the &lt;title&gt; of the HTML document. Phoenix LiveView special cases the @page_title assign to allow dynamically updating the title of the page, which is useful when using live navigation, or annotating the browser tab with a notification. For example, to update the user's notification count in the browser's title bar, first set the page_title assign on mount: def mount ( _params , _session , socket ) do socket = assign ( socket , page_title : &quot;Latest Posts&quot; ) { :ok , socket } end Then access @page_title in the root layout: &lt; title &gt; &lt; % = @page_title % &gt; &lt; / title &gt; You can also use Phoenix.LiveView.Helpers.live_title_tag/2 to support adding automatic prefix and suffix to the page title when rendered and on subsequent updates: &lt; % = live_title_tag assigns [ :page_title ] || &quot;Welcome&quot; , prefix : &quot;MyApp – &quot; % &gt; Although the root layout is not updated by LiveView, by simply assigning to page_title , LiveView knows you want the title to be updated: def handle_info ( { :new_messages , count } , socket ) do { :noreply , assign ( socket , page_title : &quot;Latest Posts ( \#{ count } new)&quot; ) } end Note : If you find yourself needing to dynamically patch other parts of the base layout, such as injecting new scripts or styles into the &lt;head&gt; during live navigation, then a regular, non-live, page navigation should be used instead . Assigning the @page_title updates the document.title directly, and therefore cannot be used to update any other part of the base layout.","ref":"live-layouts.html#updating-the-html-document-title","title":"Live layouts - Updating the HTML document title","type":"extras"},{"doc":"Live navigation LiveView provides functionality to allow page navigation using the browser's pushState API . With live navigation, the page is updated without a full page reload. You can trigger live navigation in two ways: From the client - this is done by replacing Phoenix.HTML.Link.link/2 by Phoenix.LiveView.Helpers.live_patch/2 or Phoenix.LiveView.Helpers.live_redirect/2 From the server - this is done by replacing Phoenix.Controller.redirect/2 calls by Phoenix.LiveView.push_patch/2 or Phoenix.LiveView.push_redirect/2 . For example, in a template you may write: &lt; % = live_patch &quot;next&quot; , to : Routes . live_path ( @socket , MyLive , @page + 1 ) % &gt; or in a LiveView: { :noreply , push_patch ( socket , to : Routes . live_path ( socket , MyLive , page + 1 ) ) } The &quot;patch&quot; operations must be used when you want to navigate to the current LiveView, simply updating the URL and the current parameters, without mounting a new LiveView. When patch is used, the handle_params/3 callback is invoked and the minimal set of changes are sent to the client. See the next section for more information. The &quot;redirect&quot; operations must be used when you want to dismount the current LiveView and mount a new one. In those cases, an Ajax request is made to fetch the necessary information about the new LiveView, which is mounted in place of the current one within the current layout. While redirecting, a phx-disconnected class is added to the LiveView, which can be used to indicate to the user a new page is being loaded. At the end of the day, regardless if you invoke link/2 , live_patch/2 , and live_redirect/2 from the client, or redirect/2 , push_patch/2 , and push_redirect/2 from the server, the user will end-up on the same page. The difference between those is mostly the amount of data sent over the wire: link/2 and redirect/2 do full page reloads live_redirect/2 and push_redirect/2 mounts a new LiveView while keeping the current layout live_patch/2 and push_patch/2 updates the current LiveView and sends only the minimal diff while also maintaining the scroll position An easy rule of thumb is to stick with live_redirect/2 and push_redirect/2 and use the patch helpers only in the cases where you want to minimize the amount of data sent when navigating within the same LiveView (for example, if you want to change the sorting of a table while also updating the URL).","ref":"live-navigation.html","title":"Live navigation","type":"extras"},{"doc":"The handle_params/3 callback is invoked after mount/3 and before the initial render. It is also invoked every time live_patch/2 or push_patch/2 are used. It receives the request parameters as first argument, the url as second, and the socket as third. For example, imagine you have a UserTable LiveView to show all users in the system and you define it in the router as: live &quot;/users&quot; , UserTable Now to add live sorting, you could do: &lt; % = live_patch &quot;Sort by name&quot; , to : Routes . live_path ( @socket , UserTable , %{ sort_by : &quot;name&quot; } ) % &gt; When clicked, since we are navigating to the current LiveView, handle_params/3 will be invoked. Remember you should never trust the received params, so you must use the callback to validate the user input and change the state accordingly: def handle_params ( params , _uri , socket ) do socket = case params [ &quot;sort_by&quot; ] do sort_by when sort_by in ~w(name company) -&gt; assign ( socket , sort_by : sort ) _ -&gt; socket end { :noreply , load_users ( socket ) } end As with other handle_* callbacks, changes to the state inside handle_params/3 will trigger a server render. Note the parameters given to handle_params/3 are the same as the ones given to mount/3 . So how do you decide which callback to use to load data? Generally speaking, data should always be loaded on mount/3 , since mount/3 is invoked once per LiveView life-cycle. Only the params you expect to be changed via live_patch/2 or push_patch/2 must be loaded on handle_params/3 . For example, imagine you have a blog. The URL for a single post is: &quot;/blog/posts/:post_id&quot;. In the post page, you have comments and they are paginated. You use live_patch/2 to update the shown comments every time the user paginates, updating the URL to &quot;/blog/posts/:post_id?page=X&quot;. In this example, you will access &quot;post_id&quot; on mount/3 and the page of comments on handle_params/3 . Furthermore, it is very important to not access the same parameters on both mount/3 and handle_params/3 . For example, do NOT do this: def mount ( %{ &quot;post_id&quot; =&gt; post_id } , session , socket ) do # do something with post_id end def handle_params ( %{ &quot;post_id&quot; =&gt; post_id , &quot;page&quot; =&gt; page } , url , socket ) do # do something with post_id and page end If you do that, because mount/3 is called once and handle_params/3 multiple times, the &quot;post_id&quot; read on mount can get out of sync with the one in handle_params/3 . So once a parameter is read on mount, it should not be read elsewhere. Instead, do this: def mount ( %{ &quot;post_id&quot; =&gt; post_id } , session , socket ) do # do something with post_id end def handle_params ( %{ &quot;sort_by&quot; =&gt; sort_by } , url , socket ) do post_id = socket . assigns . post . id # do something with sort_by end","ref":"live-navigation.html#handle_params-3","title":"Live navigation - handle_params/3","type":"extras"},{"doc":"LiveView also allows the current browser URL to be replaced. This is useful when you want certain events to change the URL but without polluting the browser's history. This can be done by passing the replace: true option to any of the navigation helpers.","ref":"live-navigation.html#replace-page-address","title":"Live navigation - Replace page address","type":"extras"},{"doc":"LiveView allows you to have multiple LiveViews in the same page by calling Phoenix.LiveView.Helpers.live_render/3 in your templates. However, only the LiveViews defined directly in your router can use the &quot;Live Navigation&quot; functionality described here. This is important because LiveViews work closely with your router, guaranteeing you can only navigate to known routes.","ref":"live-navigation.html#multiple-liveviews-in-the-same-page","title":"Live navigation - Multiple LiveViews in the same page","type":"extras"},{"doc":"Security considerations of the LiveView model LiveView begins its life-cycle as a regular HTTP request. Then a stateful connection is established. Both the HTTP request and the stateful connection receives the client data via parameters and session. This means that any session validation must happen both in the HTTP request (plug pipeline) and the stateful connection (LiveView mount).","ref":"security-model.html","title":"Security considerations of the LiveView model","type":"extras"},{"doc":"When speaking about security, there are two terms commonly used: authentication and authorization. Authentication is about identifying a user. Authorization is about telling if a user has access to a certain resource or feature in the system. In a regular web application, once a user is authenticated, for example by entering his email and password, or by using a third-party service such as Google, Twitter, or Facebook, a token identifying the user is stored in the session, which is a cookie (a key-value pair) stored in the user's browser. Every time there is a request, we read the value from the session, and, if valid, we fetch the user stored in the session from the database. The session is automatically validated by Phoenix and tools like mix phx.gen.auth can generate the building blocks of an authentication system for you. Once the user is authenticated, they may perform many actions on the page, and some of those actions require specific permissions. This is called authorization and the specific rules often change per application. In a regular web application, we perform authentication and authorization checks on every request. In LiveView, we should also run those exact same checks, always. Once the user is authenticated, we typically validate the sessions on the mount callback. Authorization rules generally happen on mount (for instance, is the user allowed to see this page?) and also on handle_event (is the user allowed to delete this item?).","ref":"security-model.html#authentication-vs-authorization","title":"Security considerations of the LiveView model - Authentication vs authorization","type":"extras"},{"doc":"If you perform user authentication and confirmation on every HTTP request via Plugs, such as this: plug :ensure_user_authenticated plug :ensure_user_confirmed Then the mount/3 callback of your LiveView should execute those same verifications: def mount ( params , %{ &quot;user_id&quot; =&gt; user_id } = _session , socket ) do socket = assign ( socket , current_user : Accounts . get_user! ( user_id ) ) socket = if socket . assigns . current_user . confirmed_at do socket else redirect ( socket , to : &quot;/login&quot; ) end { :ok , socket } end LiveView v0.16 includes the on_mount ( Phoenix.LiveView.on_mount/1 ) hook, which allows you to encapsulate this logic and execute it on every mount, as you would with plug: defmodule MyAppWeb.UserLiveAuth do import Phoenix.LiveView def mount ( params , %{ &quot;user_id&quot; =&gt; user_id } = _session , socket ) do socket = assign_new ( socket , :current_user , fn -&gt; Accounts . get_user! ( user_id ) end ) if socket . assigns . current_user . confirmed_at do { :cont , socket } else { :halt , redirect ( socket , to : &quot;/login&quot; ) } end end end and then use it on all relevant LiveViews: defmodule MyAppWeb.PageLive do use Phoenix.LiveView on_mount MyAppWeb.UserLiveAuth ... end Note in the snippet above we used assign_new/3 . This is a convenience to avoid fetching the current_user multiple times across LiveViews.","ref":"security-model.html#mounting-considerations","title":"Security considerations of the LiveView model - Mounting considerations","type":"extras"},{"doc":"Every time the user performs an action on your system, you should verify if the user is authorized to do so, regardless if you are using LiveViews or not. For example, imagine a user can see all projects in a web application, but they may not have permission to delete any of them. At the UI level, you handle this accordingly by not showing the delete button in the projects listing, but a savvy user can directly talk to the server and request a deletion anyway. For this reason, you must always verify permissions on the server . In LiveView, most actions are handled by the handle_event callback. Therefore, you typically authorize the user within those callbacks. In the scenario just described, one might implement this: on_mount MyAppWeb.UserLiveAuth def mount ( _params , session , socket ) do { :ok , load_projects ( socket ) } end def handle_event ( &quot;delete_project&quot; , %{ &quot;project_id&quot; =&gt; project_id } , socket ) do Project . delete! ( socket . assigns . current_user , project_id ) { :noreply , assign ( socket , post : updated_post ) } end defp load_projects ( socket ) do projects = Project . all_projects ( socket . assigns . current_user ) assign ( socket , projects : projects ) end First, we used on_mount to authenticate the user based on the data stored in the session. Then we load all projects based on the authenticated user. Now, whenever there is a request to delete a project, we still pass the current user as argument to the Project context, so it verifies if the user is allowed to delete it or not. In case it cannot delete, it is fine to just raise an exception. After all, users are not meant to trigger this code path anyway (unless they are fiddling with something they are not supposed to!).","ref":"security-model.html#events-considerations","title":"Security considerations of the LiveView model - Events considerations","type":"extras"},{"doc":"So far, the security model between LiveView and regular web applications have been remarkably similar. After all, we must always authenticate and authorize every user. The main difference between them happens on logout or when revoking access. Because LiveView is a permanent connection between client and server, if a user is logged out, or removed from the system, this change won't reflect on the LiveView part unless the user reloads the page. Luckily, it is possible to address this by setting a live_socket_id in the session. For example, when logging in a user, you could do: conn |&gt; put_session ( :current_user_id , user . id ) |&gt; put_session ( :live_socket_id , &quot;users_socket: \#{ user . id } &quot; ) Now all LiveView sockets will be identified and listen to the given live_socket_id . You can then disconnect all live users identified by said ID by broadcasting on the topic: MyAppWeb.Endpoint . broadcast ( &quot;users_socket: \#{ user . id } &quot; , &quot;disconnect&quot; , %{ } ) Note: If you use mix phx.gen.auth to generate your authentication system, those lines are already present in the generated code. Once a LiveView is disconnected, the client will attempt to reestablish the connection and re-execute the mount/3 callback. In this case, if the user is no longer logged in or it no longer has access to the current resource, mount/3 will fail and the user will be redirected. This is the same mechanism provided by Phoenix.Channel s. Therefore, if your application uses both channels and LiveViews, you can use the same technique to disconnect any stateful connection.","ref":"security-model.html#disconnecting-all-instances-of-a-live-user","title":"Security considerations of the LiveView model - Disconnecting all instances of a live user","type":"extras"},{"doc":"LiveView supports live redirect, which allows users to navigate between pages over the LiveView connection. Whenever there is a live_redirect , a new LiveView will be mounted, skipping the regular HTTP requests and without going through the plug pipeline. However, if you want to draw stronger boundaries between parts of your application, you can also use Phoenix.LiveView.Router.live_session/2 to group your live routes. This can be handy because you can only live_redirect between LiveViews in the same live_session . For example, imagine you need to authenticate two distinct types of users. Your regular users login via email and password, and you have an admin dashboard that uses http auth. You can specify different live_session s for each authentication flow: live_session :default do scope &quot;/&quot; do pipe_through [ :authenticate_user ] get ... live ... end end live_session :admin do scope &quot;/admin&quot; do pipe_through [ :http_auth_admin ] get ... live ... end end Now every time you try to navigate to an admin panel, and out of it, a regular page navigation will happen and a brand new live connection will be established. Once again, it is worth remembering that LiveViews require their own security checks, so we use pipe_through above to protect the regular routes (get, post, etc.) and the LiveViews should run their own checks using on_mount hooks. live_session can also be used to enforce each LiveView group has a different root layout, since layouts are not updated between live redirects: live_session :default , root_layout : { LayoutView , &quot;app.html&quot; } do ... end live_session :admin , root_layout : { LayoutView , &quot;admin.html&quot; } do ... end Finally, you can even combine live_session with on_mount . Instead of declaring on_mount on every LiveView, you can declare it at the router level and it will enforce it on all LiveViews under it: live_session :default , on_mount : MyAppWeb.UserLiveAuth do scope &quot;/&quot; do pipe_through [ :authenticate_user ] live ... end end live_session :admin , on_mount : MyAppWeb.AdminLiveAuth do scope &quot;/admin&quot; do pipe_through [ :authenticate_admin ] live ... end end Each live route under the :default live_session will invoke the MyAppWeb.UserLiveAuth hook on mount. This module was defined earlier in this guide. We will also pipe regular web requests through :authenticate_user , which must execute the same checks as MyAppWeb.UserLiveAuth , but tailored to plug. Similarly, the :admin live_session has its own authentication flow, powered by MyAppWeb.AdminLiveAuth . It also defines a plug equivalent named :authenticate_admin , which will be used by any regular request. If there are no regular web requests defined under a live session, then the pipe_through checks are not necessary. Declaring the on_mount on live_session is exactly the same as declaring it in each LiveView inside the live_session . It will be executed every time a LiveView is mounted, even after live_redirect s. The important to keep in mind is: If you have both LiveViews and regular web requests, then you must always authorize and authenticate your LiveViews (using on mount hooks) and your web requests (using plugs) All actions (events) must also be explicitly authorized by checking permissions. Those permissions are often domain/business specific, and typically happen in your context modules live_session can be used to draw boundaries between groups of LiveViews. While you could use live_session to draw lines between different authorization rules, doing so would lead to frequent page reloads. For this reason, we typically use live_session to enforce different authentication requirements or whenever you need to change root layouts","ref":"security-model.html#live_session-and-live_redirect","title":"Security considerations of the LiveView model - live_session and live_redirect","type":"extras"},{"doc":"Telemetry LiveView currently exposes the following telemetry events: [:phoenix, :live_view, :mount, :start] - Dispatched by a Phoenix.LiveView immediately before mount/3 is invoked. Measurement: %{ system_time : System . monotonic_time } Metadata: %{ socket : Phoenix.LiveView.Socket . t , params : unsigned_params | :not_mounted_at_router , session : map } [:phoenix, :live_view, :mount, :stop] - Dispatched by a Phoenix.LiveView when the mount/3 callback completes successfully. Measurement: %{ duration : native_time } Metadata: %{ socket : Phoenix.LiveView.Socket . t , params : unsigned_params | :not_mounted_at_router , session : map } [:phoenix, :live_view, :mount, :exception] - Dispatched by a Phoenix.LiveView when an exception is raised in the mount/3 callback. Measurement: %{duration: native_time} Metadata: %{ socket : Phoenix.LiveView.Socket . t , kind : atom , reason : term , params : unsigned_params | :not_mounted_at_router , session : map } [:phoenix, :live_view, :handle_params, :start] - Dispatched by a Phoenix.LiveView immediately before handle_params/3 is invoked. Measurement: %{ system_time : System . monotonic_time } Metadata: %{ socket : Phoenix.LiveView.Socket . t , params : unsigned_params , uri : String . t ( ) } [:phoenix, :live_view, :handle_params, :stop] - Dispatched by a Phoenix.LiveView when the handle_params/3 callback completes successfully. Measurement: %{ duration : native_time } Metadata: %{ socket : Phoenix.LiveView.Socket . t , params : unsigned_params , uri : String . t ( ) } [:phoenix, :live_view, :handle_params, :exception] - Dispatched by a Phoenix.LiveView when the when an exception is raised in the handle_params/3 callback. Measurement: %{ duration : native_time } Metadata: %{ socket : Phoenix.LiveView.Socket . t , kind : atom , reason : term , params : unsigned_params , uri : String . t ( ) } [:phoenix, :live_view, :handle_event, :start] - Dispatched by a Phoenix.LiveView immediately before handle_event/3 is invoked. Measurement: %{ system_time : System . monotonic_time } Metadata: %{ socket : Phoenix.LiveView.Socket . t , event : String . t ( ) , params : unsigned_params } [:phoenix, :live_view, :handle_event, :stop] - Dispatched by a Phoenix.LiveView when the handle_event/3 callback completes successfully. Measurement: %{ duration : native_time } Metadata: %{ socket : Phoenix.LiveView.Socket . t , event : String . t ( ) , params : unsigned_params } [:phoenix, :live_view, :handle_event, :exception] - Dispatched by a Phoenix.LiveView when an exception is raised in the handle_event/3 callback. Measurement: %{ duration : native_time } Metadata: %{ socket : Phoenix.LiveView.Socket . t , kind : atom , reason : term , event : String . t ( ) , params : unsigned_params } [:phoenix, :live_component, :handle_event, :start] - Dispatched by a Phoenix.LiveComponent immediately before handle_event/3 is invoked. Measurement: %{ system_time : System . monotonic_time } Metadata: %{ socket : Phoenix.LiveView.Socket . t , component : atom , event : String . t ( ) , params : unsigned_params } [:phoenix, :live_component, :handle_event, :stop] - Dispatched by a Phoenix.LiveComponent when the handle_event/3 callback completes successfully. Measurement: %{ duration : native_time } Metadata: %{ socket : Phoenix.LiveView.Socket . t , component : atom , event : String . t ( ) , params : unsigned_params } [:phoenix, :live_component, :handle_event, :exception] - Dispatched by a Phoenix.LiveComponent when an exception is raised in the handle_event/3 callback. Measurement: %{ duration : native_time } Metadata: %{ socket : Phoenix.LiveView.Socket . t , kind : atom , reason : term , component : atom , event : String . t ( ) , params : unsigned_params }","ref":"telemetry.html","title":"Telemetry","type":"extras"},{"doc":"Uploads LiveView supports interactive file uploads with progress for both direct to server uploads as well as direct-to-cloud external uploads on the client. Built-in Features Accept specification - Define accepted file types, max number of entries, max file size, etc. When the client selects file(s), the file metadata is automatically validated against the specification. See Phoenix.LiveView.allow_upload/3 . Reactive entries - Uploads are populated in an @uploads assign in the socket. Entries automatically respond to progress, errors, cancellation, etc. Drag and drop - Use the phx-drop-target attribute to enable. See Phoenix.LiveView.Helpers.live_file_input/2 .","ref":"uploads.html","title":"Uploads","type":"extras"},{"doc":"You enable an upload, typically on mount, via allow_upload/3 : @impl Phoenix.LiveView def mount ( _params , _session , socket ) do { :ok , socket |&gt; assign ( :uploaded_files , [ ] ) |&gt; allow_upload ( :avatar , accept : ~w(.jpg .jpeg) , max_entries : 2 ) } end That's it for now! We will come back to the LiveView to implement some form- and upload-related callbacks later, but most of the functionality around uploads takes place in the template.","ref":"uploads.html#allow-uploads","title":"Uploads - Allow uploads","type":"extras"},{"doc":"Use the Phoenix.LiveView.Helpers.live_file_input/2 file input generator to render a file input for the upload: # lib/my_app_web/live/upload_live.html.heex &lt; form id = &quot;upload-form&quot; phx - submit = &quot;save&quot; phx - change = &quot;validate&quot; &gt; &lt; % = live_file_input @uploads . avatar % &gt; &lt; button type = &quot;submit&quot; &gt; Upload &lt; / button &gt; &lt; / form &gt; Important: You must bind phx-submit and phx-change on the form. Note that while live_file_input/2 allows you to set additional attributes on the file input, many attributes such as id , accept , and multiple will be set automatically based on the allow_upload/3 spec. Reactive updates to the template will occur as the end-user interacts with the file input. Upload entries Uploads are populated in an @uploads assign in the socket. Each allowed upload contains a list of entries, irrespective of the :max_entries value in the allow_upload/3 spec. These entry structs contain all the information about an upload, including progress, client file info, errors, etc. Let's look at an annotated example: # lib/my_app_web/live/upload_live.html.heex &lt; % # use phx-drop-target with the upload ref to enable file drag and drop %&gt; &lt; section phx - drop - target = { @uploads . avatar . ref } &gt; &lt; % # render each avatar entry %&gt; &lt; % = for entry &lt;- @uploads . avatar . entries do % &gt; &lt; article class = &quot;upload-entry&quot; &gt; &lt; figure &gt; &lt; % # Phoenix.LiveView.Helpers.live_img_preview/2 renders a client-side preview %&gt; &lt; % = live_img_preview entry % &gt; &lt; figcaption &gt; &lt; % = entry . client_name % &gt; &lt; / figcaption &gt; &lt; / figure &gt; &lt; % # entry.progress will update automatically for in-flight entries %&gt; &lt; progress value = { entry . progress } max = &quot;100&quot; &gt; &lt; % = entry . progress % &gt; % &lt; / progress &gt; &lt; % # a regular click event whose handler will invoke Phoenix.LiveView.cancel_upload/3 %&gt; &lt; button phx - click = &quot;cancel-upload&quot; phx - value - ref = { entry . ref } aria - label = &quot;cancel&quot; &gt; &amp; times ; &lt; / button &gt; &lt; % # Phoenix.LiveView.Helpers.upload_errors/2 returns a list of error atoms %&gt; &lt; % = for err &lt;- upload_errors ( @uploads . avatar , entry ) do % &gt; &lt; p class = &quot;alert alert-danger&quot; &gt; &lt; % = error_to_string ( err ) % &gt; &lt; / p &gt; &lt; % end % &gt; &lt; / article &gt; &lt; % end % &gt; &lt; / section &gt; The section element in the example acts as the phx-drop-target for the :avatar upload. Users can interact with the file input or they can drop files over the element to add new entries. Upload entries are created when a file is added to the form input and each will exist until it has been consumed, following a successfully completed upload. Entry validation Validation occurs automatically based on any conditions that were specified in allow_upload/3 however, as mentioned previously you are required to bind phx-change on the form in order for the validation to be performed. Therefore you must implement at least a minimal callback: @impl Phoenix.LiveView def handle_event ( &quot;validate&quot; , _params , socket ) do { :noreply , socket } end Entries for files that do not match the allow_upload/3 spec will contain errors. Use Phoenix.LiveView.Helpers.upload_errors/2 and your own helper function to render a friendly error message: def error_to_string ( :too_large ) , do : &quot;Too large&quot; def error_to_string ( :too_many_files ) , do : &quot;You have selected too many files&quot; def error_to_string ( :not_accepted ) , do : &quot;You have selected an unacceptable file type&quot; Cancel an entry Upload entries may also be canceled, either programmatically or as a result of a user action. For instance, to handle the click event in the template above, you could do the following: @impl Phoenix.LiveView def handle_event ( &quot;cancel-upload&quot; , %{ &quot;ref&quot; =&gt; ref } , socket ) do { :noreply , cancel_upload ( socket , :avatar , ref ) } end","ref":"uploads.html#render-reactive-elements","title":"Uploads - Render reactive elements","type":"extras"},{"doc":"When the end-user submits a form containing a live_file_input/2 , the JavaScript client first uploads the file(s) before invoking the callback for the form's phx-submit event. Within the callback for the phx-submit event, you invoke the Phoenix.LiveView.consume_uploaded_entries/3 function to process the completed uploads, persisting the relevant upload data alongside the form data: @impl Phoenix.LiveView def handle_event ( &quot;save&quot; , _params , socket ) do uploaded_files = consume_uploaded_entries ( socket , :avatar , fn %{ path : path } , _entry -&gt; dest = Path . join ( [ :code . priv_dir ( :my_app ) , &quot;static&quot; , &quot;uploads&quot; , Path . basename ( path ) ] ) File . cp! ( path , dest ) Routes . static_path ( socket , &quot;/uploads/ \#{ Path . basename ( dest ) } &quot; ) end ) { :noreply , update ( socket , :uploaded_files , &amp; ( &amp;1 ++ uploaded_files ) ) } end Note : While client metadata cannot be trusted, max file size validations are enforced as each chunk is received when performing direct to server uploads. For more information on implementing client-side, direct-to-cloud uploads, see the External Uploads guide .","ref":"uploads.html#consume-uploaded-entries","title":"Uploads - Consume uploaded entries","type":"extras"},{"doc":"A complete example of the LiveView from this guide: # lib/my_app_web/live/upload_live.ex defmodule MyAppWeb.UploadLive do use MyAppWeb , :live_view @impl Phoenix.LiveView def mount ( _params , _session , socket ) do { :ok , socket |&gt; assign ( :uploaded_files , [ ] ) |&gt; allow_upload ( :avatar , accept : ~w(.jpg .jpeg) , max_entries : 2 ) } end @impl Phoenix.LiveView def handle_event ( &quot;validate&quot; , _params , socket ) do { :noreply , socket } end @impl Phoenix.LiveView def handle_event ( &quot;cancel-upload&quot; , %{ &quot;ref&quot; =&gt; ref } , socket ) do { :noreply , cancel_upload ( socket , :avatar , ref ) } end @impl Phoenix.LiveView def handle_event ( &quot;save&quot; , _params , socket ) do uploaded_files = consume_uploaded_entries ( socket , :avatar , fn %{ path : path } , _entry -&gt; dest = Path . join ( [ :code . priv_dir ( :my_app ) , &quot;static&quot; , &quot;uploads&quot; , Path . basename ( path ) ] ) File . cp! ( path , dest ) Routes . static_path ( socket , &quot;/uploads/ \#{ Path . basename ( dest ) } &quot; ) end ) { :noreply , update ( socket , :uploaded_files , &amp; ( &amp;1 ++ uploaded_files ) ) } end defp error_to_string ( :too_large ) , do : &quot;Too large&quot; defp error_to_string ( :too_many_files ) , do : &quot;You have selected too many files&quot; defp error_to_string ( :not_accepted ) , do : &quot;You have selected an unacceptable file type&quot; end","ref":"uploads.html#appendix-a-uploadlive","title":"Uploads - Appendix A: UploadLive","type":"extras"},{"doc":"Using Gettext for internationalization For internationalization with gettext , the locale used within your Plug pipeline can be stored in the Plug session and restored within your LiveView mount. For example, after user signs in or preference changes, you can write the locale to the session: def put_user_session ( conn , current_user ) do locale = get_locale_for_user ( current_user ) Gettext . put_locale ( MyApp.Gettext , locale ) conn |&gt; put_session ( :user_id , current_user . id ) |&gt; put_session ( :locale , locale ) end Then in your LiveView mount/3 , you can restore the locale: def mount ( _params , %{ &quot;locale&quot; =&gt; locale } , socket ) do Gettext . put_locale ( MyApp.Gettext , locale ) { :ok , socket } end","ref":"using-gettext.html","title":"Using Gettext for internationalization","type":"extras"},{"doc":"Bindings Phoenix supports DOM element bindings for client-server interaction. For example, to react to a click on a button, you would render the element: &lt; button phx - click = &quot;inc_temperature&quot; &gt; + &lt; / button &gt; Then on the server, all LiveView bindings are handled with the handle_event callback, for example: def handle_event ( &quot;inc_temperature&quot; , _value , socket ) do { :ok , new_temp } = Thermostat . inc_temperature ( socket . assigns . id ) { :noreply , assign ( socket , :temperature , new_temp ) } end Binding Attributes Params phx-value-* Click Events phx-click , phx-capture-click Form Events phx-change , phx-submit , phx-feedback-for , phx-disable-with , phx-trigger-action , phx-auto-recover Focus/Blur Events phx-blur , phx-focus , phx-window-blur , phx-window-focus Key Events phx-keydown , phx-keyup , phx-window-keydown , phx-window-keyup , phx-key DOM Patching phx-update JS Interop phx-hook Rate Limiting phx-debounce , phx-throttle Static tracking phx-track-static Loading states phx-page-loading","ref":"bindings.html","title":"Bindings","type":"extras"},{"doc":"The phx-click binding is used to send click events to the server. When any client event, such as a phx-click click is pushed, the value sent to the server will be chosen with the following priority: Any number of optional phx-value- prefixed attributes, such as: &lt; div phx - click = &quot;inc&quot; phx - value - myvar1 = &quot;val1&quot; phx - value - myvar2 = &quot;val2&quot; &gt; will send the following map of params to the server: def handle_event ( &quot;inc&quot; , %{ &quot;myvar1&quot; =&gt; &quot;val1&quot; , &quot;myvar2&quot; =&gt; &quot;val2&quot; } , socket ) do If the phx-value- prefix is used, the server payload will also contain a &quot;value&quot; if the element's value attribute exists. When receiving a map on the server, the payload will also include user defined metadata of the client event, or an empty map if none is set. For example, the following LiveSocket client option would send the coordinates and altKey information for all clicks: let liveSocket = new LiveSocket ( &quot;/live&quot; , Socket , { params : { _csrf_token : csrfToken } , metadata : { click : ( e , el ) =&gt; { return { altKey : e . altKey , clientX : e . clientX , clientY : e . clientY } } } } ) The phx-capture-click event is just like phx-click , but instead of the click event being dispatched to the closest phx-click element as it bubbles up through the DOM, the event is dispatched as it propagates from the top of the DOM tree down to the target element. This is useful when wanting to bind click events without receiving bubbled events from child UI elements. Since capturing happens before bubbling, this can also be important for preparing or preventing behaviour that will be applied during the bubbling phase.","ref":"bindings.html#click-events","title":"Bindings - Click Events","type":"extras"},{"doc":"Focus and blur events may be bound to DOM elements that emit such events, using the phx-blur , and phx-focus bindings, for example: &lt; input name = &quot;email&quot; phx - focus = &quot;myfocus&quot; phx - blur = &quot;myblur&quot; / &gt; To detect when the page itself has received focus or blur, phx-window-focus and phx-window-blur may be specified. These window level events may also be necessary if the element in consideration (most often a div with no tabindex) cannot receive focus. Like other bindings, phx-value-* can be provided on the bound element, and those values will be sent as part of the payload. For example: &lt; div class = &quot;container&quot; phx - window - focus = &quot;page-active&quot; phx - window - blur = &quot;page-inactive&quot; phx - value - page = &quot;123&quot; &gt; ... &lt; / div &gt; The following window-level bindings are supported: phx-window-focus phx-window-blur phx-window-keydown phx-window-keyup","ref":"bindings.html#focus-and-blur-events","title":"Bindings - Focus and Blur Events","type":"extras"},{"doc":"The onkeydown , and onkeyup events are supported via the phx-keydown , and phx-keyup bindings. Each binding supports a phx-key attribute, which triggers the event for the specific key press. If no phx-key is provided, the event is triggered for any key press. When pushed, the value sent to the server will contain the &quot;key&quot; that was pressed, plus any user-defined metadata. For example, pressing the Escape key looks like this: %{ &quot;key&quot; =&gt; &quot;Escape&quot; } To capture additional user-defined metadata, the metadata option for keydown events may be provided to the LiveSocket constructor. For example: let liveSocket = new LiveSocket ( &quot;/live&quot; , Socket , { params : { _csrf_token : csrfToken } , metadata : { keydown : ( e , el ) =&gt; { return { key : e . key , metaKey : e . metaKey , repeat : e . repeat } } } } ) To determine which key has been pressed you should use key value. The available options can be found on MDN or via the Key Event Viewer . By default, the bound element will be the event listener, but a window-level binding may be provided via phx-window-keydown or phx-window-keyup , for example: def render ( assigns ) do ~H&quot;&quot;&quot; &lt;div id=&quot;thermostat&quot; phx-window-keyup=&quot;update_temp&quot;&gt; Current temperature: &lt;%= @temperature %&gt; &lt;/div&gt; &quot;&quot;&quot; end def handle_event ( &quot;update_temp&quot; , %{ &quot;key&quot; =&gt; &quot;ArrowUp&quot; } , socket ) do { :ok , new_temp } = Thermostat . inc_temperature ( socket . assigns . id ) { :noreply , assign ( socket , :temperature , new_temp ) } end def handle_event ( &quot;update_temp&quot; , %{ &quot;key&quot; =&gt; &quot;ArrowDown&quot; } , socket ) do { :ok , new_temp } = Thermostat . dec_temperature ( socket . assigns . id ) { :noreply , assign ( socket , :temperature , new_temp ) } end def handle_event ( &quot;update_temp&quot; , _key , socket ) do { :noreply , socket } end","ref":"bindings.html#key-events","title":"Bindings - Key Events","type":"extras"},{"doc":"All events can be rate-limited on the client by using the phx-debounce and phx-throttle bindings, with the following behavior: phx-debounce - Accepts either an integer timeout value (in milliseconds), or &quot;blur&quot; . When an integer is provided, emitting the event is delayed by the specified milliseconds. When &quot;blur&quot; is provided, emitting the event is delayed until the field is blurred by the user. Debouncing is typically used for input elements. phx-throttle - Accepts an integer timeout value to throttle the event in milliseconds. Unlike debounce, throttle will immediately emit the event, then rate limit it at once per provided timeout. Throttling is typically used to rate limit clicks, mouse and keyboard actions. For example, to avoid validating an email until the field is blurred, while validating the username at most every 2 seconds after a user changes the field: &lt; form phx - change = &quot;validate&quot; phx - submit = &quot;save&quot; &gt; &lt; input type = &quot;text&quot; name = &quot;user[email]&quot; phx - debounce = &quot;blur&quot; / &gt; &lt; input type = &quot;text&quot; name = &quot;user[username]&quot; phx - debounce = &quot;2000&quot; / &gt; &lt; / form &gt; And to rate limit a volume up click to once every second: &lt; button phx - click = &quot;volume_up&quot; phx - throttle = &quot;1000&quot; &gt; + &lt; / button &gt; Likewise, you may throttle held-down keydown: &lt; div phx - window - keydown = &quot;keydown&quot; phx - throttle = &quot;500&quot; &gt; ... &lt; / div &gt; Unless held-down keys are required, a better approach is generally to use phx-keyup bindings which only trigger on key up, thereby being self-limiting. However, phx-keydown is useful for games and other use cases where a constant press on a key is desired. In such cases, throttle should always be used. Debounce and Throttle special behavior The following specialized behavior is performed for forms and keydown bindings: When a phx-submit , or a phx-change for a different input is triggered, any current debounce or throttle timers are reset for existing inputs. A phx-keydown binding is only throttled for key repeats. Unique keypresses back-to-back will dispatch the pressed key events.","ref":"bindings.html#rate-limiting-events-with-debounce-and-throttle","title":"Bindings - Rate limiting events with Debounce and Throttle","type":"extras"},{"doc":"The lv: event prefix supports LiveView specific features that are handled by LiveView without calling the user's handle_event/3 callbacks. Today, the following events are supported: lv:clear-flash – clears the flash when sent to the server. If a phx-value-key is provided, the specific key will be removed from the flash. For example: &lt; p class = &quot;alert&quot; phx - click = &quot;lv:clear-flash&quot; phx - value - key = &quot;info&quot; &gt; &lt; % = live_flash ( @flash , :info ) % &gt; &lt; / p &gt;","ref":"bindings.html#liveview-specific-events","title":"Bindings - LiveView Specific Events","type":"extras"},{"doc":"Form bindings","ref":"form-bindings.html","title":"Form bindings","type":"extras"},{"doc":"LiveView works with the existing Phoenix.HTML form helpers. If you want to use helpers such as text_input/2 , etc. be sure to use Phoenix.HTML at the top of your LiveView. If your application was generated with Phoenix v1.6, then mix phx.new automatically uses Phoenix.HTML when you use MyAppWeb, :live_view or use MyAppWeb, :live_component in your modules. Using the generated :live_view and :live_component helpers will also import MyAppWeb.ErrorHelpers , a generated module where error_tag/2 resides (usually located at lib/my_app_web/views/error_helpers.ex ). Since ErrorHelpers is generated into your app, it is yours to modify – you may add additional helper functions here, such as those recommended when rendering feedback for upload_errors/1,2 .","ref":"form-bindings.html#a-note-about-form-helpers","title":"Form bindings - A note about form helpers","type":"extras"},{"doc":"To handle form changes and submissions, use the phx-change and phx-submit events. In general, it is preferred to handle input changes at the form level, where all form fields are passed to the LiveView's callback given any single input change. For example, to handle real-time form validation and saving, your template would use both phx_change and phx_submit bindings: &lt; . form let = { f } for = { @changeset } phx - change = &quot;validate&quot; phx - submit = &quot;save&quot; &gt; &lt; % = label f , :username % &gt; &lt; % = text_input f , :username % &gt; &lt; % = error_tag f , :username % &gt; &lt; % = label f , :email % &gt; &lt; % = text_input f , :email % &gt; &lt; % = error_tag f , :email % &gt; &lt; % = submit &quot;Save&quot; % &gt; &lt; / . form &gt; Next, your LiveView picks up the events in handle_event callbacks: def render ( assigns ) ... def mount ( _params , _session , socket ) do { :ok , assign ( socket , %{ changeset : Accounts . change_user ( % User { } ) } ) } end def handle_event ( &quot;validate&quot; , %{ &quot;user&quot; =&gt; params } , socket ) do changeset = % User { } |&gt; Accounts . change_user ( params ) |&gt; Map . put ( :action , :insert ) { :noreply , assign ( socket , changeset : changeset ) } end def handle_event ( &quot;save&quot; , %{ &quot;user&quot; =&gt; user_params } , socket ) do case Accounts . create_user ( user_params ) do { :ok , user } -&gt; { :noreply , socket |&gt; put_flash ( :info , &quot;user created&quot; ) |&gt; redirect ( to : Routes . user_path ( MyAppWeb.Endpoint , MyAppWeb.User.ShowView , user ) ) } { :error , % Ecto.Changeset { } = changeset } -&gt; { :noreply , assign ( socket , changeset : changeset ) } end end The validate callback simply updates the changeset based on all form input values, then assigns the new changeset to the socket. If the changeset changes, such as generating new errors, render/1 is invoked and the form is re-rendered. Likewise for phx-submit bindings, the same callback is invoked and persistence is attempted. On success, a :noreply tuple is returned and the socket is annotated for redirect with Phoenix.LiveView.redirect/2 to the new user page, otherwise the socket assigns are updated with the errored changeset to be re-rendered for the client.","ref":"form-bindings.html#form-events","title":"Form bindings - Form Events","type":"extras"},{"doc":"For proper form error tag updates, the error tag must specify which input it belongs to. This is accomplished with the phx-feedback-for attribute, which specifies the name (or id, for backwards compatibility) of the input it belongs to. Failing to add the phx-feedback-for attribute will result in displaying error messages for form fields that the user has not changed yet (e.g. required fields further down on the page). For example, your MyAppWeb.ErrorHelpers may use this function: def error_tag ( form , field ) do form . errors |&gt; Keyword . get_values ( field ) |&gt; Enum . map ( fn error -&gt; content_tag ( :span , translate_error ( error ) , class : &quot;invalid-feedback&quot; , phx_feedback_for : input_name ( form , field ) ) end ) end Now, any DOM container with the phx-feedback-for attribute will receive a phx-no-feedback class in cases where the form fields has yet to receive user input/focus. The following css rules are generated in new projects to hide the errors: . phx - no - feedback . invalid - feedback , . phx - no - feedback . invalid - feedback { display : none ; }","ref":"form-bindings.html#phx-feedback-for","title":"Form bindings - phx-feedback-for","type":"extras"},{"doc":"Number inputs are a special case in LiveView forms. On programmatic updates, some browsers will clear invalid inputs. So LiveView will not send change events from the client when an input is invalid, instead allowing the browser's native validation UI to drive user interaction. Once the input becomes valid, change and submit events will be sent normally. &lt; input type = &quot;number&quot; &gt; This is known to have a plethora of problems including accessibility, large numbers are converted to exponential notation, and scrolling can accidentally increase or decrease the number. One alternative is the inputmode attribute, which may serve your application's needs and users much better. According to Can I Use? , the following is supported by 86% of the global market (as of Sep 2021): &lt; input type = &quot;text&quot; inputmode = &quot;numeric&quot; pattern = &quot;[0-9]*&quot; &gt;","ref":"form-bindings.html#number-inputs","title":"Form bindings - Number inputs","type":"extras"},{"doc":"Password inputs are also special cased in Phoenix.HTML . For security reasons, password field values are not reused when rendering a password input tag. This requires explicitly setting the :value in your markup, for example: &lt; % = password_input f , :password , value : input_value ( f , :password ) % &gt; &lt; % = password_input f , :password_confirmation , value : input_value ( f , :password_confirmation ) % &gt; &lt; % = error_tag f , :password % &gt; &lt; % = error_tag f , :password_confirmation % &gt;","ref":"form-bindings.html#password-inputs","title":"Form bindings - Password inputs","type":"extras"},{"doc":"LiveView forms support reactive file inputs , including drag and drop support via the phx-drop-target attribute: &lt; div class = &quot;container&quot; phx - drop - target = { @uploads . avatar . ref } &gt; ... &lt; % = live_file_input @uploads . avatar % &gt; &lt; / div &gt; See Phoenix.LiveView.Helpers.live_file_input/2 for more.","ref":"form-bindings.html#file-inputs","title":"Form bindings - File inputs","type":"extras"},{"doc":"The phx-trigger-action attribute can be added to a form to trigger a standard form submit on DOM patch to the URL specified in the form's standard action attribute. This is useful to perform pre-final validation of a LiveView form submit before posting to a controller route for operations that require Plug session mutation. For example, in your LiveView template you can annotate the phx-trigger-action with a boolean assign: &lt; . form let = { f } for = { @changeset } action = { Routes . reset_password_path ( @socket , :create ) } phx - submit = &quot;save&quot; , phx - trigger - action = { @trigger_submit } &gt; Then in your LiveView, you can toggle the assign to trigger the form with the current fields on next render: def handle_event ( &quot;save&quot; , params , socket ) do case validate_change_password ( socket . assigns . user , params ) do { :ok , changeset } -&gt; { :noreply , assign ( socket , changeset : changeset , trigger_submit : true ) } { :error , changeset } -&gt; { :noreply , assign ( socket , changeset : changeset ) } end end Once phx-trigger-action is true, LiveView disconnects and then submits the form.","ref":"form-bindings.html#submitting-the-form-action-over-http","title":"Form bindings - Submitting the form action over HTTP","type":"extras"},{"doc":"By default, all forms marked with phx-change will recover input values automatically after the user has reconnected or the LiveView has remounted after a crash. This is achieved by the client triggering the same phx-change to the server as soon as the mount has been completed. Note: if you want to see form recovery working in development, please make sure to disable live reloading in development by commenting out the LiveReload plug in your endpoint.ex file or by setting code_reloader: false in your config/dev.exs . Otherwise live reloading may cause the current page to be reloaded whenever you restart the server, which will discard all form state. For most use cases, this is all you need and form recovery will happen without consideration. In some cases, where forms are built step-by-step in a stateful fashion, it may require extra recovery handling on the server outside of your existing phx-change callback code. To enable specialized recovery, provide a phx-auto-recover binding on the form to specify a different event to trigger for recovery, which will receive the form params as usual. For example, imagine a LiveView wizard form where the form is stateful and built based on what step the user is on and by prior selections: &lt; form phx - change = &quot;validate_wizard_step&quot; phx - auto - recover = &quot;recover_wizard&quot; &gt; On the server, the &quot;validate_wizard_step&quot; event is only concerned with the current client form data, but the server maintains the entire state of the wizard. To recover in this scenario, you can specify a recovery event, such as &quot;recover_wizard&quot; above, which would wire up to the following server callbacks in your LiveView: def handle_event ( &quot;validate_wizard_step&quot; , params , socket ) do # regular validations for current step { :noreply , socket } end def handle_event ( &quot;recover_wizard&quot; , params , socket ) do # rebuild state based on client input data up to the current step { :noreply , socket } end To forgo automatic form recovery, set phx-auto-recover=&quot;ignore&quot; .","ref":"form-bindings.html#recovery-following-crashes-or-disconnects","title":"Form bindings - Recovery following crashes or disconnects","type":"extras"},{"doc":"The JavaScript client is always the source of truth for current input values. For any given input with focus, LiveView will never overwrite the input's current value, even if it deviates from the server's rendered updates. This works well for updates where major side effects are not expected, such as form validation errors, or additive UX around the user's input values as they fill out a form. For these use cases, the phx-change input does not concern itself with disabling input editing while an event to the server is in flight. When a phx-change event is sent to the server, the input tag and parent form tag receive the phx-change-loading css class, then the payload is pushed to the server with a &quot;_target&quot; param in the root payload containing the keyspace of the input name which triggered the change event. For example, if the following input triggered a change event: &lt; input name = &quot;user[username]&quot; / &gt; The server's handle_event/3 would receive a payload: %{ &quot;_target&quot; =&gt; [ &quot;user&quot; , &quot;username&quot; ] , &quot;user&quot; =&gt; %{ &quot;username&quot; =&gt; &quot;Name&quot; } } The phx-submit event is used for form submissions where major side effects typically happen, such as rendering new containers, calling an external service, or redirecting to a new page. On submission of a form bound with a phx-submit event: The form's inputs are set to readonly Any submit button on the form is disabled The form receives the &quot;phx-submit-loading&quot; class On completion of server processing of the phx-submit event: The submitted form is reactivated and loses the &quot;phx-submit-loading&quot; class The last input with focus is restored (unless another input has received focus) Updates are patched to the DOM as usual To handle latent events, any HTML tag can be annotated with phx-disable-with , which swaps the element's innerText with the provided value during event submission. For example, the following code would change the &quot;Save&quot; button to &quot;Saving...&quot;, and restore it to &quot;Save&quot; on acknowledgment: &lt; button type = &quot;submit&quot; phx - disable - with = &quot;Saving...&quot; &gt; Save &lt; / button &gt; You may also take advantage of LiveView's CSS loading state classes to swap out your form content while the form is submitting. For example, with the following rules in your app.css : . while - submitting { display : none ; } . inputs { display : block ; } . phx - submit - loading { . while - submitting { display : block ; } . inputs { display : none ; } } You can show and hide content with the following markup: &lt; form phx - change = &quot;update&quot; &gt; &lt; div class = &quot;while-submitting&quot; &gt; Please wait while we save our content ... &lt; / div &gt; &lt; div class = &quot;inputs&quot; &gt; &lt; input type = &quot;text&quot; name = &quot;text&quot; value = &quot;&lt;%= @text %&gt;&quot; &gt; &lt; / div &gt; &lt; / form &gt; Additionally, we strongly recommend including a unique HTML &quot;id&quot; attribute on the form. When DOM siblings change, elements without an ID will be replaced rather than moved, which can cause issues such as form fields losing focus.","ref":"form-bindings.html#javascript-client-specifics","title":"Form bindings - JavaScript client specifics","type":"extras"},{"doc":"DOM patching &amp; temporary assigns A container can be marked with phx-update , allowing the DOM patch operations to avoid updating or removing portions of the LiveView, or to append or prepend the updates rather than replacing the existing contents. This is useful for client-side interop with existing libraries that do their own DOM operations. The following phx-update values are supported: replace - the default operation. Replaces the element with the contents ignore - ignores updates to the DOM regardless of new content changes append - append the new DOM contents instead of replacing prepend - prepend the new DOM contents instead of replacing When using phx-update , a unique DOM ID must always be set in the container. If using &quot;append&quot; or &quot;prepend&quot;, a DOM ID must also be set for each child. When appending or prepending elements containing an ID already present in the container, LiveView will replace the existing element with the new content instead appending or prepending a new element. The &quot;ignore&quot; behaviour is frequently used when you need to integrate with another JS library. Note only the element contents are ignored, its attributes can still be updated. The &quot;append&quot; and &quot;prepend&quot; feature is often used with &quot;Temporary assigns&quot; to work with large amounts of data. Let's learn more.","ref":"dom-patching.html","title":"DOM patching &amp; temporary assigns","type":"extras"},{"doc":"By default, all LiveView assigns are stateful, which enables change tracking and stateful interactions. In some cases, it's useful to mark assigns as temporary, meaning they will be reset to a default value after each update. This allows otherwise large but infrequently updated values to be discarded after the client has been patched. Imagine you want to implement a chat application with LiveView. You could render each message like this: &lt; % = for message &lt;- @messages do % &gt; &lt; p &gt; &lt; span &gt; &lt; % = message . username % &gt; :&lt; / span &gt; &lt; % = message . text % &gt; &lt; / p &gt; &lt; % end % &gt; Every time there is a new message, you would append it to the @messages assign and re-render all messages. As you may suspect, keeping the whole chat conversation in memory and resending it on every update would be too expensive, even with LiveView smart change tracking. By using temporary assigns and phx-update, we don't need to keep any messages in memory, and send messages to be appended to the UI only when there are new ones. To do so, the first step is to mark which assigns are temporary and what values they should be reset to on mount: def mount ( _params , _session , socket ) do socket = assign ( socket , :messages , load_last_20_messages ( ) ) { :ok , socket , temporary_assigns : [ messages : [ ] ] } end On mount we also load the initial number of messages we want to send. After the initial render, the initial batch of messages will be reset back to an empty list. Now, whenever there are one or more new messages, we will assign only the new messages to @messages : socket = assign ( socket , :messages , new_messages ) In the template, we want to wrap all of the messages in a container and tag this content with phx-update . Remember, we must add an ID to the container as well as to each child: &lt; div id = &quot;chat-messages&quot; phx - update = &quot;append&quot; &gt; &lt; % = for message &lt;- @messages do % &gt; &lt; p id = { message . id } &gt; &lt; span &gt; &lt; % = message . username % &gt; :&lt; / span &gt; &lt; % = message . text % &gt; &lt; / p &gt; &lt; % end % &gt; &lt; / div &gt; When the client receives new messages, it now knows to append to the old content rather than replace it. You can also update the direction of messages. Suppose there is an edit to a message that is being sent to your LiveView like this: def handle_info ( { :update_message , message } , socket ) do { :noreply , update ( socket , :messages , fn messages -&gt; [ message | messages ] end ) } end You can add it to the list like you do with new messages. LiveView is aware that this message was rendered on the client, even though the message itself is discarded on the server after it is rendered. LiveView uses DOM ids to check if a message is rendered before or not. If an id is rendered before, the DOM element is updated rather than appending or prepending a new node. Also, the order of elements is not changed. You can use it to show edited messages, show likes, or anything that would require an update to a rendered message.","ref":"dom-patching.html#temporary-assigns","title":"DOM patching &amp; temporary assigns - Temporary assigns","type":"extras"},{"doc":"Temporary assigns are useful when you want to render some data and then discard it so LiveView no longer needs to keep it in memory. For this reason, a temporary assign is not re-rendered until it is set again. This means that temporary assigns should not be used to reset or control UI state. Let's see an example. Imagine you want to show an error message when the input is less than 3 chars. You can write this code: def render ( assigns ) do ~H&quot;&quot;&quot; &lt;%= if @too_short do %&gt; Input too short... &lt;% end %&gt; Searched for: &lt;%= @search %&gt; &lt;form&gt;&lt;input phx-change=&quot;search&quot; name=&quot;term&quot; /&gt;&lt;/form&gt; &quot;&quot;&quot; end def mount ( _params , _session , socket ) do { :ok , assign ( socket , too_short : false , search : &quot;&quot; ) , temporary_assigns : [ too_short : false ] } end def handle_event ( &quot;search&quot; , %{ &quot;term&quot; =&gt; term } , socket ) do # do not search if user provides less then 3 chars if String . length ( term ) &gt;= 3 do { :noreply , assign ( socket , search : term ) } else { :noreply , assign ( socket , too_short : true , search : term ) } end end The idea here is that, while the term is less than 3 characters, we will set @too_short to true and show an error message in the UI accordingly. We also set @too_short as a temporary assign, so that it resets to false after every render. However, once a temporary assign resets to its original value, it won't be re-rendered, unless we explicitly assign it to something else. This means that the LiveView will never re-render the if block and we will continue to show &quot;Input too short...&quot; even after the input has 3 or more characters. The mistake here is using :temporary_assigns to reset or control UI state, while :temporary_assigns should rather be used when we don't have (or don't want to keep) certain data around. The fix is to set too_short: false on the if branch, making sure it is reset whenever the search input changes.","ref":"dom-patching.html#pitfall-temporary-assigns-to-reset-or-control-ui-state","title":"DOM patching &amp; temporary assigns - Pitfall: temporary assigns to reset or control UI state","type":"extras"},{"doc":"JavaScript interoperability As seen earlier, you start by instantiating a single LiveSocket to enable LiveView client/server interaction, for example: import { Socket } from &quot;phoenix&quot; import { LiveSocket } from &quot;phoenix_live_view&quot; let csrfToken = document . querySelector ( &quot;meta[name=&#39;csrf-token&#39;]&quot; ) . getAttribute ( &quot;content&quot; ) let liveSocket = new LiveSocket ( &quot;/live&quot; , Socket , { params : { _csrf_token : csrfToken } } ) liveSocket . connect ( ) All options are passed directly to the Phoenix.Socket constructor, except for the following LiveView specific options: bindingPrefix - the prefix to use for phoenix bindings. Defaults &quot;phx-&quot; params - the connect_params to pass to the view's mount callback. May be a literal object or closure returning an object. When a closure is provided, the function receives the view's phx-view name. hooks – a reference to a user-defined hooks namespace, containing client callbacks for server/client interop. See the Client hooks section below for details. uploaders – a reference to a user-defined uploaders namespace, containing client callbacks for client-side direct-to-cloud uploads. See the External Uploads guide for details.","ref":"js-interop.html","title":"JavaScript interoperability","type":"extras"},{"doc":"To aid debugging on the client when troubleshooting issues, the enableDebug() and disableDebug() functions are exposed on the LiveSocket JavaScript instance. Calling enableDebug() turns on debug logging which includes LiveView life-cycle and payload events as they come and go from client to server. In practice, you can expose your instance on window for quick access in the browser's web console, for example: / / app . js let liveSocket = new LiveSocket ( ... ) liveSocket . connect ( ) window . liveSocket = liveSocket / / in the browser &#39; s web console &gt;&gt; liveSocket . enableDebug ( ) The debug state uses the browser's built-in sessionStorage , so it will remain in effect for as long as your browser session lasts.","ref":"js-interop.html#debugging-client-events","title":"JavaScript interoperability - Debugging Client Events","type":"extras"},{"doc":"Proper handling of latency is critical for good UX. LiveView's CSS loading states allow the client to provide user feedback while awaiting a server response. In development, near zero latency on localhost does not allow latency to be easily represented or tested, so LiveView includes a latency simulator with the JavaScript client to ensure your application provides a pleasant experience. Like the enableDebug() function above, the LiveSocket instance includes enableLatencySim(milliseconds) and disableLatencySim() functions which apply throughout the current browser session. The enableLatencySim function accepts an integer in milliseconds for the round-trip-time to the server. For example: / / app . js let liveSocket = new LiveSocket ( ... ) liveSocket . connect ( ) window . liveSocket = liveSocket / / in the browser &#39; s web console &gt;&gt; liveSocket . enableLatencySim ( 1000 ) [ Log ] latency simulator enabled for the duration of this browser session . Call disableLatencySim ( ) to disable","ref":"js-interop.html#simulating-latency","title":"JavaScript interoperability - Simulating Latency","type":"extras"},{"doc":"By default, the following classes are applied to the LiveView's parent container: &quot;phx-connected&quot; - applied when the view has connected to the server &quot;phx-disconnected&quot; - applied when the view is not connected to the server &quot;phx-error&quot; - applied when an error occurs on the server. Note, this class will be applied in conjunction with &quot;phx-disconnected&quot; if connection to the server is lost. All phx- event bindings apply their own css classes when pushed. For example the following markup: &lt; button phx - click = &quot;clicked&quot; phx - window - keydown = &quot;key&quot; &gt; ... &lt; / button &gt; On click, would receive the phx-click-loading class, and on keydown would receive the phx-keydown-loading class. The css loading classes are maintained until an acknowledgement is received on the client for the pushed event. In the case of forms, when a phx-change is sent to the server, the input element which emitted the change receives the phx-change-loading class, along with the parent form tag. The following events receive css loading classes: phx-click - phx-click-loading phx-change - phx-change-loading phx-submit - phx-submit-loading phx-focus - phx-focus-loading phx-blur - phx-blur-loading phx-window-keydown - phx-keydown-loading phx-window-keyup - phx-keyup-loading For live page navigation via live_redirect and live_patch , as well as form submits via phx-submit , the JavaScript events &quot;phx:page-loading-start&quot; and &quot;phx:page-loading-stop&quot; are dispatched on window. Additionally, any phx- event may dispatch page loading events by annotating the DOM element with phx-page-loading . This is useful for showing main page loading status, for example: / / app . js import topbar from &quot;topbar&quot; window . addEventListener ( &quot;phx:page-loading-start&quot; , info =&gt; topbar . show ( ) ) window . addEventListener ( &quot;phx:page-loading-stop&quot; , info =&gt; topbar . hide ( ) ) Within the callback, info.detail will be an object that contains a kind key, with a value that depends on the triggering event: &quot;redirect&quot; - the event was triggered by a redirect &quot;patch&quot; - the event was triggered by a patch &quot;initial&quot; - the event was triggered by initial page load &quot;element&quot; - the event was triggered by a phx- bound element, such as phx-click For all kinds of page loading events, all but &quot;element&quot; will receive an additional to key in the info metadata pointing to the href associated with the page load. In the case of an &quot;element&quot; page loading event, the info will contain a &quot;target&quot; key containing the DOM element which triggered the page loading state.","ref":"js-interop.html#loading-state-and-errors","title":"JavaScript interoperability - Loading state and errors","type":"extras"},{"doc":"Often it is desirable to trigger an event on a DOM element without explicit user interaction on the element. For example, a custom form element such as a date picker or custom select input which utilizes a hidden input element to store the selected state. In these cases, the event functions on the DOM API can be used, for example to trigger a phx-change event: document.getElementById(&quot;my-select&quot;).dispatchEvent( new Event(&quot;input&quot;, {bubbles: true}) ) When using a client hook, this.el can be used to determine the element as outlined in the &quot;Client hooks&quot; documentation. It is also possible to trigger a phx-submit using a &quot;submit&quot; event: document.getElementById(&quot;my-form&quot;).dispatchEvent( new Event(&quot;submit&quot;, {bubbles: true}) )","ref":"js-interop.html#triggering-phx-form-events-with-javascript","title":"JavaScript interoperability - Triggering phx form events with JavaScript","type":"extras"},{"doc":"To handle custom client-side JavaScript when an element is added, updated, or removed by the server, a hook object may be provided with the following life-cycle callbacks: mounted - the element has been added to the DOM and its server LiveView has finished mounting beforeUpdate - the element is about to be updated in the DOM. Note : any call here must be synchronous as the operation cannot be deferred or cancelled. updated - the element has been updated in the DOM by the server destroyed - the element has been removed from the page, either by a parent update, or by the parent being removed entirely disconnected - the element's parent LiveView has disconnected from the server reconnected - the element's parent LiveView has reconnected to the server The above life-cycle callbacks have in-scope access to the following attributes: el - attribute referencing the bound DOM node, viewName - attribute matching the DOM node's phx-view value pushEvent(event, payload, (reply, ref) =&gt; ...) - method to push an event from the client to the LiveView server pushEventTo(selectorOrTarget, event, payload, (reply, ref) =&gt; ...) - method to push targeted events from the client to LiveViews and LiveComponents. It sends the event to the LiveComponent or LiveView the selectorOrTarget is defined in, where it's value can be either a query selector or an actual DOM element. If the query selector returns more than one element it will send the event to all of them, even if all the elements are in the same LiveComponent or LiveView. handleEvent(event, (payload) =&gt; ...) - method to handle an event pushed from the server upload(name, files) - method to inject a list of file-like objects into an uploader. uploadTo(selectorOrTarget, name, files) - method to inject a list of file-like objects into an uploader. The hook will send the files to the uploader with name defined by allow_upload/3 on the server-side. Dispatching new uploads triggers an input change event which will be sent to the LiveComponent or LiveView the selectorOrTarget is defined in, where it's value can be either a query selector or an actual DOM element. If the query selector returns more than one live file input, an error will be logged. For example, the markup for a controlled input for phone-number formatting could be written like this: &lt; input type = &quot;text&quot; name = &quot;user[phone_number]&quot; id = &quot;user-phone-number&quot; phx - hook = &quot;PhoneNumber&quot; / &gt; Then a hook callback object could be defined and passed to the socket: let Hooks = { } Hooks.PhoneNumber = { mounted ( ) { this . el . addEventListener ( &quot;input&quot; , e =&gt; { let match = this . el . value . replace ( / \\ D / g , &quot;&quot; ) . match ( / ^ ( \\ d { 3 } ) ( \\ d { 3 } ) ( \\ d { 4 } ) $ / ) if ( match ) { this . el . value = ` $ { match [ 1 ] } - $ { match [ 2 ] } - $ { match [ 3 ] } ` } } ) } } let liveSocket = new LiveSocket ( &quot;/live&quot; , Socket , { hooks : Hooks , ... } ) ... Note : when using phx-hook , a unique DOM ID must always be set. For integration with client-side libraries which require a broader access to full DOM management, the LiveSocket constructor accepts a dom option with an onBeforeElUpdated callback. The fromEl and toEl DOM nodes are passed to the function just before the DOM patch operations occurs in LiveView. This allows external libraries to (re)initialize DOM elements or copy attributes as necessary as LiveView performs its own patch operations. The update operation cannot be cancelled or deferred, and the return value is ignored. For example, the following option could be used to add Alpine.js support to your project: let liveSocket = new LiveSocket ( &quot;/live&quot; , Socket , { ... , dom : { onBeforeElUpdated ( from , to ) { if ( from . _x_dataStack ) { window . Alpine . clone ( from , to ) } } } , } ) Client-server communication A hook can push events to the LiveView by using the pushEvent function and receive a reply from the server via a {:reply, map, socket} return value. The reply payload will be passed to the optional pushEvent response callback. Communication with the hook from the server can be done by reading data attributes on the hook element or by using Phoenix.LiveView.push_event/3 on the server and handleEvent on the client. For example, to implement infinite scrolling, one can pass the current page using data attributes: &lt; div id = &quot;infinite-scroll&quot; phx - hook = &quot;InfiniteScroll&quot; data - page = &quot;&lt;%= @page %&gt;&quot; &gt; And then in the client: Hooks.InfiniteScroll = { page ( ) { return this . el . dataset . page } , mounted ( ) { this . pending = this . page ( ) window . addEventListener ( &quot;scroll&quot; , e =&gt; { if ( this . pending == this . page ( ) &amp;&amp; scrollAt ( ) &gt; 90 ) { this . pending = this . page ( ) + 1 this . pushEvent ( &quot;load-more&quot; , { } ) } } ) } , updated ( ) { this . pending = this . page ( ) } } However, the data attribute approach is not a good approach if you need to frequently push data to the client. To push out-of-band events to the client, for example to render charting points, one could do: &lt; div id = &quot;chart&quot; phx - hook = &quot;Chart&quot; &gt; { :noreply , push_event ( socket , &quot;points&quot; , %{ points : new_points } ) } And then on the client: Hooks.Chart = { mounted ( ) { this . handleEvent ( &quot;points&quot; , ( { points } ) =&gt; MyChartLib . addPoints ( points ) ) } } Note : events pushed from the server via push_event are global and will be dispatched to all active hooks on the client who are handling that event. Note : In case a LiveView pushes events and renders content, handleEvent callbacks are invoked after the page is updated. Therefore, if the LiveView redirects at the same time it pushes events, callbacks won't be invoked on the old page's elements. Callbacks would be invoked on the redirected page's newly mounted hook elements.","ref":"js-interop.html#client-hooks","title":"JavaScript interoperability - Client hooks","type":"extras"},{"doc":"External Uploads This guide continues from the configuration started in the server Uploads guide . Uploads to external cloud providers, such as Amazon S3, Google Cloud, etc., can be achieved by using the :external option in allow_upload/3 . You provide a 2-arity function to allow the server to generate metadata for each upload entry, which is passed to a user-specified JavaScript function on the client. Typically when your function is invoked, you will generate a pre-signed URL, specific to your cloud storage provider, that will provide temporary access for the end-user to upload data directly to your cloud storage.","ref":"uploads-external.html","title":"External Uploads","type":"extras"},{"doc":"For any service that supports large file uploads via chunked HTTP requests with Content-Range headers, you can use the UpChunk JS library by Mux to do all the hard work of uploading the file. You only need to wire the UpChunk instance to the LiveView UploadEntry callbacks, and LiveView will take care of the rest. Install UpChunk by saving its contents to assets/vendor/upchunk.js or by installing it with npm : $ npm install --prefix assets --save @mux/upchunk Configure your uploader on Phoenix.LiveView.mount/3 : def mount ( _params , _session , socket ) do { :ok , socket |&gt; assign ( :uploaded_files , [ ] ) |&gt; allow_upload ( :avatar , accept : :any , max_entries : 3 , external : &amp; presign_upload / 2 ) } end Supply the :external option to Phoenix.LiveView.allow_upload/3 . It requires a 2-arity function that generates a signed URL where the client will push the bytes for the upload entry. For example, if you were using a context that provided a start_session function, you might write something like this: defp presign_upload ( entry , socket ) do { :ok , %{ &quot;Location&quot; =&gt; link } } = SomeTube . start_session ( %{ &quot;uploadType&quot; =&gt; &quot;resumable&quot; , &quot;x-upload-content-length&quot; =&gt; entry . client_size } ) { :ok , %{ uploader : &quot;UpChunk&quot; , entrypoint : link } , socket } end Finally, on the client-side, we use UpChunk to create an upload from the temporary URL generated on the server and attach listeners for its events to the entry's callbacks: import * as UpChunk from &quot;@mux/upchunk&quot; let Uploaders = {} Uploaders.UpChunk = function(entries, onViewError){ entries.forEach(entry =&gt; { // create the upload session with UpChunk let { file, meta: { entrypoint } } = entry let upload = UpChunk.createUpload({ entrypoint, file }) // stop uploading in the event of a view error onViewError(() =&gt; upload.pause()) // upload error triggers LiveView error upload.on(&quot;error&quot;, (e) =&gt; entry.error(e.detail.message)) // notify progress events to LiveView upload.on(&quot;progress&quot;, (e) =&gt; { if(e.detail &lt; 100){ entry.progress(e.detail) } }) // success completes the UploadEntry upload.on(&quot;success&quot;, () =&gt; entry.progress(100)) }) } // Don't forget to assign Uploaders to the liveSocket let liveSocket = new LiveSocket(&quot;/live&quot;, Socket, { uploaders: Uploaders, params: {_csrf_token: csrfToken} })","ref":"uploads-external.html#chunked-http-uploads","title":"External Uploads - Chunked HTTP Uploads","type":"extras"},{"doc":"In order to enforce all of your file constraints when uploading to S3, it is necessary to perform a multipart form POST with your file data. The following example uses a zero-dependency module called SimpleS3Upload written by Chris McCord to generate pre-signed URLs for S3. def mount ( _params , _session , socket ) do { :ok , socket |&gt; assign ( :uploaded_files , [ ] ) |&gt; allow_upload ( :avatar , accept : :any , max_entries : 3 , external : &amp; presign_upload / 2 ) } end defp presign_upload ( entry , socket ) do uploads = socket . assigns . uploads bucket = &quot;phx-upload-example&quot; key = &quot;public/ \#{ entry . client_name } &quot; config = %{ region : &quot;us-east-1&quot; , access_key_id : System . fetch_env! ( &quot;AWS_ACCESS_KEY_ID&quot; ) , secret_access_key : System . fetch_env! ( &quot;AWS_SECRET_ACCESS_KEY&quot; ) } { :ok , fields } = SimpleS3Upload . sign_form_upload ( config , bucket , key : key , content_type : entry . client_type , max_file_size : uploads . avatar . max_file_size , expires_in : :timer . hours ( 1 ) ) meta = %{ uploader : &quot;S3&quot; , key : key , url : &quot;http:// \#{ bucket } .s3- \#{ config . region } .amazonaws.com&quot; , fields : fields } { :ok , meta , socket } end Here, we implemented a presign_upload/2 function, which we passed as a captured anonymous function to :external . Next, we generate a pre-signed URL for the upload. Lastly, we return our :ok result, with a payload of metadata for the client, along with our unchanged socket. The metadata must contain the :uploader key, specifying the name of the JavaScript client-side uploader, in this case &quot;S3&quot; . To complete the flow, we can implement our S3 client uploader and tell the LiveSocket where to find it: let Uploaders = {} Uploaders.S3 = function(entries, onViewError){ entries.forEach(entry =&gt; { let formData = new FormData() let {url, fields} = entry.meta Object.entries(fields).forEach(([key, val]) =&gt; formData.append(key, val)) formData.append(&quot;file&quot;, entry.file) let xhr = new XMLHttpRequest() onViewError(() =&gt; xhr.abort()) xhr.onload = () =&gt; xhr.status === 204 ? entry.progress(100) : entry.error() xhr.onerror = () =&gt; entry.error() xhr.upload.addEventListener(&quot;progress&quot;, (event) =&gt; { if(event.lengthComputable){ let percent = Math.round((event.loaded / event.total) * 100) if(percent &lt; 100){ entry.progress(percent) } } }) xhr.open(&quot;POST&quot;, url, true) xhr.send(formData) }) } let liveSocket = new LiveSocket(&quot;/live&quot;, Socket, { uploaders: Uploaders, params: {_csrf_token: csrfToken} }) We define an Uploaders.S3 function, which receives our entries. It then performs an AJAX request for each entry, using the entry.progress() and entry.error() . functions to report upload events back to the LiveView. Lastly, we pass the uploaders namespace to the LiveSocket constructor to tell phoenix where to find the uploaders returned within the external metadata.","ref":"uploads-external.html#direct-to-s3","title":"External Uploads - Direct to S3","type":"extras"}]